// The MIT License (MIT)

// Copyright (c) 2017-2020 Uber Technologies Inc.

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

// Code generated by protoc-gen-yarpc-go. DO NOT EDIT.
// source: uber/cadence/matching/v1/service.proto

package matchingv1

import (
	"context"
	"io/ioutil"
	"reflect"

	"github.com/gogo/protobuf/jsonpb"
	"github.com/gogo/protobuf/proto"
	"go.uber.org/fx"
	"go.uber.org/yarpc"
	"go.uber.org/yarpc/api/transport"
	"go.uber.org/yarpc/api/x/restriction"
	"go.uber.org/yarpc/encoding/protobuf"
	"go.uber.org/yarpc/encoding/protobuf/reflection"
)

var _ = ioutil.NopCloser

// MatchingAPIYARPCClient is the YARPC client-side interface for the MatchingAPI service.
type MatchingAPIYARPCClient interface {
	PollForDecisionTask(context.Context, *PollForDecisionTaskRequest, ...yarpc.CallOption) (*PollForDecisionTaskResponse, error)
	PollForActivityTask(context.Context, *PollForActivityTaskRequest, ...yarpc.CallOption) (*PollForActivityTaskResponse, error)
	AddDecisionTask(context.Context, *AddDecisionTaskRequest, ...yarpc.CallOption) (*AddDecisionTaskResponse, error)
	AddActivityTask(context.Context, *AddActivityTaskRequest, ...yarpc.CallOption) (*AddActivityTaskResponse, error)
	QueryWorkflow(context.Context, *QueryWorkflowRequest, ...yarpc.CallOption) (*QueryWorkflowResponse, error)
	RespondQueryTaskCompleted(context.Context, *RespondQueryTaskCompletedRequest, ...yarpc.CallOption) (*RespondQueryTaskCompletedResponse, error)
	CancelOutstandingPoll(context.Context, *CancelOutstandingPollRequest, ...yarpc.CallOption) (*CancelOutstandingPollResponse, error)
	DescribeTaskList(context.Context, *DescribeTaskListRequest, ...yarpc.CallOption) (*DescribeTaskListResponse, error)
	ListTaskListPartitions(context.Context, *ListTaskListPartitionsRequest, ...yarpc.CallOption) (*ListTaskListPartitionsResponse, error)
}

func newMatchingAPIYARPCClient(clientConfig transport.ClientConfig, anyResolver jsonpb.AnyResolver, options ...protobuf.ClientOption) MatchingAPIYARPCClient {
	return &_MatchingAPIYARPCCaller{protobuf.NewStreamClient(
		protobuf.ClientParams{
			ServiceName:  "uber.cadence.matching.v1.MatchingAPI",
			ClientConfig: clientConfig,
			AnyResolver:  anyResolver,
			Options:      options,
		},
	)}
}

// NewMatchingAPIYARPCClient builds a new YARPC client for the MatchingAPI service.
func NewMatchingAPIYARPCClient(clientConfig transport.ClientConfig, options ...protobuf.ClientOption) MatchingAPIYARPCClient {
	return newMatchingAPIYARPCClient(clientConfig, nil, options...)
}

// MatchingAPIYARPCServer is the YARPC server-side interface for the MatchingAPI service.
type MatchingAPIYARPCServer interface {
	PollForDecisionTask(context.Context, *PollForDecisionTaskRequest) (*PollForDecisionTaskResponse, error)
	PollForActivityTask(context.Context, *PollForActivityTaskRequest) (*PollForActivityTaskResponse, error)
	AddDecisionTask(context.Context, *AddDecisionTaskRequest) (*AddDecisionTaskResponse, error)
	AddActivityTask(context.Context, *AddActivityTaskRequest) (*AddActivityTaskResponse, error)
	QueryWorkflow(context.Context, *QueryWorkflowRequest) (*QueryWorkflowResponse, error)
	RespondQueryTaskCompleted(context.Context, *RespondQueryTaskCompletedRequest) (*RespondQueryTaskCompletedResponse, error)
	CancelOutstandingPoll(context.Context, *CancelOutstandingPollRequest) (*CancelOutstandingPollResponse, error)
	DescribeTaskList(context.Context, *DescribeTaskListRequest) (*DescribeTaskListResponse, error)
	ListTaskListPartitions(context.Context, *ListTaskListPartitionsRequest) (*ListTaskListPartitionsResponse, error)
}

type buildMatchingAPIYARPCProceduresParams struct {
	Server      MatchingAPIYARPCServer
	AnyResolver jsonpb.AnyResolver
}

func buildMatchingAPIYARPCProcedures(params buildMatchingAPIYARPCProceduresParams) []transport.Procedure {
	handler := &_MatchingAPIYARPCHandler{params.Server}
	return protobuf.BuildProcedures(
		protobuf.BuildProceduresParams{
			ServiceName: "uber.cadence.matching.v1.MatchingAPI",
			UnaryHandlerParams: []protobuf.BuildProceduresUnaryHandlerParams{
				{
					MethodName: "PollForDecisionTask",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.PollForDecisionTask,
							NewRequest:  newMatchingAPIServicePollForDecisionTaskYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
				{
					MethodName: "PollForActivityTask",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.PollForActivityTask,
							NewRequest:  newMatchingAPIServicePollForActivityTaskYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
				{
					MethodName: "AddDecisionTask",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.AddDecisionTask,
							NewRequest:  newMatchingAPIServiceAddDecisionTaskYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
				{
					MethodName: "AddActivityTask",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.AddActivityTask,
							NewRequest:  newMatchingAPIServiceAddActivityTaskYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
				{
					MethodName: "QueryWorkflow",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.QueryWorkflow,
							NewRequest:  newMatchingAPIServiceQueryWorkflowYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
				{
					MethodName: "RespondQueryTaskCompleted",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.RespondQueryTaskCompleted,
							NewRequest:  newMatchingAPIServiceRespondQueryTaskCompletedYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
				{
					MethodName: "CancelOutstandingPoll",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.CancelOutstandingPoll,
							NewRequest:  newMatchingAPIServiceCancelOutstandingPollYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
				{
					MethodName: "DescribeTaskList",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.DescribeTaskList,
							NewRequest:  newMatchingAPIServiceDescribeTaskListYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
				{
					MethodName: "ListTaskListPartitions",
					Handler: protobuf.NewUnaryHandler(
						protobuf.UnaryHandlerParams{
							Handle:      handler.ListTaskListPartitions,
							NewRequest:  newMatchingAPIServiceListTaskListPartitionsYARPCRequest,
							AnyResolver: params.AnyResolver,
						},
					),
				},
			},
			OnewayHandlerParams: []protobuf.BuildProceduresOnewayHandlerParams{},
			StreamHandlerParams: []protobuf.BuildProceduresStreamHandlerParams{},
		},
	)
}

// BuildMatchingAPIYARPCProcedures prepares an implementation of the MatchingAPI service for YARPC registration.
func BuildMatchingAPIYARPCProcedures(server MatchingAPIYARPCServer) []transport.Procedure {
	return buildMatchingAPIYARPCProcedures(buildMatchingAPIYARPCProceduresParams{Server: server})
}

// FxMatchingAPIYARPCClientParams defines the input
// for NewFxMatchingAPIYARPCClient. It provides the
// paramaters to get a MatchingAPIYARPCClient in an
// Fx application.
type FxMatchingAPIYARPCClientParams struct {
	fx.In

	Provider    yarpc.ClientConfig
	AnyResolver jsonpb.AnyResolver  `name:"yarpcfx" optional:"true"`
	Restriction restriction.Checker `optional:"true"`
}

// FxMatchingAPIYARPCClientResult defines the output
// of NewFxMatchingAPIYARPCClient. It provides a
// MatchingAPIYARPCClient to an Fx application.
type FxMatchingAPIYARPCClientResult struct {
	fx.Out

	Client MatchingAPIYARPCClient

	// We are using an fx.Out struct here instead of just returning a client
	// so that we can add more values or add named versions of the client in
	// the future without breaking any existing code.
}

// NewFxMatchingAPIYARPCClient provides a MatchingAPIYARPCClient
// to an Fx application using the given name for routing.
//
//  fx.Provide(
//    matchingv1.NewFxMatchingAPIYARPCClient("service-name"),
//    ...
//  )
func NewFxMatchingAPIYARPCClient(name string, options ...protobuf.ClientOption) interface{} {
	return func(params FxMatchingAPIYARPCClientParams) FxMatchingAPIYARPCClientResult {
		cc := params.Provider.ClientConfig(name)

		if params.Restriction != nil {
			if namer, ok := cc.GetUnaryOutbound().(transport.Namer); ok {
				if err := params.Restriction.Check(protobuf.Encoding, namer.TransportName()); err != nil {
					panic(err.Error())
				}
			}
		}

		return FxMatchingAPIYARPCClientResult{
			Client: newMatchingAPIYARPCClient(cc, params.AnyResolver, options...),
		}
	}
}

// FxMatchingAPIYARPCProceduresParams defines the input
// for NewFxMatchingAPIYARPCProcedures. It provides the
// paramaters to get MatchingAPIYARPCServer procedures in an
// Fx application.
type FxMatchingAPIYARPCProceduresParams struct {
	fx.In

	Server      MatchingAPIYARPCServer
	AnyResolver jsonpb.AnyResolver `name:"yarpcfx" optional:"true"`
}

// FxMatchingAPIYARPCProceduresResult defines the output
// of NewFxMatchingAPIYARPCProcedures. It provides
// MatchingAPIYARPCServer procedures to an Fx application.
//
// The procedures are provided to the "yarpcfx" value group.
// Dig 1.2 or newer must be used for this feature to work.
type FxMatchingAPIYARPCProceduresResult struct {
	fx.Out

	Procedures     []transport.Procedure `group:"yarpcfx"`
	ReflectionMeta reflection.ServerMeta `group:"yarpcfx"`
}

// NewFxMatchingAPIYARPCProcedures provides MatchingAPIYARPCServer procedures to an Fx application.
// It expects a MatchingAPIYARPCServer to be present in the container.
//
//  fx.Provide(
//    matchingv1.NewFxMatchingAPIYARPCProcedures(),
//    ...
//  )
func NewFxMatchingAPIYARPCProcedures() interface{} {
	return func(params FxMatchingAPIYARPCProceduresParams) FxMatchingAPIYARPCProceduresResult {
		return FxMatchingAPIYARPCProceduresResult{
			Procedures: buildMatchingAPIYARPCProcedures(buildMatchingAPIYARPCProceduresParams{
				Server:      params.Server,
				AnyResolver: params.AnyResolver,
			}),
			ReflectionMeta: reflection.ServerMeta{
				ServiceName:     "uber.cadence.matching.v1.MatchingAPI",
				FileDescriptors: yarpcFileDescriptorClosure826e827d3aabf7fc,
			},
		}
	}
}

type _MatchingAPIYARPCCaller struct {
	streamClient protobuf.StreamClient
}

func (c *_MatchingAPIYARPCCaller) PollForDecisionTask(ctx context.Context, request *PollForDecisionTaskRequest, options ...yarpc.CallOption) (*PollForDecisionTaskResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "PollForDecisionTask", request, newMatchingAPIServicePollForDecisionTaskYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*PollForDecisionTaskResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServicePollForDecisionTaskYARPCResponse, responseMessage)
	}
	return response, err
}

func (c *_MatchingAPIYARPCCaller) PollForActivityTask(ctx context.Context, request *PollForActivityTaskRequest, options ...yarpc.CallOption) (*PollForActivityTaskResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "PollForActivityTask", request, newMatchingAPIServicePollForActivityTaskYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*PollForActivityTaskResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServicePollForActivityTaskYARPCResponse, responseMessage)
	}
	return response, err
}

func (c *_MatchingAPIYARPCCaller) AddDecisionTask(ctx context.Context, request *AddDecisionTaskRequest, options ...yarpc.CallOption) (*AddDecisionTaskResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "AddDecisionTask", request, newMatchingAPIServiceAddDecisionTaskYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*AddDecisionTaskResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServiceAddDecisionTaskYARPCResponse, responseMessage)
	}
	return response, err
}

func (c *_MatchingAPIYARPCCaller) AddActivityTask(ctx context.Context, request *AddActivityTaskRequest, options ...yarpc.CallOption) (*AddActivityTaskResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "AddActivityTask", request, newMatchingAPIServiceAddActivityTaskYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*AddActivityTaskResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServiceAddActivityTaskYARPCResponse, responseMessage)
	}
	return response, err
}

func (c *_MatchingAPIYARPCCaller) QueryWorkflow(ctx context.Context, request *QueryWorkflowRequest, options ...yarpc.CallOption) (*QueryWorkflowResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "QueryWorkflow", request, newMatchingAPIServiceQueryWorkflowYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*QueryWorkflowResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServiceQueryWorkflowYARPCResponse, responseMessage)
	}
	return response, err
}

func (c *_MatchingAPIYARPCCaller) RespondQueryTaskCompleted(ctx context.Context, request *RespondQueryTaskCompletedRequest, options ...yarpc.CallOption) (*RespondQueryTaskCompletedResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "RespondQueryTaskCompleted", request, newMatchingAPIServiceRespondQueryTaskCompletedYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*RespondQueryTaskCompletedResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServiceRespondQueryTaskCompletedYARPCResponse, responseMessage)
	}
	return response, err
}

func (c *_MatchingAPIYARPCCaller) CancelOutstandingPoll(ctx context.Context, request *CancelOutstandingPollRequest, options ...yarpc.CallOption) (*CancelOutstandingPollResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "CancelOutstandingPoll", request, newMatchingAPIServiceCancelOutstandingPollYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*CancelOutstandingPollResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServiceCancelOutstandingPollYARPCResponse, responseMessage)
	}
	return response, err
}

func (c *_MatchingAPIYARPCCaller) DescribeTaskList(ctx context.Context, request *DescribeTaskListRequest, options ...yarpc.CallOption) (*DescribeTaskListResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "DescribeTaskList", request, newMatchingAPIServiceDescribeTaskListYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*DescribeTaskListResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServiceDescribeTaskListYARPCResponse, responseMessage)
	}
	return response, err
}

func (c *_MatchingAPIYARPCCaller) ListTaskListPartitions(ctx context.Context, request *ListTaskListPartitionsRequest, options ...yarpc.CallOption) (*ListTaskListPartitionsResponse, error) {
	responseMessage, err := c.streamClient.Call(ctx, "ListTaskListPartitions", request, newMatchingAPIServiceListTaskListPartitionsYARPCResponse, options...)
	if responseMessage == nil {
		return nil, err
	}
	response, ok := responseMessage.(*ListTaskListPartitionsResponse)
	if !ok {
		return nil, protobuf.CastError(emptyMatchingAPIServiceListTaskListPartitionsYARPCResponse, responseMessage)
	}
	return response, err
}

type _MatchingAPIYARPCHandler struct {
	server MatchingAPIYARPCServer
}

func (h *_MatchingAPIYARPCHandler) PollForDecisionTask(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *PollForDecisionTaskRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*PollForDecisionTaskRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServicePollForDecisionTaskYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.PollForDecisionTask(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func (h *_MatchingAPIYARPCHandler) PollForActivityTask(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *PollForActivityTaskRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*PollForActivityTaskRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServicePollForActivityTaskYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.PollForActivityTask(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func (h *_MatchingAPIYARPCHandler) AddDecisionTask(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *AddDecisionTaskRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*AddDecisionTaskRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServiceAddDecisionTaskYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.AddDecisionTask(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func (h *_MatchingAPIYARPCHandler) AddActivityTask(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *AddActivityTaskRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*AddActivityTaskRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServiceAddActivityTaskYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.AddActivityTask(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func (h *_MatchingAPIYARPCHandler) QueryWorkflow(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *QueryWorkflowRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*QueryWorkflowRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServiceQueryWorkflowYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.QueryWorkflow(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func (h *_MatchingAPIYARPCHandler) RespondQueryTaskCompleted(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *RespondQueryTaskCompletedRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*RespondQueryTaskCompletedRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServiceRespondQueryTaskCompletedYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.RespondQueryTaskCompleted(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func (h *_MatchingAPIYARPCHandler) CancelOutstandingPoll(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *CancelOutstandingPollRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*CancelOutstandingPollRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServiceCancelOutstandingPollYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.CancelOutstandingPoll(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func (h *_MatchingAPIYARPCHandler) DescribeTaskList(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *DescribeTaskListRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*DescribeTaskListRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServiceDescribeTaskListYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.DescribeTaskList(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func (h *_MatchingAPIYARPCHandler) ListTaskListPartitions(ctx context.Context, requestMessage proto.Message) (proto.Message, error) {
	var request *ListTaskListPartitionsRequest
	var ok bool
	if requestMessage != nil {
		request, ok = requestMessage.(*ListTaskListPartitionsRequest)
		if !ok {
			return nil, protobuf.CastError(emptyMatchingAPIServiceListTaskListPartitionsYARPCRequest, requestMessage)
		}
	}
	response, err := h.server.ListTaskListPartitions(ctx, request)
	if response == nil {
		return nil, err
	}
	return response, err
}

func newMatchingAPIServicePollForDecisionTaskYARPCRequest() proto.Message {
	return &PollForDecisionTaskRequest{}
}

func newMatchingAPIServicePollForDecisionTaskYARPCResponse() proto.Message {
	return &PollForDecisionTaskResponse{}
}

func newMatchingAPIServicePollForActivityTaskYARPCRequest() proto.Message {
	return &PollForActivityTaskRequest{}
}

func newMatchingAPIServicePollForActivityTaskYARPCResponse() proto.Message {
	return &PollForActivityTaskResponse{}
}

func newMatchingAPIServiceAddDecisionTaskYARPCRequest() proto.Message {
	return &AddDecisionTaskRequest{}
}

func newMatchingAPIServiceAddDecisionTaskYARPCResponse() proto.Message {
	return &AddDecisionTaskResponse{}
}

func newMatchingAPIServiceAddActivityTaskYARPCRequest() proto.Message {
	return &AddActivityTaskRequest{}
}

func newMatchingAPIServiceAddActivityTaskYARPCResponse() proto.Message {
	return &AddActivityTaskResponse{}
}

func newMatchingAPIServiceQueryWorkflowYARPCRequest() proto.Message {
	return &QueryWorkflowRequest{}
}

func newMatchingAPIServiceQueryWorkflowYARPCResponse() proto.Message {
	return &QueryWorkflowResponse{}
}

func newMatchingAPIServiceRespondQueryTaskCompletedYARPCRequest() proto.Message {
	return &RespondQueryTaskCompletedRequest{}
}

func newMatchingAPIServiceRespondQueryTaskCompletedYARPCResponse() proto.Message {
	return &RespondQueryTaskCompletedResponse{}
}

func newMatchingAPIServiceCancelOutstandingPollYARPCRequest() proto.Message {
	return &CancelOutstandingPollRequest{}
}

func newMatchingAPIServiceCancelOutstandingPollYARPCResponse() proto.Message {
	return &CancelOutstandingPollResponse{}
}

func newMatchingAPIServiceDescribeTaskListYARPCRequest() proto.Message {
	return &DescribeTaskListRequest{}
}

func newMatchingAPIServiceDescribeTaskListYARPCResponse() proto.Message {
	return &DescribeTaskListResponse{}
}

func newMatchingAPIServiceListTaskListPartitionsYARPCRequest() proto.Message {
	return &ListTaskListPartitionsRequest{}
}

func newMatchingAPIServiceListTaskListPartitionsYARPCResponse() proto.Message {
	return &ListTaskListPartitionsResponse{}
}

var (
	emptyMatchingAPIServicePollForDecisionTaskYARPCRequest        = &PollForDecisionTaskRequest{}
	emptyMatchingAPIServicePollForDecisionTaskYARPCResponse       = &PollForDecisionTaskResponse{}
	emptyMatchingAPIServicePollForActivityTaskYARPCRequest        = &PollForActivityTaskRequest{}
	emptyMatchingAPIServicePollForActivityTaskYARPCResponse       = &PollForActivityTaskResponse{}
	emptyMatchingAPIServiceAddDecisionTaskYARPCRequest            = &AddDecisionTaskRequest{}
	emptyMatchingAPIServiceAddDecisionTaskYARPCResponse           = &AddDecisionTaskResponse{}
	emptyMatchingAPIServiceAddActivityTaskYARPCRequest            = &AddActivityTaskRequest{}
	emptyMatchingAPIServiceAddActivityTaskYARPCResponse           = &AddActivityTaskResponse{}
	emptyMatchingAPIServiceQueryWorkflowYARPCRequest              = &QueryWorkflowRequest{}
	emptyMatchingAPIServiceQueryWorkflowYARPCResponse             = &QueryWorkflowResponse{}
	emptyMatchingAPIServiceRespondQueryTaskCompletedYARPCRequest  = &RespondQueryTaskCompletedRequest{}
	emptyMatchingAPIServiceRespondQueryTaskCompletedYARPCResponse = &RespondQueryTaskCompletedResponse{}
	emptyMatchingAPIServiceCancelOutstandingPollYARPCRequest      = &CancelOutstandingPollRequest{}
	emptyMatchingAPIServiceCancelOutstandingPollYARPCResponse     = &CancelOutstandingPollResponse{}
	emptyMatchingAPIServiceDescribeTaskListYARPCRequest           = &DescribeTaskListRequest{}
	emptyMatchingAPIServiceDescribeTaskListYARPCResponse          = &DescribeTaskListResponse{}
	emptyMatchingAPIServiceListTaskListPartitionsYARPCRequest     = &ListTaskListPartitionsRequest{}
	emptyMatchingAPIServiceListTaskListPartitionsYARPCResponse    = &ListTaskListPartitionsResponse{}
)

var yarpcFileDescriptorClosure826e827d3aabf7fc = [][]byte{
	// uber/cadence/matching/v1/service.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xc4, 0x59, 0x5f, 0x6f, 0xdb, 0xc8,
		0x11, 0x07, 0xed, 0xd8, 0xb2, 0x46, 0xb2, 0xe2, 0x6c, 0x7a, 0x0e, 0xad, 0xc4, 0xb1, 0xa3, 0xeb,
		0x5d, 0xdd, 0xe2, 0x4a, 0xd5, 0xba, 0x3f, 0xcd, 0x25, 0x28, 0x0a, 0x9f, 0x1d, 0xf7, 0x04, 0x34,
		0x4d, 0x8e, 0x51, 0x53, 0xa0, 0x28, 0x42, 0xac, 0xc8, 0xb5, 0xc5, 0x9a, 0xe2, 0x32, 0xdc, 0xa5,
		0x7c, 0x7a, 0xe9, 0x43, 0x51, 0x14, 0x05, 0xee, 0xb5, 0x9f, 0xa0, 0xbd, 0x0f, 0xd3, 0x87, 0x7e,
		0x83, 0xa2, 0x8f, 0xfd, 0x1e, 0xc5, 0xfe, 0x21, 0x25, 0x4a, 0x94, 0x2c, 0xd9, 0xed, 0xe5, 0xcd,
		0xdc, 0x9d, 0xf9, 0xcd, 0xcc, 0xce, 0xcc, 0x6f, 0x67, 0x2d, 0xf8, 0x30, 0xe9, 0x92, 0xb8, 0xe9,
		0x62, 0x8f, 0x84, 0x2e, 0x69, 0xf6, 0x31, 0x77, 0x7b, 0x7e, 0x78, 0xde, 0x1c, 0x1c, 0x36, 0x19,
		0x89, 0x07, 0xbe, 0x4b, 0xac, 0x28, 0xa6, 0x9c, 0x22, 0x53, 0xc8, 0x59, 0x5a, 0xce, 0x4a, 0xe5,
		0xac, 0xc1, 0x61, 0xfd, 0xe1, 0x39, 0xa5, 0xe7, 0x01, 0x69, 0x4a, 0xb9, 0x6e, 0x72, 0xd6, 0xf4,
		0x92, 0x18, 0x73, 0x9f, 0x86, 0x4a, 0xb3, 0xbe, 0x37, 0xb9, 0xcf, 0xfd, 0x3e, 0x61, 0x1c, 0xf7,
		0x23, 0x2d, 0x30, 0x05, 0x70, 0x19, 0xe3, 0x28, 0x22, 0x31, 0xd3, 0xfb, 0xfb, 0x39, 0x17, 0x71,
		0xe4, 0x0b, 0xef, 0x5c, 0xda, 0xef, 0x8f, 0x4c, 0x14, 0x49, 0xbc, 0x4d, 0x48, 0x3c, 0xd4, 0x02,
		0x8d, 0x22, 0x01, 0x8e, 0xd9, 0x45, 0xe0, 0x33, 0xae, 0x65, 0x0e, 0x8a, 0x64, 0xf4, 0x21, 0x38,
		0x97, 0x34, 0xbe, 0x20, 0xb1, 0x96, 0xfc, 0xd1, 0x55, 0x92, 0x67, 0x01, 0xbd, 0xd4, 0xb2, 0xdf,
		0xcf, 0xc9, 0xb2, 0x1e, 0x8e, 0x89, 0x27, 0xc4, 0x7b, 0x3e, 0xe3, 0x34, 0xf3, 0xef, 0x83, 0x19,
		0x52, 0x79, 0x17, 0x1b, 0xff, 0x30, 0xa0, 0xfe, 0x92, 0x06, 0xc1, 0x29, 0x8d, 0x4f, 0x88, 0xeb,
		0x33, 0x9f, 0x86, 0x1d, 0xcc, 0x2e, 0x6c, 0xf2, 0x36, 0x21, 0x8c, 0xa3, 0x36, 0x94, 0x62, 0xf5,
		0xa7, 0x69, 0xec, 0x1b, 0x07, 0x95, 0x56, 0xd3, 0xca, 0x65, 0x0d, 0x47, 0xbe, 0x35, 0x38, 0xb4,
		0x66, 0x23, 0xd8, 0xa9, 0x3e, 0xba, 0x0f, 0x65, 0x8f, 0xf6, 0xb1, 0x1f, 0x3a, 0xbe, 0x67, 0xae,
		0xec, 0x1b, 0x07, 0x65, 0x7b, 0x43, 0x2d, 0xb4, 0x3d, 0xb1, 0x19, 0xd1, 0x20, 0x20, 0xb1, 0xd8,
		0x5c, 0x55, 0x9b, 0x6a, 0xa1, 0xed, 0xa1, 0x0f, 0xa0, 0x76, 0x46, 0xe3, 0x4b, 0x1c, 0x7b, 0xc4,
		0x73, 0xce, 0x62, 0xda, 0x37, 0x6f, 0x49, 0x89, 0xcd, 0x6c, 0xf5, 0x34, 0xa6, 0xfd, 0xc6, 0x9f,
		0xca, 0x70, 0xbf, 0xd0, 0x11, 0x16, 0xd1, 0x90, 0x11, 0xb4, 0x0b, 0x20, 0x82, 0x77, 0x38, 0xbd,
		0x20, 0xa1, 0x0c, 0xa7, 0x6a, 0x97, 0xc5, 0x4a, 0x47, 0x2c, 0xa0, 0x5f, 0x03, 0x4a, 0x0f, 0xda,
		0x21, 0x5f, 0x13, 0x37, 0x11, 0x05, 0x27, 0x1d, 0xad, 0xb4, 0x3e, 0x2c, 0x8c, 0xfa, 0x37, 0x5a,
		0xfc, 0x59, 0x2a, 0x6d, 0xdf, 0xb9, 0x9c, 0x5c, 0x42, 0xa7, 0xb0, 0x99, 0xc1, 0xf2, 0x61, 0x44,
		0x64, 0x74, 0x95, 0xd6, 0xa3, 0xb9, 0x88, 0x9d, 0x61, 0x44, 0xec, 0xea, 0xe5, 0xd8, 0x17, 0x7a,
		0x0d, 0x3b, 0x51, 0x4c, 0x06, 0x3e, 0x4d, 0x98, 0xc3, 0x38, 0x8e, 0x39, 0xf1, 0x1c, 0x32, 0x20,
		0x21, 0x17, 0x27, 0x76, 0x4b, 0x62, 0xde, 0xb7, 0x54, 0xd9, 0x5b, 0x69, 0xd9, 0x5b, 0xed, 0x90,
		0x7f, 0xf6, 0xc9, 0x6b, 0x1c, 0x24, 0xc4, 0xde, 0x4e, 0xb5, 0x5f, 0x29, 0xe5, 0x67, 0x42, 0xb7,
		0xed, 0xa1, 0x03, 0xd8, 0x9a, 0x82, 0x5b, 0xdb, 0x37, 0x0e, 0x56, 0xed, 0x1a, 0xcb, 0x4b, 0x9a,
		0x50, 0xc2, 0x9c, 0x93, 0x7e, 0xc4, 0xcd, 0xf5, 0x7d, 0xe3, 0x60, 0xcd, 0x4e, 0x3f, 0x51, 0x03,
		0x36, 0x43, 0xf2, 0x35, 0x1f, 0x01, 0x94, 0x24, 0x40, 0x45, 0x2c, 0xa6, 0xda, 0x1f, 0x01, 0xea,
		0x62, 0xf7, 0x22, 0xa0, 0xe7, 0x8e, 0x4b, 0x93, 0x90, 0x3b, 0x3d, 0x3f, 0xe4, 0xe6, 0x86, 0x14,
		0xdc, 0xd2, 0x3b, 0xc7, 0x62, 0xe3, 0x4b, 0x3f, 0xe4, 0xe8, 0x31, 0x98, 0x8c, 0xfb, 0xee, 0xc5,
		0x70, 0x94, 0x0a, 0x87, 0x84, 0xb8, 0x1b, 0x10, 0xcf, 0x2c, 0xef, 0x1b, 0x07, 0x1b, 0xf6, 0xb6,
		0xda, 0xcf, 0x0e, 0xfa, 0x99, 0xda, 0x45, 0x8f, 0x61, 0x4d, 0xb6, 0xa9, 0x09, 0xf2, 0x4c, 0x1a,
		0x73, 0xcf, 0xf9, 0x2b, 0x21, 0x69, 0x2b, 0x05, 0x64, 0xc3, 0xa6, 0xa7, 0xeb, 0xc6, 0xf1, 0xc3,
		0x33, 0x6a, 0x56, 0x24, 0xc2, 0x8f, 0xf3, 0x08, 0xaa, 0x93, 0x04, 0x48, 0x27, 0xc6, 0x21, 0xf3,
		0x49, 0xc8, 0xd3, 0x6a, 0x6b, 0x87, 0x67, 0xd4, 0xae, 0x7a, 0x63, 0x5f, 0xe8, 0x0d, 0x3c, 0x98,
		0x2e, 0x2a, 0x47, 0x96, 0xa1, 0x68, 0x42, 0xb3, 0x2a, 0x4d, 0xec, 0x16, 0x3a, 0x29, 0x8a, 0xf7,
		0x97, 0x3e, 0xe3, 0xf6, 0xce, 0x54, 0x55, 0xa5, 0x5b, 0xc8, 0x82, 0xbb, 0xea, 0xd0, 0x45, 0xeb,
		0x13, 0x67, 0x40, 0x62, 0x61, 0xda, 0xdc, 0x94, 0xf9, 0xb9, 0x23, 0xb7, 0x5e, 0x89, 0x9d, 0xd7,
		0x6a, 0x03, 0x3d, 0x82, 0x6a, 0x37, 0xc6, 0xa1, 0xdb, 0xd3, 0x5d, 0x50, 0x93, 0x5d, 0x50, 0x51,
		0x6b, 0xaa, 0x0f, 0x8e, 0xa0, 0xc6, 0xdc, 0x1e, 0xf1, 0x92, 0x80, 0x78, 0x8e, 0x20, 0x56, 0xf3,
		0xb6, 0x74, 0xb2, 0x3e, 0x55, 0x5d, 0x9d, 0x94, 0x75, 0xed, 0xcd, 0x4c, 0x43, 0xac, 0xa1, 0x9f,
		0x41, 0x35, 0xad, 0x29, 0x09, 0xb0, 0x75, 0x25, 0x40, 0x45, 0xcb, 0x4b, 0xf5, 0xdf, 0x41, 0x49,
		0x64, 0xc4, 0x27, 0xcc, 0xbc, 0xb3, 0xbf, 0x7a, 0x50, 0x69, 0x7d, 0x61, 0xcd, 0xba, 0x2a, 0xac,
		0x39, 0x0d, 0x6f, 0x7d, 0xa5, 0x40, 0x9e, 0x85, 0x3c, 0x1e, 0xda, 0x29, 0x64, 0xfd, 0x0d, 0x54,
		0xc7, 0x37, 0xd0, 0x16, 0xac, 0x5e, 0x90, 0xa1, 0xe4, 0x83, 0xb2, 0x2d, 0xfe, 0x14, 0x25, 0x34,
		0x10, 0x3d, 0xa3, 0x9b, 0x7f, 0xa1, 0x12, 0x92, 0x0a, 0x4f, 0x56, 0x1e, 0x1b, 0xe3, 0x8c, 0x7a,
		0xe4, 0x72, 0x7f, 0xe0, 0xf3, 0xe1, 0xf5, 0x19, 0xb5, 0x00, 0xe1, 0x3b, 0x64, 0xd4, 0x6f, 0x36,
		0x32, 0x46, 0xcd, 0x3b, 0xf2, 0x4e, 0x19, 0x75, 0x0f, 0x2a, 0x58, 0x7b, 0x33, 0x8a, 0x0d, 0xd2,
		0xa5, 0xb6, 0x27, 0x28, 0x37, 0x13, 0x90, 0x94, 0x7b, 0x6b, 0x0e, 0xe5, 0x66, 0x81, 0x49, 0xca,
		0xc5, 0x63, 0x5f, 0xa8, 0x05, 0x6b, 0x7e, 0x18, 0x25, 0x5c, 0xf2, 0x61, 0xa5, 0xf5, 0xa0, 0x38,
		0x51, 0x78, 0x18, 0x50, 0xec, 0xd9, 0x4a, 0xb4, 0xa0, 0x7b, 0xd6, 0x6f, 0xda, 0x3d, 0xa5, 0xe5,
		0xba, 0xa7, 0x03, 0x3b, 0x29, 0x9e, 0xc3, 0xa9, 0xe3, 0x06, 0x94, 0x11, 0x09, 0x44, 0x13, 0xc5,
		0xb7, 0x95, 0xd6, 0xce, 0x14, 0xd6, 0x89, 0x1e, 0xb0, 0xec, 0xed, 0x54, 0xb7, 0x43, 0x8f, 0x85,
		0x66, 0x47, 0x29, 0xa2, 0x5f, 0xc1, 0xb6, 0x34, 0x32, 0x0d, 0x59, 0xbe, 0x0a, 0xf2, 0xae, 0x54,
		0x9c, 0xc0, 0x3b, 0x85, 0x3b, 0x3d, 0x82, 0x63, 0xde, 0x25, 0x98, 0x67, 0x50, 0x70, 0x15, 0xd4,
		0x56, 0xa6, 0x93, 0xe2, 0x8c, 0x5d, 0x4a, 0x95, 0xfc, 0xa5, 0xf4, 0x06, 0x1e, 0xe6, 0x33, 0xe1,
		0xd0, 0x33, 0x87, 0xf7, 0x7c, 0xe6, 0xa4, 0x0a, 0xd5, 0x2b, 0x0f, 0xb6, 0x9e, 0xcb, 0xcc, 0x8b,
		0xb3, 0x4e, 0xcf, 0x67, 0x47, 0x1a, 0xbf, 0x3d, 0x1e, 0x81, 0x47, 0x38, 0xf6, 0x03, 0x26, 0x89,
		0xf7, 0xaa, 0x4a, 0x19, 0x05, 0x71, 0xa2, 0xb4, 0xa6, 0x67, 0x84, 0xda, 0xf5, 0x66, 0x84, 0x1f,
		0xc0, 0xed, 0x0c, 0x47, 0x11, 0x81, 0xe4, 0xee, 0xb2, 0x5d, 0x4b, 0x97, 0x4f, 0xe4, 0x2a, 0xfa,
		0x18, 0xd6, 0x7b, 0x04, 0x7b, 0x24, 0xd6, 0xd4, 0x7c, 0xbf, 0xd0, 0xd2, 0x97, 0x52, 0xc4, 0xd6,
		0xa2, 0x8d, 0xbf, 0xad, 0xc2, 0xf6, 0x91, 0xe7, 0x15, 0x8d, 0x89, 0x39, 0x26, 0x32, 0x26, 0x98,
		0xe8, 0xff, 0x44, 0x03, 0x4f, 0xa0, 0x3c, 0xba, 0x47, 0x57, 0x17, 0xb9, 0x47, 0x37, 0x78, 0x7a,
		0x6d, 0xee, 0x41, 0x25, 0xeb, 0x11, 0x3d, 0x3e, 0xad, 0xda, 0x90, 0x2e, 0xb5, 0xbd, 0xc9, 0x26,
		0xd2, 0xa5, 0xaf, 0xcb, 0x74, 0x6d, 0x89, 0x26, 0x92, 0xd3, 0x56, 0x5a, 0xac, 0x4f, 0x60, 0x9d,
		0xd1, 0x24, 0x76, 0x15, 0x29, 0xd4, 0x26, 0x6f, 0x96, 0xb1, 0xd1, 0x02, 0xb3, 0x8b, 0x57, 0x52,
		0xd2, 0xd6, 0x1a, 0x05, 0x94, 0x5d, 0x2a, 0xa2, 0xec, 0x1d, 0xb8, 0x37, 0x95, 0x23, 0xc5, 0xd6,
		0x8d, 0x7f, 0xaa, 0xfc, 0x15, 0x5d, 0x4a, 0xef, 0x22, 0x7f, 0x62, 0xf0, 0x94, 0xa1, 0x39, 0x23,
		0xd3, 0x8a, 0xcb, 0x6b, 0x6a, 0xfd, 0x24, 0x75, 0x20, 0x97, 0xe9, 0x5b, 0x37, 0xca, 0xf4, 0xda,
		0x72, 0x99, 0x5e, 0xbf, 0x79, 0xa6, 0x4b, 0xff, 0x83, 0x4c, 0x6f, 0xcc, 0xce, 0x74, 0xd1, 0xbd,
		0xdc, 0xf8, 0x97, 0x01, 0xdf, 0x93, 0x73, 0x49, 0x9a, 0x88, 0x34, 0xcf, 0xc7, 0x93, 0xc3, 0xc7,
		0x0f, 0x0b, 0xcf, 0xb1, 0x48, 0x77, 0xc1, 0xb1, 0xe3, 0x26, 0x5d, 0xb9, 0xe0, 0x54, 0xf2, 0x77,
		0x03, 0xde, 0x9b, 0xf0, 0x50, 0xcf, 0x23, 0x3f, 0x87, 0xaa, 0x1c, 0xe5, 0x9d, 0x98, 0xb0, 0x24,
		0x48, 0x63, 0x9c, 0xcf, 0xc6, 0x15, 0xa9, 0x61, 0x4b, 0x05, 0xd4, 0x86, 0x5a, 0x0a, 0xf0, 0x7b,
		0xe2, 0x72, 0xe2, 0xcd, 0x1d, 0x01, 0xd5, 0xe8, 0xa7, 0x25, 0xed, 0xcd, 0xb7, 0xe3, 0x9f, 0x8d,
		0xff, 0x18, 0xb0, 0xaf, 0x1c, 0xf3, 0xa4, 0x9c, 0x88, 0xf7, 0x98, 0xf6, 0xa3, 0x80, 0x08, 0x61,
		0x7d, 0x94, 0x2f, 0x26, 0xf3, 0xf1, 0x69, 0xa1, 0xa1, 0xab, 0x70, 0xbe, 0x83, 0xdc, 0xdc, 0x83,
		0x92, 0xd4, 0xd5, 0x6c, 0x59, 0xb6, 0xd7, 0xc5, 0x67, 0xdb, 0x6b, 0xbc, 0x0f, 0x8f, 0xe6, 0xb8,
		0xa7, 0x0b, 0xf2, 0xdf, 0x06, 0x3c, 0x38, 0xc6, 0xa1, 0x4b, 0x82, 0x17, 0x09, 0x67, 0x1c, 0x87,
		0x9e, 0x1f, 0x9e, 0x8b, 0xc9, 0x72, 0x21, 0x02, 0xca, 0x8d, 0xb2, 0x2b, 0x13, 0xa3, 0xec, 0x2f,
		0xa0, 0x96, 0x05, 0x35, 0x7a, 0x60, 0xd7, 0x66, 0x5c, 0x9e, 0x69, 0x64, 0xea, 0xf2, 0xe4, 0x63,
		0x5f, 0x37, 0x61, 0x99, 0xc6, 0x1e, 0xec, 0xce, 0x08, 0x4f, 0x1f, 0xc0, 0x1f, 0xe0, 0xde, 0x09,
		0x61, 0x6e, 0xec, 0x77, 0x49, 0xa6, 0xae, 0x43, 0x3f, 0x9d, 0xac, 0x81, 0x8f, 0x0a, 0xad, 0xce,
		0x50, 0x5f, 0x2c, 0xf5, 0x8d, 0x6f, 0x0d, 0x30, 0xa7, 0x11, 0x74, 0xdb, 0x7c, 0x0e, 0x25, 0x75,
		0x9c, 0xcc, 0x34, 0xe4, 0x7b, 0x6b, 0x6f, 0xe6, 0x93, 0x84, 0xc4, 0xf2, 0x91, 0x9b, 0xca, 0xa3,
		0xe7, 0xb0, 0x35, 0x3a, 0x7d, 0xc6, 0x31, 0x4f, 0x98, 0x6e, 0x99, 0xf7, 0xe7, 0x9e, 0xdd, 0x2b,
		0x29, 0x6a, 0xd7, 0x78, 0xee, 0xbb, 0xc1, 0x60, 0x57, 0xe6, 0x43, 0xaf, 0xbe, 0xc4, 0x31, 0xf7,
		0x05, 0xcf, 0xb2, 0xf4, 0xb0, 0xb6, 0x61, 0x5d, 0x0f, 0x36, 0xaa, 0x48, 0xf4, 0x57, 0x3e, 0x79,
		0x2b, 0xcb, 0x25, 0xef, 0xcf, 0x2b, 0xf0, 0x70, 0x96, 0x55, 0x7d, 0x42, 0x6f, 0x61, 0x77, 0xf4,
		0xa2, 0xc8, 0xe2, 0x8d, 0x32, 0x41, 0x7d, 0x6e, 0xd6, 0x5c, 0x93, 0x19, 0xee, 0x73, 0xc2, 0xb1,
		0x87, 0x39, 0xb6, 0xeb, 0x78, 0x8c, 0xbd, 0xf3, 0xa6, 0x85, 0xc9, 0xec, 0xbf, 0x11, 0x85, 0x26,
		0x57, 0xae, 0x67, 0xd2, 0x1b, 0x1b, 0x0d, 0xf2, 0x26, 0x5b, 0xdf, 0x96, 0xa1, 0xf2, 0x5c, 0xbf,
		0xad, 0x8f, 0x5e, 0xb6, 0xd1, 0x1f, 0x0d, 0xb8, 0x5b, 0xf0, 0xbe, 0x46, 0x9f, 0x2c, 0xf9, 0x1c,
		0x97, 0xa9, 0xab, 0x7f, 0x7a, 0xad, 0x47, 0xfc, 0xb8, 0x13, 0xe3, 0x77, 0xdd, 0x02, 0x4e, 0x14,
		0x0c, 0x3a, 0x0b, 0x38, 0x51, 0xf8, 0xd0, 0x1d, 0xc0, 0xed, 0x89, 0xa9, 0x0a, 0xfd, 0x64, 0x36,
		0x52, 0xf1, 0x90, 0x5c, 0x3f, 0x5c, 0x42, 0x23, 0x67, 0x37, 0x17, 0xf7, 0x7c, 0xbb, 0x45, 0x31,
		0x1f, 0x2e, 0xa1, 0xa1, 0xed, 0x46, 0xb0, 0x99, 0xbb, 0x61, 0x91, 0x35, 0x1b, 0xa3, 0x68, 0x58,
		0xa8, 0x37, 0x17, 0x96, 0xd7, 0x16, 0xff, 0x6a, 0xc0, 0xce, 0xcc, 0x7b, 0x04, 0x3d, 0x99, 0x0d,
		0x77, 0xd5, 0xdd, 0x58, 0x7f, 0x7a, 0x2d, 0x5d, 0xed, 0xd6, 0x5f, 0x0c, 0x78, 0xaf, 0x90, 0xd9,
		0xd1, 0x67, 0xb3, 0x61, 0xe7, 0xdd, 0x74, 0xf5, 0x9f, 0x2e, 0xad, 0xa7, 0x5d, 0x19, 0xc2, 0xd6,
		0x24, 0x83, 0xa3, 0x39, 0xa9, 0x9d, 0x71, 0x5f, 0xd4, 0x5b, 0xcb, 0xa8, 0x68, 0xd3, 0xdf, 0x18,
		0xb0, 0x5d, 0xcc, 0x90, 0x68, 0x4e, 0x38, 0x73, 0x99, 0xbc, 0xfe, 0x78, 0x79, 0x45, 0xe5, 0xcd,
		0x17, 0x4f, 0x7f, 0xfb, 0xf9, 0xb9, 0xcf, 0x7b, 0x49, 0xd7, 0x72, 0x69, 0xbf, 0x99, 0xfb, 0x99,
		0xc3, 0x3a, 0x27, 0xa1, 0xfa, 0xd1, 0x67, 0xfc, 0x77, 0xa7, 0xa7, 0xe9, 0xdf, 0x83, 0xc3, 0xee,
		0xba, 0xdc, 0xfd, 0xf8, 0xbf, 0x01, 0x00, 0x00, 0xff, 0xff, 0x9d, 0x1c, 0xdc, 0x60, 0xa5, 0x1a,
		0x00, 0x00,
	},
	// google/protobuf/duration.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xe2, 0x92, 0x4b, 0xcf, 0xcf, 0x4f,
		0xcf, 0x49, 0xd5, 0x2f, 0x28, 0xca, 0x2f, 0xc9, 0x4f, 0x2a, 0x4d, 0xd3, 0x4f, 0x29, 0x2d, 0x4a,
		0x2c, 0xc9, 0xcc, 0xcf, 0xd3, 0x03, 0x8b, 0x08, 0xf1, 0x43, 0xe4, 0xf5, 0x60, 0xf2, 0x4a, 0x56,
		0x5c, 0x1c, 0x2e, 0x50, 0x25, 0x42, 0x12, 0x5c, 0xec, 0xc5, 0xa9, 0xc9, 0xf9, 0x79, 0x29, 0xc5,
		0x12, 0x8c, 0x0a, 0x8c, 0x1a, 0xcc, 0x41, 0x30, 0xae, 0x90, 0x08, 0x17, 0x6b, 0x5e, 0x62, 0x5e,
		0x7e, 0xb1, 0x04, 0x93, 0x02, 0xa3, 0x06, 0x6b, 0x10, 0x84, 0xe3, 0xd4, 0xcc, 0xc8, 0x25, 0x9c,
		0x9c, 0x9f, 0xab, 0x87, 0x66, 0xa6, 0x13, 0x2f, 0xcc, 0xc4, 0x00, 0x90, 0x48, 0x00, 0x63, 0x94,
		0x21, 0x54, 0x45, 0x7a, 0x7e, 0x4e, 0x62, 0x5e, 0xba, 0x5e, 0x7e, 0x51, 0x3a, 0xc2, 0x81, 0x25,
		0x95, 0x05, 0xa9, 0xc5, 0xfa, 0xd9, 0x79, 0xf9, 0xe5, 0x79, 0x70, 0xc7, 0x16, 0x24, 0xfd, 0x60,
		0x64, 0x5c, 0xc4, 0xc4, 0xec, 0x1e, 0xe0, 0xb4, 0x8a, 0x49, 0xce, 0x1d, 0xa2, 0x39, 0x00, 0xaa,
		0x43, 0x2f, 0x3c, 0x35, 0x27, 0xc7, 0x1b, 0xa4, 0x3e, 0x04, 0xa4, 0x35, 0x89, 0x0d, 0x6c, 0x94,
		0x31, 0x20, 0x00, 0x00, 0xff, 0xff, 0xef, 0x8a, 0xb4, 0xc3, 0xfb, 0x00, 0x00, 0x00,
	},
	// google/protobuf/timestamp.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xe2, 0x92, 0x4f, 0xcf, 0xcf, 0x4f,
		0xcf, 0x49, 0xd5, 0x2f, 0x28, 0xca, 0x2f, 0xc9, 0x4f, 0x2a, 0x4d, 0xd3, 0x2f, 0xc9, 0xcc, 0x4d,
		0x2d, 0x2e, 0x49, 0xcc, 0x2d, 0xd0, 0x03, 0x0b, 0x09, 0xf1, 0x43, 0x14, 0xe8, 0xc1, 0x14, 0x28,
		0x59, 0x73, 0x71, 0x86, 0xc0, 0xd4, 0x08, 0x49, 0x70, 0xb1, 0x17, 0xa7, 0x26, 0xe7, 0xe7, 0xa5,
		0x14, 0x4b, 0x30, 0x2a, 0x30, 0x6a, 0x30, 0x07, 0xc1, 0xb8, 0x42, 0x22, 0x5c, 0xac, 0x79, 0x89,
		0x79, 0xf9, 0xc5, 0x12, 0x4c, 0x0a, 0x8c, 0x1a, 0xac, 0x41, 0x10, 0x8e, 0x53, 0x2b, 0x23, 0x97,
		0x70, 0x72, 0x7e, 0xae, 0x1e, 0x9a, 0xa1, 0x4e, 0x7c, 0x70, 0x23, 0x03, 0x40, 0x42, 0x01, 0x8c,
		0x51, 0x46, 0x50, 0x25, 0xe9, 0xf9, 0x39, 0x89, 0x79, 0xe9, 0x7a, 0xf9, 0x45, 0xe9, 0x48, 0x6e,
		0xac, 0x2c, 0x48, 0x2d, 0xd6, 0xcf, 0xce, 0xcb, 0x2f, 0xcf, 0x43, 0xb8, 0xb7, 0x20, 0xe9, 0x07,
		0x23, 0xe3, 0x22, 0x26, 0x66, 0xf7, 0x00, 0xa7, 0x55, 0x4c, 0x72, 0xee, 0x10, 0xdd, 0x01, 0x50,
		0x2d, 0x7a, 0xe1, 0xa9, 0x39, 0x39, 0xde, 0x20, 0x0d, 0x21, 0x20, 0xbd, 0x49, 0x6c, 0x60, 0xb3,
		0x8c, 0x01, 0x01, 0x00, 0x00, 0xff, 0xff, 0xae, 0x65, 0xce, 0x7d, 0xff, 0x00, 0x00, 0x00,
	},
	// google/protobuf/wrappers.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xe2, 0x92, 0x4b, 0xcf, 0xcf, 0x4f,
		0xcf, 0x49, 0xd5, 0x2f, 0x28, 0xca, 0x2f, 0xc9, 0x4f, 0x2a, 0x4d, 0xd3, 0x2f, 0x2f, 0x4a, 0x2c,
		0x28, 0x48, 0x2d, 0x2a, 0xd6, 0x03, 0x8b, 0x08, 0xf1, 0x43, 0xe4, 0xf5, 0x60, 0xf2, 0x4a, 0xca,
		0x5c, 0xdc, 0x2e, 0xf9, 0xa5, 0x49, 0x39, 0xa9, 0x61, 0x89, 0x39, 0xa5, 0xa9, 0x42, 0x22, 0x5c,
		0xac, 0x65, 0x20, 0x86, 0x04, 0xa3, 0x02, 0xa3, 0x06, 0x63, 0x10, 0x84, 0xa3, 0xa4, 0xc4, 0xc5,
		0xe5, 0x96, 0x93, 0x9f, 0x58, 0x82, 0x45, 0x0d, 0x13, 0x92, 0x1a, 0xcf, 0xbc, 0x12, 0x33, 0x13,
		0x2c, 0x6a, 0x98, 0x61, 0x6a, 0x94, 0xb9, 0xb8, 0x43, 0x71, 0x29, 0x62, 0x41, 0x35, 0xc8, 0xd8,
		0x08, 0x8b, 0x1a, 0x56, 0x34, 0x83, 0xb0, 0x2a, 0xe2, 0x85, 0x29, 0x52, 0xe4, 0xe2, 0x74, 0xca,
		0xcf, 0xcf, 0xc1, 0xa2, 0x84, 0x03, 0xc9, 0x9c, 0xe0, 0x92, 0xa2, 0xcc, 0xbc, 0x74, 0x2c, 0x8a,
		0x38, 0x91, 0x1c, 0xe4, 0x54, 0x59, 0x92, 0x5a, 0x8c, 0x45, 0x0d, 0x0f, 0x54, 0x8d, 0x53, 0x33,
		0x23, 0x97, 0x70, 0x72, 0x7e, 0xae, 0x1e, 0x5a, 0xf0, 0x3a, 0xf1, 0x86, 0x43, 0xc3, 0x3f, 0x00,
		0x24, 0x12, 0xc0, 0x18, 0x65, 0x08, 0x55, 0x91, 0x9e, 0x9f, 0x93, 0x98, 0x97, 0xae, 0x97, 0x5f,
		0x94, 0x8e, 0x88, 0xab, 0x92, 0xca, 0x82, 0xd4, 0x62, 0xfd, 0xec, 0xbc, 0xfc, 0xf2, 0x3c, 0x78,
		0xbc, 0x15, 0x24, 0xfd, 0x60, 0x64, 0x5c, 0xc4, 0xc4, 0xec, 0x1e, 0xe0, 0xb4, 0x8a, 0x49, 0xce,
		0x1d, 0xa2, 0x39, 0x00, 0xaa, 0x43, 0x2f, 0x3c, 0x35, 0x27, 0xc7, 0x1b, 0xa4, 0x3e, 0x04, 0xa4,
		0x35, 0x89, 0x0d, 0x6c, 0x94, 0x31, 0x20, 0x00, 0x00, 0xff, 0xff, 0x3c, 0x92, 0x48, 0x30, 0x06,
		0x02, 0x00, 0x00,
	},
	// uber/cadence/api/v1/common.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xbc, 0x55, 0x4f, 0x6f, 0xdb, 0x36,
		0x14, 0x9f, 0xe2, 0xda, 0x49, 0x9f, 0xdd, 0xd4, 0x63, 0xd6, 0xd4, 0xc9, 0xfe, 0x79, 0x06, 0x86,
		0x66, 0x3b, 0x48, 0x88, 0x7b, 0x29, 0x56, 0x14, 0x83, 0x13, 0x3b, 0xab, 0xda, 0x2d, 0x31, 0x64,
		0x23, 0xc1, 0x76, 0x98, 0x40, 0x4b, 0x4f, 0x2e, 0x67, 0x89, 0x14, 0x28, 0xca, 0x89, 0x6f, 0xfb,
		0x24, 0x3b, 0xec, 0x2b, 0xed, 0x0b, 0x0d, 0x94, 0xe8, 0xd8, 0xee, 0x3c, 0xf4, 0x32, 0xec, 0x46,
		0xbe, 0xdf, 0x9f, 0xf7, 0xa3, 0xf0, 0x48, 0x41, 0x3b, 0x9f, 0xa0, 0x74, 0x02, 0x1a, 0x22, 0x0f,
		0xd0, 0xa1, 0x29, 0x73, 0xe6, 0xa7, 0x4e, 0x20, 0x92, 0x44, 0x70, 0x3b, 0x95, 0x42, 0x09, 0x72,
		0xa0, 0x19, 0xb6, 0x61, 0xd8, 0x34, 0x65, 0xf6, 0xfc, 0xf4, 0xf8, 0x8b, 0xa9, 0x10, 0xd3, 0x18,
		0x9d, 0x82, 0x32, 0xc9, 0x23, 0x27, 0xcc, 0x25, 0x55, 0x6c, 0x29, 0xea, 0xbc, 0x85, 0x8f, 0x6f,
		0x84, 0x9c, 0x45, 0xb1, 0xb8, 0x1d, 0xdc, 0x61, 0x90, 0x6b, 0x88, 0x7c, 0x09, 0xf5, 0x5b, 0x53,
		0xf4, 0x59, 0xd8, 0xb2, 0xda, 0xd6, 0xc9, 0x43, 0x0f, 0x96, 0x25, 0x37, 0x24, 0x4f, 0xa0, 0x26,
		0x73, 0xae, 0xb1, 0x9d, 0x02, 0xab, 0xca, 0x9c, 0xbb, 0x61, 0xa7, 0x03, 0x8d, 0xa5, 0xd9, 0x78,
		0x91, 0x22, 0x21, 0xf0, 0x80, 0xd3, 0x04, 0x8d, 0x41, 0xb1, 0xd6, 0x9c, 0x5e, 0xa0, 0xd8, 0x9c,
		0xa9, 0xc5, 0xbf, 0x72, 0x3e, 0x87, 0xdd, 0x21, 0x5d, 0xc4, 0x82, 0x86, 0x1a, 0x0e, 0xa9, 0xa2,
		0x05, 0xdc, 0xf0, 0x8a, 0x75, 0xe7, 0x25, 0xec, 0x5e, 0x50, 0x16, 0xe7, 0x12, 0xc9, 0x21, 0xd4,
		0x24, 0xd2, 0x4c, 0x70, 0xa3, 0x37, 0x3b, 0xd2, 0x82, 0xdd, 0x10, 0x15, 0x65, 0x71, 0x56, 0x24,
		0x6c, 0x78, 0xcb, 0x6d, 0xe7, 0x0f, 0x0b, 0x1e, 0xfc, 0x84, 0x89, 0x20, 0xaf, 0xa0, 0x16, 0x31,
		0x8c, 0xc3, 0xac, 0x65, 0xb5, 0x2b, 0x27, 0xf5, 0xee, 0xd7, 0xf6, 0x96, 0xef, 0x67, 0x6b, 0xaa,
		0x7d, 0x51, 0xf0, 0x06, 0x5c, 0xc9, 0x85, 0x67, 0x44, 0xc7, 0x37, 0x50, 0x5f, 0x2b, 0x93, 0x26,
		0x54, 0x66, 0xb8, 0x30, 0x29, 0xf4, 0x92, 0x74, 0xa1, 0x3a, 0xa7, 0x71, 0x8e, 0x45, 0x80, 0x7a,
		0xf7, 0xb3, 0xad, 0xf6, 0xe6, 0x98, 0x5e, 0x49, 0xfd, 0x6e, 0xe7, 0x85, 0xd5, 0xf9, 0xd3, 0x82,
		0xda, 0x6b, 0xa4, 0x21, 0x4a, 0xf2, 0xfd, 0x7b, 0x11, 0x9f, 0x6d, 0xf5, 0x28, 0xc9, 0xff, 0x6f,
		0xc8, 0xbf, 0x2c, 0x68, 0x8e, 0x90, 0xca, 0xe0, 0x5d, 0x4f, 0x29, 0xc9, 0x26, 0xb9, 0xc2, 0x8c,
		0xf8, 0xb0, 0xcf, 0x78, 0x88, 0x77, 0x18, 0xfa, 0x1b, 0xb1, 0x5f, 0x6c, 0x75, 0x7d, 0x5f, 0x6e,
		0xbb, 0xa5, 0x76, 0xfd, 0x1c, 0x8f, 0xd8, 0x7a, 0xed, 0xf8, 0x57, 0x20, 0xff, 0x24, 0xfd, 0x87,
		0xa7, 0x8a, 0x60, 0xaf, 0x4f, 0x15, 0x3d, 0x8b, 0xc5, 0x84, 0x5c, 0xc0, 0x23, 0xe4, 0x81, 0x08,
		0x19, 0x9f, 0xfa, 0x6a, 0x91, 0x96, 0x03, 0xba, 0xdf, 0xfd, 0x6a, 0xab, 0xd7, 0xc0, 0x30, 0xf5,
		0x44, 0x7b, 0x0d, 0x5c, 0xdb, 0xdd, 0x0f, 0xf0, 0xce, 0xda, 0x00, 0x0f, 0xcb, 0x4b, 0x87, 0xf2,
		0x1a, 0x65, 0xc6, 0x04, 0x77, 0x79, 0x24, 0x34, 0x91, 0x25, 0x69, 0xbc, 0xbc, 0x08, 0x7a, 0x4d,
		0x9e, 0xc1, 0xe3, 0x08, 0xa9, 0xca, 0x25, 0xfa, 0xf3, 0x92, 0x6a, 0x2e, 0xdc, 0xbe, 0x29, 0x1b,
		0x83, 0xce, 0x5b, 0x78, 0x3a, 0xca, 0xd3, 0x54, 0x48, 0x85, 0xe1, 0x79, 0xcc, 0x90, 0x2b, 0x83,
		0x64, 0xfa, 0xae, 0x4e, 0x85, 0x9f, 0x85, 0x33, 0xe3, 0x5c, 0x9d, 0x8a, 0x51, 0x38, 0x23, 0x47,
		0xb0, 0xf7, 0x1b, 0x9d, 0xd3, 0x02, 0x28, 0x3d, 0x77, 0xf5, 0x7e, 0x14, 0xce, 0x3a, 0xbf, 0x57,
		0xa0, 0xee, 0xa1, 0x92, 0x8b, 0xa1, 0x88, 0x59, 0xb0, 0x20, 0x7d, 0x68, 0x32, 0xce, 0x14, 0xa3,
		0xb1, 0xcf, 0xb8, 0x42, 0x39, 0xa7, 0x65, 0xca, 0x7a, 0xf7, 0xc8, 0x2e, 0x9f, 0x17, 0x7b, 0xf9,
		0xbc, 0xd8, 0x7d, 0xf3, 0xbc, 0x78, 0x8f, 0x8d, 0xc4, 0x35, 0x0a, 0xe2, 0xc0, 0xc1, 0x84, 0x06,
		0x33, 0x11, 0x45, 0x7e, 0x20, 0x30, 0x8a, 0x58, 0xa0, 0x63, 0x16, 0xbd, 0x2d, 0x8f, 0x18, 0xe8,
		0x7c, 0x85, 0xe8, 0xb6, 0x09, 0xbd, 0x63, 0x49, 0x9e, 0xac, 0xda, 0x56, 0x3e, 0xd8, 0xd6, 0x48,
		0xee, 0xdb, 0x7e, 0xb3, 0x72, 0xa1, 0x4a, 0x61, 0x92, 0xaa, 0xac, 0xf5, 0xa0, 0x6d, 0x9d, 0x54,
		0xef, 0xa9, 0x3d, 0x53, 0x26, 0xaf, 0xe0, 0x53, 0x2e, 0xb8, 0x2f, 0xf5, 0xd1, 0xe9, 0x24, 0x46,
		0x1f, 0xa5, 0x14, 0xd2, 0x2f, 0x9f, 0x94, 0xac, 0x55, 0x6d, 0x57, 0x4e, 0x1e, 0x7a, 0x2d, 0x2e,
		0xb8, 0xb7, 0x64, 0x0c, 0x34, 0xc1, 0x2b, 0x71, 0xf2, 0x06, 0x0e, 0xf0, 0x2e, 0x65, 0x65, 0x90,
		0x55, 0xe4, 0xda, 0x87, 0x22, 0x93, 0x95, 0x6a, 0x99, 0xfa, 0xdb, 0x5b, 0x68, 0xac, 0xcf, 0x14,
		0x39, 0x82, 0x27, 0x83, 0xcb, 0xf3, 0xab, 0xbe, 0x7b, 0xf9, 0x83, 0x3f, 0xfe, 0x79, 0x38, 0xf0,
		0xdd, 0xcb, 0xeb, 0xde, 0x8f, 0x6e, 0xbf, 0xf9, 0x11, 0x39, 0x86, 0xc3, 0x4d, 0x68, 0xfc, 0xda,
		0x73, 0x2f, 0xc6, 0xde, 0x4d, 0xd3, 0x22, 0x87, 0x40, 0x36, 0xb1, 0x37, 0xa3, 0xab, 0xcb, 0xe6,
		0x0e, 0x69, 0xc1, 0x27, 0x9b, 0xf5, 0xa1, 0x77, 0x35, 0xbe, 0x7a, 0xde, 0xac, 0x9c, 0x5d, 0xc3,
		0xd3, 0x40, 0x24, 0xdb, 0x86, 0xfc, 0x6c, 0xaf, 0x97, 0xb2, 0xa1, 0x4e, 0x3f, 0xb4, 0x7e, 0x71,
		0xa6, 0x4c, 0xbd, 0xcb, 0x27, 0x76, 0x20, 0x12, 0x67, 0xe3, 0xc7, 0x64, 0x4f, 0x91, 0x97, 0x3f,
		0x1b, 0xf3, 0x8f, 0x7a, 0x49, 0x53, 0x36, 0x3f, 0x9d, 0xd4, 0x8a, 0xda, 0xf3, 0xbf, 0x03, 0x00,
		0x00, 0xff, 0xff, 0xdc, 0x8c, 0x77, 0x9a, 0xc7, 0x06, 0x00, 0x00,
	},
	// uber/cadence/api/v1/query.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x8c, 0x53, 0x4d, 0x6f, 0xd3, 0x40,
		0x14, 0xc4, 0x41, 0xaa, 0xe8, 0x6b, 0x0b, 0xd6, 0x16, 0x44, 0xa8, 0xfa, 0x11, 0xa5, 0x3d, 0x54,
		0x11, 0xb2, 0x49, 0xe1, 0xd6, 0x93, 0xeb, 0x2c, 0xc8, 0xc8, 0xb5, 0x5d, 0x7b, 0x93, 0x2a, 0x5c,
		0x2c, 0xc7, 0x59, 0x82, 0x69, 0xe2, 0x35, 0xbb, 0x76, 0x42, 0xfe, 0x00, 0x77, 0x7e, 0x0d, 0x7f,
		0x0f, 0xd9, 0x71, 0x48, 0x4a, 0x5c, 0xc4, 0xed, 0x79, 0xde, 0x8c, 0x67, 0x46, 0xab, 0x07, 0x27,
		0xd9, 0x80, 0x72, 0x35, 0x0c, 0x86, 0x34, 0x0e, 0xa9, 0x1a, 0x24, 0x91, 0x3a, 0x6d, 0xab, 0xdf,
		0x32, 0xca, 0xe7, 0x4a, 0xc2, 0x59, 0xca, 0xd0, 0x7e, 0x4e, 0x50, 0x4a, 0x82, 0x12, 0x24, 0x91,
		0x32, 0x6d, 0x1f, 0x34, 0xaa, 0x54, 0x21, 0x9b, 0x4c, 0x58, 0xbc, 0x90, 0x1d, 0x34, 0xab, 0x18,
		0x33, 0xc6, 0xef, 0x3e, 0x8f, 0xd9, 0x6c, 0xc1, 0x69, 0xde, 0xc1, 0xde, 0x6d, 0x89, 0xdc, 0xe4,
		0x8e, 0xe8, 0x08, 0xa0, 0xb0, 0xf6, 0xd3, 0x79, 0x42, 0xeb, 0x52, 0x43, 0x3a, 0xdf, 0x76, 0xb7,
		0x0b, 0x84, 0xcc, 0x13, 0x8a, 0x2e, 0x97, 0xeb, 0x80, 0x8f, 0x44, 0xbd, 0xd6, 0x90, 0xce, 0x77,
		0x2e, 0x0e, 0x95, 0x8a, 0x7c, 0x8a, 0x13, 0xcc, 0xc7, 0x2c, 0x18, 0x96, 0x62, 0x8d, 0x8f, 0x44,
		0xf3, 0x97, 0x04, 0xfb, 0xf7, 0xdc, 0x5c, 0x2a, 0xb2, 0x71, 0x8a, 0x30, 0xec, 0xf0, 0x62, 0x5a,
		0x99, 0x3e, 0xbd, 0x38, 0xab, 0xfc, 0xeb, 0x9a, 0x2c, 0xcf, 0xe3, 0x02, 0xff, 0x33, 0xa3, 0x77,
		0xb0, 0x15, 0xc4, 0x62, 0x46, 0xf9, 0x7f, 0xe5, 0x2a, 0xb9, 0xe8, 0x14, 0xf6, 0x28, 0xe7, 0x8c,
		0xfb, 0x13, 0x2a, 0x44, 0x30, 0xa2, 0xf5, 0xc7, 0x45, 0xe7, 0xdd, 0x02, 0xbc, 0x5e, 0x60, 0x4d,
		0x0a, 0x7b, 0xa5, 0xf3, 0x57, 0x1a, 0xa6, 0x74, 0x88, 0x08, 0xec, 0x86, 0x63, 0x26, 0xa8, 0x2f,
		0xd2, 0x20, 0xcd, 0x44, 0x99, 0xb9, 0x5d, 0xe9, 0xb8, 0xac, 0x8c, 0xbf, 0xd3, 0x30, 0x4b, 0x23,
		0x16, 0xeb, 0xb9, 0xd2, 0x2b, 0x84, 0xee, 0x4e, 0xb8, 0xfa, 0x68, 0xc5, 0xf0, 0xec, 0xaf, 0x82,
		0xe8, 0x08, 0x5e, 0xdd, 0x74, 0xb1, 0xdb, 0xf7, 0x5d, 0xec, 0x75, 0x4d, 0xe2, 0x93, 0xbe, 0x83,
		0x7d, 0xc3, 0xea, 0x69, 0xa6, 0xd1, 0x91, 0x1f, 0xa1, 0x63, 0x38, 0xd8, 0x5c, 0x6b, 0x96, 0x77,
		0x8b, 0x5d, 0xdc, 0x91, 0x25, 0x74, 0x08, 0xf5, 0xcd, 0xfd, 0x7b, 0xcd, 0x30, 0x71, 0x47, 0xae,
		0xb5, 0x7e, 0x4a, 0xf0, 0x7c, 0xad, 0x97, 0xce, 0xe2, 0x61, 0x94, 0x07, 0x44, 0x4d, 0x38, 0x5e,
		0xca, 0x3e, 0x62, 0x9d, 0xf8, 0xba, 0x6d, 0x75, 0x0c, 0x62, 0xd8, 0xd6, 0x9a, 0xf5, 0x29, 0x9c,
		0x3c, 0xc0, 0xb1, 0x6c, 0xe2, 0xdb, 0x0e, 0xb6, 0x64, 0x09, 0xbd, 0x81, 0xd7, 0xff, 0x20, 0xe9,
		0xf6, 0xb5, 0x63, 0x62, 0x82, 0x3b, 0xbe, 0x6e, 0x62, 0xcd, 0x32, 0xfb, 0x72, 0xad, 0xf5, 0x43,
		0x82, 0x17, 0x45, 0x26, 0x9d, 0xc5, 0x22, 0x12, 0x29, 0x8d, 0xc3, 0xb9, 0x49, 0xa7, 0x74, 0xbc,
		0x32, 0xd4, 0x6d, 0xcb, 0x33, 0x3c, 0x82, 0x2d, 0xbd, 0xef, 0x9b, 0xb8, 0x87, 0xcd, 0xb5, 0x54,
		0x67, 0xd0, 0x78, 0x88, 0x84, 0x7b, 0xd8, 0x22, 0x5d, 0xcd, 0x94, 0xa5, 0x55, 0xbf, 0x4d, 0x96,
		0x47, 0x5c, 0xdb, 0xfa, 0x20, 0xd7, 0xae, 0x7a, 0xf0, 0x32, 0x64, 0x93, 0xaa, 0x17, 0xbd, 0x7a,
		0xa2, 0x25, 0x91, 0x93, 0xdf, 0x8f, 0x23, 0x7d, 0x52, 0x47, 0x51, 0xfa, 0x25, 0x1b, 0x28, 0x21,
		0x9b, 0xa8, 0xf7, 0x0e, 0x4e, 0x19, 0xd1, 0x58, 0x2d, 0xae, 0xac, 0xbc, 0xbd, 0xcb, 0x20, 0x89,
		0xa6, 0xed, 0xc1, 0x56, 0x81, 0xbd, 0xfd, 0x1d, 0x00, 0x00, 0xff, 0xff, 0x96, 0x7d, 0xf9, 0x10,
		0xf7, 0x03, 0x00, 0x00,
	},
	// uber/cadence/api/v1/workflow.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xc4, 0x59, 0x4d, 0x6f, 0xdb, 0xca,
		0xd5, 0x7e, 0x29, 0xd9, 0x8e, 0x7d, 0xe4, 0x0f, 0x7a, 0x1c, 0xc7, 0xca, 0xb7, 0xa3, 0xfb, 0x26,
		0x71, 0xd4, 0x1b, 0xf9, 0x3a, 0xb9, 0xb9, 0x69, 0x6e, 0x9a, 0xa6, 0x34, 0x49, 0xc7, 0x4c, 0x64,
		0x4a, 0x1d, 0x51, 0x71, 0x7c, 0x51, 0x94, 0xa0, 0xa5, 0xb1, 0x4d, 0x44, 0x22, 0x05, 0x72, 0x94,
		0xc4, 0xfb, 0x02, 0x5d, 0x77, 0x57, 0xb4, 0x9b, 0xfe, 0x80, 0x02, 0x45, 0x7f, 0x40, 0xd1, 0xa2,
		0x8b, 0xee, 0xba, 0xed, 0xb2, 0xfb, 0xfe, 0x8b, 0x62, 0x86, 0x43, 0x89, 0xb2, 0x3e, 0xa8, 0xb4,
		0xc0, 0xed, 0xce, 0x3c, 0x7c, 0x9e, 0x87, 0x67, 0xce, 0x9c, 0xf3, 0x70, 0x68, 0x41, 0xa1, 0x7b,
		0x4c, 0x82, 0xed, 0x86, 0xd3, 0x24, 0x5e, 0x83, 0x6c, 0x3b, 0x1d, 0x77, 0xfb, 0xc3, 0xce, 0xf6,
		0x47, 0x3f, 0x78, 0x7f, 0xd2, 0xf2, 0x3f, 0x96, 0x3a, 0x81, 0x4f, 0x7d, 0xb4, 0xc6, 0x30, 0x25,
		0x81, 0x29, 0x39, 0x1d, 0xb7, 0xf4, 0x61, 0xe7, 0xda, 0xad, 0x53, 0xdf, 0x3f, 0x6d, 0x91, 0x6d,
		0x0e, 0x39, 0xee, 0x9e, 0x6c, 0x37, 0xbb, 0x81, 0x43, 0x5d, 0xdf, 0x8b, 0x48, 0xd7, 0x6e, 0x5f,
		0xbc, 0x4f, 0xdd, 0x36, 0x09, 0xa9, 0xd3, 0xee, 0x08, 0xc0, 0xe6, 0xa8, 0x27, 0x37, 0xfc, 0x76,
		0xbb, 0x27, 0x31, 0x32, 0x37, 0xea, 0x84, 0xef, 0x5b, 0x6e, 0x48, 0x23, 0x4c, 0xe1, 0x2f, 0x73,
		0xb0, 0x7e, 0x28, 0xd2, 0xd5, 0x3f, 0x91, 0x46, 0x97, 0xa5, 0x60, 0x78, 0x27, 0x3e, 0xaa, 0x03,
		0x8a, 0xd7, 0x61, 0x93, 0xf8, 0x4e, 0x5e, 0xda, 0x94, 0xb6, 0x72, 0x8f, 0xee, 0x95, 0x46, 0x2c,
		0xa9, 0x34, 0xa4, 0x83, 0x57, 0x3f, 0x5e, 0x0c, 0xa1, 0x27, 0x30, 0x43, 0xcf, 0x3b, 0x24, 0x9f,
		0xe1, 0x42, 0x77, 0x26, 0x0a, 0x59, 0xe7, 0x1d, 0x82, 0x39, 0x1c, 0x3d, 0x03, 0x08, 0xa9, 0x13,
		0x50, 0x9b, 0x95, 0x21, 0x9f, 0xe5, 0xe4, 0x6b, 0xa5, 0xa8, 0x46, 0xa5, 0xb8, 0x46, 0x25, 0x2b,
		0xae, 0x11, 0x5e, 0xe0, 0x68, 0x76, 0xcd, 0xa8, 0x8d, 0x96, 0x1f, 0x92, 0x88, 0x3a, 0x93, 0x4e,
		0xe5, 0x68, 0x4e, 0xb5, 0x60, 0x31, 0xa2, 0x86, 0xd4, 0xa1, 0xdd, 0x30, 0x3f, 0xbb, 0x29, 0x6d,
		0x2d, 0x3f, 0xda, 0x99, 0x6e, 0xf5, 0x2a, 0x63, 0xd6, 0x38, 0x11, 0xe7, 0x1a, 0xfd, 0x0b, 0x74,
		0x17, 0x96, 0xcf, 0xdc, 0x90, 0xfa, 0xc1, 0xb9, 0xdd, 0x22, 0xde, 0x29, 0x3d, 0xcb, 0xcf, 0x6d,
		0x4a, 0x5b, 0x59, 0xbc, 0x24, 0xa2, 0x65, 0x1e, 0x44, 0x3f, 0x83, 0xf5, 0x8e, 0x13, 0x10, 0x8f,
		0xf6, 0xcb, 0x6f, 0xbb, 0xde, 0x89, 0x9f, 0xbf, 0xc4, 0x97, 0xb0, 0x35, 0x32, 0x8b, 0x2a, 0x67,
		0x0c, 0xec, 0x24, 0x5e, 0xeb, 0x0c, 0x07, 0x91, 0x02, 0xcb, 0x7d, 0x59, 0x5e, 0x99, 0xf9, 0xd4,
		0xca, 0x2c, 0xf5, 0x18, 0xbc, 0x3a, 0x0f, 0x61, 0xa6, 0x4d, 0xda, 0x7e, 0x7e, 0x81, 0x13, 0xaf,
		0x8e, 0xcc, 0xe7, 0x80, 0xb4, 0x7d, 0xcc, 0x61, 0x08, 0xc3, 0x6a, 0x48, 0x9c, 0xa0, 0x71, 0x66,
		0x3b, 0x94, 0x06, 0xee, 0x71, 0x97, 0x92, 0x30, 0x0f, 0x9c, 0x7b, 0x77, 0x24, 0xb7, 0xc6, 0xd1,
		0x4a, 0x0f, 0x8c, 0xe5, 0xf0, 0x42, 0x04, 0x95, 0x61, 0xd5, 0xe9, 0x52, 0xdf, 0x0e, 0x48, 0x48,
		0xa8, 0xdd, 0xf1, 0x5d, 0x8f, 0x86, 0xf9, 0x1c, 0xd7, 0xdc, 0x1c, 0xa9, 0x89, 0x19, 0xb0, 0xca,
		0x71, 0x78, 0x85, 0x51, 0x13, 0x01, 0x74, 0x1d, 0x16, 0xd8, 0x78, 0xd8, 0x6c, 0x3e, 0xf2, 0x8b,
		0x9b, 0xd2, 0xd6, 0x02, 0x9e, 0x67, 0x81, 0xb2, 0x1b, 0x52, 0xb4, 0x01, 0x97, 0xdc, 0xd0, 0x6e,
		0x04, 0xbe, 0x97, 0x5f, 0xda, 0x94, 0xb6, 0xe6, 0xf1, 0x9c, 0x1b, 0xaa, 0x81, 0xef, 0x15, 0x7e,
		0x9d, 0x81, 0x5b, 0xc3, 0x9b, 0xef, 0x7b, 0x27, 0xee, 0xa9, 0x18, 0x69, 0xf4, 0x6d, 0x52, 0x38,
		0x1a, 0xa1, 0x9b, 0x23, 0xd3, 0xb3, 0xc4, 0xd3, 0x12, 0xcf, 0x75, 0x60, 0xb3, 0xbf, 0x51, 0x62,
		0x06, 0x7c, 0xbb, 0xdf, 0xd1, 0x7e, 0x97, 0x8a, 0x61, 0xba, 0x3a, 0xb4, 0x75, 0x9a, 0x48, 0x00,
		0xdf, 0xe8, 0x49, 0xd4, 0xf8, 0x5c, 0xf8, 0x6a, 0xdc, 0xe3, 0x7e, 0x97, 0xa2, 0x43, 0xb8, 0xce,
		0xd3, 0x1b, 0xa3, 0x9e, 0x4d, 0x53, 0xdf, 0x60, 0xec, 0x11, 0xc2, 0x85, 0xbf, 0x4b, 0xb0, 0x36,
		0xa2, 0x23, 0x59, 0xa1, 0x9b, 0x7e, 0xdb, 0x71, 0x3d, 0xdb, 0x6d, 0xf2, 0x7a, 0x2c, 0xe0, 0xf9,
		0x28, 0x60, 0x34, 0xd1, 0x6d, 0xc8, 0x89, 0x9b, 0x9e, 0xd3, 0x8e, 0x8c, 0x62, 0x01, 0x43, 0x14,
		0x32, 0x9d, 0x36, 0x19, 0xe3, 0x4c, 0xd9, 0xff, 0xd6, 0x99, 0xee, 0xc0, 0xa2, 0xeb, 0xb9, 0xd4,
		0x75, 0x28, 0x69, 0xb2, 0xbc, 0x66, 0xf8, 0x50, 0xe6, 0x7a, 0x31, 0xa3, 0x59, 0xf8, 0x95, 0x04,
		0xeb, 0xfa, 0x27, 0x4a, 0x02, 0xcf, 0x69, 0x7d, 0x2f, 0x6e, 0x79, 0x31, 0xa7, 0xcc, 0x70, 0x4e,
		0xff, 0x9c, 0x85, 0xb5, 0x2a, 0xf1, 0x9a, 0xae, 0x77, 0xaa, 0x34, 0xa8, 0xfb, 0xc1, 0xa5, 0xe7,
		0x3c, 0xa3, 0xdb, 0x90, 0x73, 0xc4, 0x75, 0xbf, 0xca, 0x10, 0x87, 0x8c, 0x26, 0xda, 0x83, 0xa5,
		0x1e, 0x20, 0xd5, 0x92, 0x63, 0x69, 0x6e, 0xc9, 0x8b, 0x4e, 0xe2, 0x0a, 0xbd, 0x84, 0x59, 0x66,
		0x8f, 0x91, 0x2b, 0x2f, 0x3f, 0x7a, 0x30, 0xda, 0x97, 0x06, 0x33, 0x64, 0x4e, 0x48, 0x70, 0xc4,
		0x43, 0x06, 0xac, 0x9e, 0x11, 0x27, 0xa0, 0xc7, 0xc4, 0xa1, 0x76, 0x93, 0x50, 0xc7, 0x6d, 0x85,
		0xc2, 0xa7, 0x6f, 0x8c, 0x31, 0xb9, 0xf3, 0x96, 0xef, 0x34, 0xb1, 0xdc, 0xa3, 0x69, 0x11, 0x0b,
		0xbd, 0x86, 0xb5, 0x96, 0x13, 0x52, 0xbb, 0xaf, 0xc7, 0xad, 0x6d, 0x36, 0xd5, 0xda, 0x56, 0x19,
		0x6d, 0x3f, 0x66, 0x71, 0x7b, 0xdb, 0x03, 0x1e, 0x8c, 0xa6, 0x82, 0x34, 0x23, 0xa5, 0xb9, 0x54,
		0xa5, 0x15, 0x46, 0xaa, 0x45, 0x1c, 0xae, 0x93, 0x87, 0x4b, 0x0e, 0xa5, 0xa4, 0xdd, 0xa1, 0xdc,
		0xb9, 0x67, 0x71, 0x7c, 0x89, 0x1e, 0x80, 0xdc, 0x76, 0x3e, 0xb9, 0xed, 0x6e, 0xdb, 0x16, 0xa1,
		0x90, 0xbb, 0xf0, 0x2c, 0x5e, 0x11, 0x71, 0x45, 0x84, 0x99, 0x5d, 0x87, 0x8d, 0x33, 0xd2, 0xec,
		0xb6, 0xe2, 0x4c, 0x16, 0xd2, 0xed, 0xba, 0xc7, 0xe0, 0x79, 0xa8, 0xb0, 0x42, 0x3e, 0x75, 0xdc,
		0x68, 0x66, 0x23, 0x0d, 0x48, 0xd5, 0x58, 0xee, 0x53, 0xb8, 0xc8, 0x4b, 0x58, 0xe4, 0x45, 0x39,
		0x71, 0xdc, 0x56, 0x37, 0x20, 0xc2, 0x6b, 0x47, 0x6f, 0xd3, 0x5e, 0x84, 0xc1, 0x39, 0xc6, 0x10,
		0x17, 0xe8, 0x2b, 0xb8, 0xcc, 0x05, 0x58, 0xaf, 0x93, 0xc0, 0x76, 0x9b, 0xc4, 0xa3, 0x2e, 0x3d,
		0x17, 0x76, 0x8b, 0xd8, 0xbd, 0x43, 0x7e, 0xcb, 0x10, 0x77, 0x0a, 0xbf, 0xcd, 0xc0, 0x55, 0xd1,
		0x3e, 0xea, 0x99, 0xdb, 0x6a, 0x7e, 0x2f, 0x83, 0xf7, 0x65, 0x42, 0x96, 0x0d, 0x47, 0xd2, 0x8b,
		0xe4, 0x8f, 0x89, 0xf3, 0x09, 0x77, 0xa4, 0x8b, 0x63, 0x9a, 0x1d, 0x1a, 0x53, 0xf4, 0x16, 0xc4,
		0x6b, 0x58, 0x98, 0x6b, 0xc7, 0x6f, 0xb9, 0x8d, 0x73, 0xde, 0xe6, 0xcb, 0x63, 0x12, 0x8d, 0x9c,
		0x93, 0x1b, 0x6a, 0x95, 0xa3, 0xf1, 0x6a, 0xe7, 0x62, 0xa8, 0xf0, 0xb7, 0x4c, 0x6f, 0xfc, 0x35,
		0xd2, 0x70, 0xc3, 0xb8, 0x2e, 0xbd, 0xa9, 0x94, 0xd2, 0xa7, 0x32, 0x26, 0x0e, 0x4c, 0xe5, 0x70,
		0xc7, 0x65, 0x3e, 0xb7, 0xe3, 0x5e, 0xc0, 0xe2, 0xc0, 0xf0, 0xa4, 0x1f, 0xdb, 0x72, 0xe1, 0xe8,
		0xc1, 0x99, 0x19, 0x1c, 0x1c, 0x0c, 0x1b, 0x7e, 0xe0, 0x9e, 0xba, 0x9e, 0xd3, 0xb2, 0x2f, 0x24,
		0x99, 0x3e, 0xea, 0xeb, 0x31, 0xb5, 0x96, 0x4c, 0xb6, 0xf0, 0xa7, 0x0c, 0x5c, 0x8d, 0xed, 0xa9,
		0xec, 0x37, 0x9c, 0x96, 0xe6, 0x86, 0x1d, 0x87, 0x36, 0xce, 0xa6, 0x73, 0xd3, 0xff, 0x7d, 0xb9,
		0x7e, 0x0e, 0xb7, 0x06, 0x33, 0xb0, 0xfd, 0x13, 0x9b, 0x9e, 0xb9, 0xa1, 0x9d, 0xac, 0xe2, 0x64,
		0xc1, 0x6b, 0x03, 0x19, 0x55, 0x4e, 0xac, 0x33, 0x37, 0x14, 0x1e, 0x84, 0x6e, 0x02, 0xf0, 0x53,
		0x02, 0xf5, 0xdf, 0x13, 0x8f, 0xd7, 0x79, 0x11, 0xf3, 0x63, 0x8d, 0xc5, 0x02, 0x85, 0xd7, 0x90,
		0x4b, 0x9e, 0xa5, 0x9e, 0xc3, 0x9c, 0x38, 0x8e, 0x49, 0x9b, 0xd9, 0xad, 0xdc, 0xa3, 0x2f, 0x52,
		0x8e, 0x63, 0xfc, 0xa4, 0x2a, 0x28, 0x85, 0x3f, 0x64, 0x60, 0x79, 0xf0, 0x16, 0xba, 0x0f, 0x2b,
		0xc7, 0xae, 0xe7, 0x04, 0xe7, 0x76, 0xe3, 0x8c, 0x34, 0xde, 0x87, 0xdd, 0xb6, 0xd8, 0x84, 0xe5,
		0x28, 0xac, 0x8a, 0x28, 0x5a, 0x87, 0xb9, 0xa0, 0xeb, 0xc5, 0x2f, 0xcb, 0x05, 0x3c, 0x1b, 0x74,
		0xd9, 0xa9, 0xe2, 0x05, 0x5c, 0x3f, 0x71, 0x83, 0x90, 0xbd, 0x60, 0xa2, 0x66, 0xb7, 0x1b, 0x7e,
		0xbb, 0xd3, 0x22, 0x03, 0x13, 0x9b, 0xe7, 0x90, 0x78, 0x1c, 0xd4, 0x18, 0xc0, 0xe9, 0x8b, 0x8d,
		0x80, 0x38, 0xbd, 0xbd, 0x49, 0x2f, 0x65, 0x4e, 0xe0, 0x85, 0x6d, 0x2e, 0x71, 0x23, 0x75, 0xbd,
		0xd3, 0x69, 0xdb, 0x74, 0x31, 0x26, 0x70, 0x81, 0x5b, 0x00, 0xfc, 0x8c, 0x4b, 0x9d, 0xe3, 0x56,
		0xf4, 0x16, 0x9a, 0xc7, 0x89, 0x48, 0xf1, 0x8f, 0x12, 0x5c, 0x1e, 0xf5, 0x8e, 0x45, 0x05, 0xb8,
		0x55, 0xd5, 0x4d, 0xcd, 0x30, 0x5f, 0xd9, 0x8a, 0x6a, 0x19, 0x6f, 0x0d, 0xeb, 0xc8, 0xae, 0x59,
		0x8a, 0xa5, 0xdb, 0x86, 0xf9, 0x56, 0x29, 0x1b, 0x9a, 0xfc, 0x7f, 0xe8, 0xff, 0x61, 0x73, 0x0c,
		0xa6, 0xa6, 0xee, 0xeb, 0x5a, 0xbd, 0xac, 0x6b, 0xb2, 0x34, 0x41, 0xa9, 0x66, 0x29, 0xd8, 0xd2,
		0x35, 0x39, 0x83, 0x7e, 0x00, 0xf7, 0xc7, 0x60, 0x54, 0xc5, 0x54, 0xf5, 0xb2, 0x8d, 0xf5, 0x9f,
		0xd6, 0xf5, 0x1a, 0x03, 0x67, 0x8b, 0xbf, 0xe8, 0xe7, 0x3c, 0xe0, 0x40, 0xc9, 0x27, 0x69, 0xba,
		0x6a, 0xd4, 0x8c, 0x8a, 0x39, 0x29, 0xe7, 0x0b, 0x98, 0x31, 0x39, 0x5f, 0x44, 0xc5, 0x39, 0x17,
		0x7f, 0x99, 0xe9, 0x7f, 0x02, 0x1b, 0x4d, 0x4c, 0xba, 0xb1, 0xb7, 0xb2, 0x67, 0x1c, 0x56, 0xf0,
		0x9b, 0xbd, 0x72, 0xe5, 0xd0, 0x36, 0x34, 0x1b, 0xeb, 0xf5, 0x9a, 0x6e, 0x57, 0x2b, 0x65, 0x43,
		0x3d, 0x4a, 0x64, 0xf2, 0x43, 0xf8, 0x7a, 0x2c, 0x4a, 0x29, 0xb3, 0xa8, 0x56, 0xaf, 0x96, 0x0d,
		0x95, 0x3d, 0x75, 0x4f, 0x31, 0xca, 0xba, 0x66, 0x57, 0xcc, 0xf2, 0x91, 0x2c, 0xa1, 0x2f, 0x61,
		0x6b, 0x5a, 0xa6, 0x9c, 0x41, 0x0f, 0xe1, 0xc1, 0x58, 0x34, 0xd6, 0x5f, 0xeb, 0xaa, 0x95, 0x80,
		0x67, 0xd1, 0x0e, 0x3c, 0x1c, 0x0b, 0xb7, 0x74, 0x7c, 0x60, 0x98, 0xbc, 0xa0, 0x7b, 0x36, 0xae,
		0x9b, 0xa6, 0x61, 0xbe, 0x92, 0x67, 0x8a, 0xbf, 0x93, 0x60, 0x75, 0xe8, 0xa5, 0x83, 0x6e, 0xc3,
		0xf5, 0xaa, 0x82, 0x75, 0xd3, 0xb2, 0xd5, 0x72, 0x65, 0x54, 0x01, 0xc6, 0x00, 0x94, 0x5d, 0xc5,
		0xd4, 0x2a, 0xa6, 0x2c, 0xa1, 0x7b, 0x50, 0x18, 0x05, 0x10, 0xbd, 0x20, 0x5a, 0x43, 0xce, 0xa0,
		0x3b, 0x70, 0x73, 0x14, 0xae, 0x97, 0xad, 0x9c, 0x2d, 0xfe, 0x2b, 0x03, 0x37, 0x26, 0x7d, 0x69,
		0xb3, 0x0e, 0xec, 0x2d, 0x5b, 0x7f, 0xa7, 0xab, 0x75, 0x8b, 0xed, 0x79, 0xa4, 0xc7, 0x76, 0xbe,
		0x5e, 0x4b, 0x64, 0x9e, 0x2c, 0xe9, 0x18, 0xb0, 0x5a, 0x39, 0xa8, 0x96, 0x75, 0x8b, 0x77, 0x53,
		0x11, 0xee, 0xa5, 0xc1, 0xa3, 0x0d, 0x96, 0x33, 0x03, 0x7b, 0x3b, 0x4e, 0x9a, 0xaf, 0x9b, 0x8d,
		0x02, 0x2a, 0x41, 0x31, 0x0d, 0xdd, 0xab, 0x82, 0x26, 0xcf, 0xa0, 0xaf, 0xe1, 0xab, 0xf4, 0xc4,
		0x4d, 0xcb, 0x30, 0xeb, 0xba, 0x66, 0x2b, 0x35, 0xdb, 0xd4, 0x0f, 0xe5, 0xd9, 0x69, 0x96, 0x6b,
		0x19, 0x07, 0xac, 0x3f, 0xeb, 0x96, 0x3c, 0x57, 0xfc, 0xb3, 0x04, 0x57, 0x54, 0xdf, 0xa3, 0xae,
		0xd7, 0x25, 0x4a, 0x68, 0x92, 0x8f, 0x46, 0x74, 0x9e, 0xf1, 0x03, 0x74, 0x17, 0xee, 0xc4, 0xfa,
		0x42, 0xde, 0x36, 0x4c, 0xc3, 0x32, 0x14, 0xab, 0x82, 0x13, 0xf5, 0x9d, 0x08, 0x63, 0x03, 0xa9,
		0xe9, 0x38, 0xaa, 0xeb, 0x78, 0x18, 0xd6, 0x2d, 0x7c, 0x24, 0x5a, 0x21, 0x72, 0x98, 0xf1, 0x58,
		0x15, 0xb3, 0xf9, 0x16, 0xf3, 0x2f, 0x67, 0x8b, 0xbf, 0x97, 0x20, 0x27, 0xbe, 0x45, 0xf9, 0xa7,
		0x4a, 0x1e, 0x2e, 0xb3, 0x05, 0x56, 0xea, 0x96, 0x6d, 0x1d, 0x55, 0xf5, 0xc1, 0x1e, 0x1e, 0xb8,
		0xc3, 0xed, 0xc1, 0xb6, 0x2a, 0x51, 0x75, 0x22, 0x27, 0x19, 0x04, 0x88, 0xa7, 0x30, 0x0c, 0x07,
		0xcb, 0x99, 0x89, 0x98, 0x48, 0x27, 0x8b, 0xae, 0xc1, 0x95, 0x01, 0xcc, 0xbe, 0xae, 0x60, 0x6b,
		0x57, 0x57, 0x2c, 0x79, 0xa6, 0xf8, 0x1b, 0x09, 0xae, 0xc6, 0x4e, 0x68, 0xb1, 0x17, 0xab, 0xdb,
		0x26, 0xcd, 0x4a, 0x97, 0xaa, 0x4e, 0x37, 0x24, 0xe8, 0x01, 0xdc, 0xed, 0x79, 0x98, 0xa5, 0xd4,
		0xde, 0xf4, 0xf7, 0xca, 0x56, 0x15, 0x36, 0xdc, 0xfd, 0xd5, 0xa4, 0x42, 0x45, 0x0a, 0xb2, 0x84,
		0xee, 0xc3, 0x17, 0x93, 0xa1, 0x58, 0xaf, 0xe9, 0x96, 0x9c, 0x29, 0xfe, 0x23, 0x07, 0x1b, 0xc9,
		0xe4, 0xd8, 0x81, 0x9e, 0x34, 0xa3, 0xd4, 0xee, 0x41, 0x61, 0x50, 0x44, 0xf8, 0xdc, 0xc5, 0xbc,
		0x76, 0xe0, 0xe1, 0x04, 0x5c, 0xdd, 0xdc, 0x57, 0x4c, 0x8d, 0x5d, 0xc7, 0x20, 0x59, 0x42, 0x2f,
		0xe1, 0xf9, 0x04, 0xca, 0xae, 0xa2, 0xf5, 0xab, 0xdc, 0x7b, 0xe3, 0x28, 0x96, 0x85, 0x8d, 0xdd,
		0xba, 0xa5, 0xd7, 0xe4, 0x0c, 0xd2, 0x41, 0x49, 0x11, 0x18, 0xf4, 0xa1, 0x91, 0x32, 0x59, 0xf4,
		0x0c, 0x9e, 0xa4, 0xe5, 0x11, 0xb5, 0x8c, 0x71, 0xa0, 0xe3, 0x24, 0x75, 0x06, 0x7d, 0x0b, 0xdf,
		0xa4, 0x50, 0xc5, 0x93, 0x87, 0xb8, 0xb3, 0xe8, 0x39, 0x3c, 0x4d, 0xcd, 0x5e, 0xad, 0x60, 0xcd,
		0x3e, 0x50, 0xf0, 0x9b, 0x41, 0xf2, 0x1c, 0x32, 0x40, 0x4f, 0x7b, 0xb0, 0x70, 0x37, 0x7b, 0x84,
		0x2f, 0x24, 0xa4, 0x2e, 0x4d, 0x51, 0x45, 0x16, 0x48, 0x91, 0x99, 0x47, 0xaf, 0x40, 0x9d, 0xae,
		0x14, 0x93, 0x85, 0x16, 0xd0, 0x3b, 0xb0, 0x3e, 0x6f, 0x57, 0xf5, 0x77, 0x96, 0x8e, 0x4d, 0x25,
		0x4d, 0x19, 0xd0, 0x0b, 0x78, 0x96, 0x5a, 0xb4, 0x41, 0xff, 0x49, 0xd0, 0x73, 0xe8, 0x29, 0x3c,
		0x9e, 0x40, 0x4f, 0xf6, 0x48, 0xff, 0x54, 0x60, 0x68, 0xf2, 0x22, 0x7a, 0x02, 0x3b, 0x13, 0x88,
		0x7c, 0x0a, 0xed, 0x9a, 0x65, 0xa8, 0x6f, 0x8e, 0xa2, 0xdb, 0x65, 0xa3, 0x66, 0xc9, 0x4b, 0xe8,
		0x27, 0xf0, 0xa3, 0x09, 0xb4, 0xde, 0x62, 0xd9, 0x1f, 0x3a, 0x4e, 0x8c, 0x18, 0x83, 0xd5, 0xb1,
		0x2e, 0x2f, 0x4f, 0xb1, 0x27, 0x35, 0xe3, 0x55, 0x7a, 0xe5, 0x56, 0x90, 0x0a, 0x2f, 0xa7, 0x1a,
		0x11, 0x75, 0xdf, 0x28, 0x6b, 0xa3, 0x45, 0x64, 0xf4, 0x18, 0xb6, 0x27, 0x88, 0xec, 0x55, 0xb0,
		0xaa, 0x8b, 0x37, 0x56, 0xcf, 0x24, 0x56, 0xd1, 0x37, 0xf0, 0x68, 0x12, 0x49, 0x31, 0xca, 0x95,
		0xb7, 0x3a, 0xbe, 0xc8, 0x43, 0xec, 0x35, 0x3a, 0xdd, 0xd2, 0x0d, 0xb3, 0x5a, 0xb7, 0xec, 0x9a,
		0xf1, 0x9d, 0x2e, 0xaf, 0xb1, 0xd7, 0x68, 0xea, 0x4e, 0xc5, 0xb5, 0x92, 0x2f, 0x0f, 0x9b, 0xf1,
		0xd0, 0x43, 0x76, 0x0d, 0x53, 0xc1, 0x47, 0xf2, 0x7a, 0x4a, 0xef, 0x0d, 0x1b, 0xdd, 0x40, 0x0b,
		0x5d, 0x99, 0x66, 0x39, 0xba, 0x82, 0xd5, 0xfd, 0x64, 0xc5, 0x37, 0xd8, 0x5b, 0xe7, 0x0e, 0xff,
		0xc7, 0xca, 0xd0, 0xb9, 0x2a, 0x69, 0xf1, 0x3b, 0xf0, 0x30, 0xda, 0xb7, 0x11, 0x5d, 0x30, 0xc6,
		0xed, 0x77, 0xe1, 0xc7, 0xd3, 0x51, 0x7a, 0xf7, 0x95, 0x32, 0xd6, 0x15, 0xed, 0xa8, 0x77, 0x24,
		0x95, 0x8a, 0x7f, 0x95, 0xa0, 0xa8, 0x3a, 0x5e, 0x83, 0xb4, 0xe2, 0xff, 0xbb, 0x4e, 0xcc, 0xf2,
		0x39, 0x3c, 0x9d, 0x62, 0xde, 0xc7, 0xe4, 0x7b, 0x08, 0xb5, 0xcf, 0x25, 0xd7, 0xcd, 0x37, 0x66,
		0xe5, 0xd0, 0x9c, 0x44, 0x10, 0x8b, 0xa8, 0xb9, 0xa7, 0xfc, 0x9f, 0xc6, 0xd3, 0x2d, 0x42, 0xb4,
		0xdd, 0x7f, 0xb6, 0x88, 0xcf, 0x25, 0x4f, 0xb5, 0x88, 0xdd, 0xb7, 0xb0, 0xd1, 0xf0, 0xdb, 0xa3,
		0xbe, 0xe2, 0x77, 0xe7, 0x95, 0x8e, 0x5b, 0x65, 0x5f, 0xb0, 0x55, 0xe9, 0xbb, 0xed, 0x53, 0x97,
		0x9e, 0x75, 0x8f, 0x4b, 0x0d, 0xbf, 0xbd, 0x3d, 0xf0, 0xfb, 0x63, 0xe9, 0x94, 0x78, 0xd1, 0xaf,
		0x99, 0xe2, 0xa7, 0xc8, 0xe7, 0x4e, 0xc7, 0xfd, 0xb0, 0x73, 0x3c, 0xc7, 0x63, 0x8f, 0xff, 0x1d,
		0x00, 0x00, 0xff, 0xff, 0x1c, 0xfe, 0xdc, 0x7d, 0x4a, 0x1d, 0x00, 0x00,
	},
	// uber/cadence/api/v1/tasklist.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x7c, 0x54, 0xdd, 0x6e, 0xdb, 0x36,
		0x14, 0x9e, 0xe2, 0xb4, 0x73, 0x98, 0x25, 0xd5, 0xb8, 0xb5, 0x8d, 0xdd, 0x75, 0xcb, 0x74, 0x51,
		0x04, 0xc5, 0x20, 0x21, 0x19, 0x76, 0xb5, 0x8b, 0xc1, 0x89, 0x83, 0x55, 0x88, 0xe3, 0x1a, 0x92,
		0x1a, 0x20, 0xbb, 0xe1, 0x28, 0xf1, 0xd4, 0x21, 0xf4, 0x43, 0x81, 0xa4, 0x92, 0xf8, 0x45, 0xf6,
		0x30, 0x7b, 0xa2, 0x3d, 0xc6, 0x40, 0x4a, 0xf6, 0xdc, 0xc4, 0xeb, 0x1d, 0x79, 0xbe, 0xf3, 0x9d,
		0x9f, 0x8f, 0xe7, 0x10, 0x79, 0x4d, 0x0a, 0x32, 0xc8, 0x28, 0x83, 0x2a, 0x83, 0x80, 0xd6, 0x3c,
		0xb8, 0x3d, 0x0e, 0x34, 0x55, 0x79, 0xc1, 0x95, 0xf6, 0x6b, 0x29, 0xb4, 0xc0, 0xdf, 0x18, 0x1f,
		0xbf, 0xf3, 0xf1, 0x69, 0xcd, 0xfd, 0xdb, 0xe3, 0xe1, 0xf7, 0x73, 0x21, 0xe6, 0x05, 0x04, 0xd6,
		0x25, 0x6d, 0x3e, 0x06, 0xac, 0x91, 0x54, 0x73, 0x51, 0xb5, 0xa4, 0xe1, 0x0f, 0x0f, 0x71, 0xcd,
		0x4b, 0x50, 0x9a, 0x96, 0x75, 0xe7, 0xf0, 0x28, 0xc0, 0x9d, 0xa4, 0x75, 0x0d, 0x52, 0xb5, 0xb8,
		0xf7, 0x01, 0xf5, 0x13, 0xaa, 0xf2, 0x09, 0x57, 0x1a, 0x63, 0xb4, 0x5d, 0xd1, 0x12, 0x0e, 0x9c,
		0x43, 0xe7, 0x68, 0x27, 0xb2, 0x67, 0xfc, 0x0b, 0xda, 0xce, 0x79, 0xc5, 0x0e, 0xb6, 0x0e, 0x9d,
		0xa3, 0xfd, 0x93, 0x1f, 0xfd, 0x0d, 0x45, 0xfa, 0xcb, 0x00, 0x17, 0xbc, 0x62, 0x91, 0x75, 0xf7,
		0x28, 0x72, 0x97, 0xd6, 0x4b, 0xd0, 0x94, 0x51, 0x4d, 0xf1, 0x25, 0xfa, 0xb6, 0xa4, 0xf7, 0xc4,
		0xb4, 0xad, 0x48, 0x0d, 0x92, 0x28, 0xc8, 0x44, 0xc5, 0x6c, 0xba, 0xdd, 0x93, 0xef, 0xfc, 0xb6,
		0x52, 0x7f, 0x59, 0xa9, 0x3f, 0x16, 0x4d, 0x5a, 0xc0, 0x15, 0x2d, 0x1a, 0x88, 0xbe, 0x2e, 0xe9,
		0xbd, 0x09, 0xa8, 0x66, 0x20, 0x63, 0x4b, 0xf3, 0x3e, 0xa0, 0xc1, 0x32, 0xc5, 0x8c, 0x4a, 0xcd,
		0x8d, 0x2a, 0xab, 0x5c, 0x2e, 0xea, 0xe5, 0xb0, 0xe8, 0x3a, 0x31, 0x47, 0xfc, 0x06, 0x3d, 0x13,
		0x77, 0x15, 0x48, 0x72, 0x23, 0x94, 0x26, 0xb6, 0xcf, 0x2d, 0x8b, 0xee, 0x59, 0xf3, 0x3b, 0xa1,
		0xf4, 0x94, 0x96, 0xe0, 0xfd, 0xe3, 0xa0, 0xfd, 0x65, 0xdc, 0x58, 0x53, 0xdd, 0x28, 0xfc, 0x13,
		0xc2, 0x29, 0xcd, 0xf2, 0x42, 0xcc, 0x49, 0x26, 0x9a, 0x4a, 0x93, 0x1b, 0x5e, 0x69, 0x1b, 0xbb,
		0x17, 0xb9, 0x1d, 0x72, 0x66, 0x80, 0x77, 0xbc, 0xd2, 0xf8, 0x35, 0x42, 0x12, 0x28, 0x23, 0x05,
		0xdc, 0x42, 0x61, 0x73, 0xf4, 0xa2, 0x1d, 0x63, 0x99, 0x18, 0x03, 0x7e, 0x85, 0x76, 0x68, 0x96,
		0x77, 0x68, 0xcf, 0xa2, 0x7d, 0x9a, 0xe5, 0x2d, 0xf8, 0x06, 0x3d, 0x93, 0x54, 0xc3, 0xba, 0x3a,
		0xdb, 0x87, 0xce, 0x91, 0x13, 0xed, 0x19, 0xf3, 0xaa, 0x77, 0x3c, 0x46, 0x7b, 0x46, 0x46, 0xc2,
		0x19, 0x49, 0x0b, 0x91, 0xe5, 0x07, 0x4f, 0xac, 0x86, 0x87, 0xff, 0xfb, 0x3c, 0xe1, 0xf8, 0xd4,
		0xf8, 0x45, 0xbb, 0x86, 0x16, 0x32, 0x7b, 0xf1, 0x7e, 0x43, 0xbb, 0x6b, 0x18, 0x1e, 0xa0, 0xbe,
		0xd2, 0x54, 0x6a, 0xc2, 0x59, 0xd7, 0xdc, 0x97, 0xf6, 0x1e, 0x32, 0xfc, 0x1c, 0x3d, 0x85, 0x8a,
		0x19, 0xa0, 0xed, 0xe7, 0x09, 0x54, 0x2c, 0x64, 0xde, 0x5f, 0x0e, 0x42, 0x33, 0x51, 0x14, 0x20,
		0xc3, 0xea, 0xa3, 0xc0, 0x63, 0xe4, 0x16, 0x54, 0x69, 0x42, 0xb3, 0x0c, 0x94, 0x22, 0x66, 0x14,
		0xbb, 0xc7, 0x1d, 0x3e, 0x7a, 0xdc, 0x64, 0x39, 0xa7, 0xd1, 0xbe, 0xe1, 0x8c, 0x2c, 0xc5, 0x18,
		0xf1, 0x10, 0xf5, 0x39, 0x83, 0x4a, 0x73, 0xbd, 0xe8, 0x5e, 0x68, 0x75, 0xdf, 0xa4, 0x4f, 0x6f,
		0x83, 0x3e, 0xde, 0xdf, 0x0e, 0x1a, 0xc4, 0x9a, 0x67, 0xf9, 0xe2, 0xfc, 0x1e, 0xb2, 0xc6, 0x8c,
		0xc6, 0x48, 0x6b, 0xc9, 0xd3, 0x46, 0x83, 0xc2, 0xbf, 0x23, 0xf7, 0x4e, 0xc8, 0x1c, 0xa4, 0x9d,
		0x45, 0x62, 0x76, 0xb0, 0xab, 0xf3, 0xf5, 0x67, 0xe7, 0x3b, 0xda, 0x6f, 0x69, 0xab, 0x85, 0x49,
		0xd0, 0x40, 0x65, 0x37, 0xc0, 0x9a, 0x02, 0x88, 0x16, 0xa4, 0x55, 0xcf, 0xb4, 0x2d, 0x1a, 0x6d,
		0x6b, 0xdf, 0x3d, 0x19, 0x3c, 0x1e, 0xeb, 0x6e, 0x83, 0xa3, 0x17, 0x4b, 0x6e, 0x22, 0x62, 0xc3,
		0x4c, 0x5a, 0xe2, 0xdb, 0x3f, 0xd1, 0x57, 0xeb, 0x1b, 0x85, 0x87, 0xe8, 0x45, 0x32, 0x8a, 0x2f,
		0xc8, 0x24, 0x8c, 0x13, 0x72, 0x11, 0x4e, 0xc7, 0x24, 0x9c, 0x5e, 0x8d, 0x26, 0xe1, 0xd8, 0xfd,
		0x02, 0x0f, 0xd0, 0xf3, 0x07, 0xd8, 0xf4, 0x7d, 0x74, 0x39, 0x9a, 0xb8, 0xce, 0x06, 0x28, 0x4e,
		0xc2, 0xb3, 0x8b, 0x6b, 0x77, 0xeb, 0x2d, 0xfb, 0x2f, 0x43, 0xb2, 0xa8, 0xe1, 0xd3, 0x0c, 0xc9,
		0xf5, 0xec, 0x7c, 0x2d, 0xc3, 0x2b, 0xf4, 0xf2, 0x01, 0x36, 0x3e, 0x3f, 0x0b, 0xe3, 0xf0, 0xfd,
		0xd4, 0x75, 0x36, 0x80, 0xa3, 0xb3, 0x24, 0xbc, 0x0a, 0x93, 0x6b, 0x77, 0xeb, 0xf4, 0x0a, 0xbd,
		0xcc, 0x44, 0xb9, 0x49, 0xd1, 0xd3, 0xfe, 0xa8, 0xe6, 0x33, 0x23, 0xc8, 0xcc, 0xf9, 0x23, 0x98,
		0x73, 0x7d, 0xd3, 0xa4, 0x7e, 0x26, 0xca, 0xe0, 0x93, 0x6f, 0xd2, 0x9f, 0x43, 0xd5, 0xfe, 0x5b,
		0xdd, 0x8f, 0xf9, 0x2b, 0xad, 0xf9, 0xed, 0x71, 0xfa, 0xd4, 0xda, 0x7e, 0xfe, 0x37, 0x00, 0x00,
		0xff, 0xff, 0x58, 0x62, 0x2b, 0xbf, 0x55, 0x05, 0x00, 0x00,
	},
	// uber/cadence/api/v1/service_worker.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xe4, 0x5a, 0x5b, 0x6f, 0x1b, 0xc7,
		0x15, 0xc6, 0xea, 0xae, 0xa3, 0x1b, 0x35, 0x46, 0x5d, 0x8a, 0xb6, 0x65, 0x99, 0xa9, 0x6d, 0xa6,
		0x0d, 0xc8, 0x5a, 0x49, 0x1d, 0xc7, 0x8e, 0x8b, 0xea, 0x62, 0xc1, 0x2a, 0xd2, 0x54, 0x59, 0x33,
		0x0a, 0x90, 0x02, 0x5e, 0x8c, 0x76, 0x47, 0xe2, 0x40, 0xcb, 0x1d, 0x7a, 0x77, 0x96, 0x32, 0x5f,
		0xfa, 0xd0, 0xc7, 0xa4, 0x05, 0x0a, 0x14, 0x4d, 0x51, 0xa0, 0x80, 0x9f, 0xfb, 0x03, 0xfa, 0x43,
		0xfa, 0xdc, 0xa7, 0xfe, 0x91, 0xa2, 0xd8, 0x99, 0xd9, 0xe5, 0x92, 0x1c, 0x0e, 0x2f, 0x7d, 0x50,
		0x80, 0xbe, 0x69, 0x67, 0xcf, 0xf9, 0xf6, 0xcc, 0xf9, 0xce, 0x9c, 0xf9, 0x66, 0x44, 0xa8, 0xc4,
		0x67, 0x24, 0xac, 0xb9, 0xd8, 0x23, 0x81, 0x4b, 0x6a, 0xb8, 0x45, 0x6b, 0xed, 0x47, 0xb5, 0x88,
		0x84, 0x6d, 0xea, 0x12, 0xe7, 0x8a, 0x85, 0x97, 0x24, 0xac, 0xb6, 0x42, 0xc6, 0x19, 0xba, 0x91,
		0x58, 0x56, 0x95, 0x65, 0x15, 0xb7, 0x68, 0xb5, 0xfd, 0xa8, 0xb4, 0x7d, 0xc1, 0xd8, 0x85, 0x4f,
		0x6a, 0xc2, 0xe4, 0x2c, 0x3e, 0xaf, 0x79, 0x71, 0x88, 0x39, 0x65, 0x81, 0x74, 0x2a, 0xdd, 0xed,
		0x7f, 0xcf, 0x69, 0x93, 0x44, 0x1c, 0x37, 0x5b, 0xca, 0x60, 0x00, 0xe0, 0x2a, 0xc4, 0xad, 0x16,
		0x09, 0x23, 0xf5, 0x7e, 0x47, 0x17, 0x9f, 0xcb, 0x9a, 0xcd, 0xec, 0x13, 0x65, 0x9d, 0x85, 0x47,
		0x5c, 0x1a, 0x75, 0xc3, 0xb8, 0xa7, 0xb3, 0x69, 0xd0, 0x88, 0xb3, 0xb0, 0x93, 0x46, 0xaa, 0x33,
		0x79, 0x13, 0x93, 0xcc, 0x40, 0xfb, 0x1d, 0x8e, 0xa3, 0x4b, 0x9f, 0x46, 0xdc, 0x64, 0x93, 0x64,
		0xf1, 0xdc, 0x67, 0x57, 0xd2, 0xa6, 0xfc, 0x0f, 0x0b, 0x4a, 0x27, 0xcc, 0xf7, 0x8f, 0x58, 0x78,
		0xa8, 0xa2, 0xac, 0xe3, 0xe8, 0xd2, 0x26, 0x6f, 0x62, 0x12, 0x71, 0x74, 0x13, 0x16, 0x3c, 0xd6,
		0xc4, 0x34, 0x28, 0x5a, 0x3b, 0x56, 0x65, 0xd9, 0x56, 0x4f, 0xe8, 0x29, 0x2c, 0x27, 0x1f, 0x73,
		0x92, 0xaf, 0x15, 0x67, 0x76, 0xac, 0xca, 0xca, 0xee, 0x9d, 0xaa, 0x86, 0x92, 0x6a, 0x02, 0xf6,
		0x19, 0x8d, 0xb8, 0xbd, 0xc4, 0xd5, 0x5f, 0xa8, 0x04, 0x4b, 0xd4, 0x23, 0x01, 0xa7, 0xbc, 0x53,
		0x9c, 0x15, 0xa8, 0xd9, 0x33, 0x7a, 0x08, 0x1b, 0x67, 0x34, 0xc0, 0x61, 0xc7, 0x71, 0x1b, 0xc4,
		0xbd, 0x8c, 0xe2, 0x66, 0x71, 0x4e, 0x98, 0xac, 0xcb, 0xe1, 0x03, 0x35, 0x5a, 0xfe, 0xcf, 0x22,
		0xdc, 0xd2, 0xc6, 0x1d, 0xb5, 0x58, 0x10, 0x11, 0x74, 0x07, 0x40, 0x04, 0xc8, 0xd9, 0x25, 0x91,
		0xc1, 0xaf, 0xda, 0x22, 0xe4, 0x7a, 0x32, 0x80, 0xbe, 0x04, 0x94, 0x26, 0xc2, 0x21, 0x6f, 0x89,
		0x1b, 0x27, 0x55, 0xa2, 0x26, 0xf2, 0x40, 0x3b, 0x91, 0xaf, 0x94, 0xf9, 0x8b, 0xd4, 0xda, 0xde,
		0xbc, 0xea, 0x1f, 0x42, 0x47, 0xb0, 0x96, 0xc1, 0xf2, 0x4e, 0x8b, 0x88, 0xf9, 0xad, 0xec, 0xde,
		0x33, 0x22, 0xd6, 0x3b, 0x2d, 0x62, 0xaf, 0x5e, 0xe5, 0x9e, 0xd0, 0x29, 0x6c, 0xb5, 0x42, 0xd2,
		0xa6, 0x2c, 0x8e, 0x9c, 0x88, 0xe3, 0x90, 0x13, 0xcf, 0x21, 0x6d, 0x12, 0x70, 0x87, 0x7a, 0x22,
		0x21, 0x2b, 0xbb, 0xb7, 0xaa, 0xb2, 0x56, 0xab, 0x69, 0xad, 0x56, 0x8f, 0x03, 0xfe, 0xf8, 0xa3,
		0x53, 0xec, 0xc7, 0xc4, 0xbe, 0x99, 0x7a, 0xbf, 0x92, 0xce, 0x2f, 0x12, 0xdf, 0x63, 0x0f, 0x55,
		0xa0, 0x30, 0x00, 0x37, 0xbf, 0x63, 0x55, 0x66, 0xed, 0xf5, 0xa8, 0xd7, 0xb2, 0x08, 0x8b, 0x98,
		0x73, 0xd2, 0x6c, 0xf1, 0xe2, 0x82, 0x30, 0x48, 0x1f, 0xd1, 0x07, 0x80, 0xce, 0xb0, 0x7b, 0xe9,
		0xb3, 0x0b, 0xc7, 0x65, 0x71, 0xc0, 0x9d, 0x06, 0x0d, 0x78, 0x71, 0x51, 0x18, 0x15, 0xd4, 0x9b,
		0x83, 0xe4, 0xc5, 0x4b, 0x1a, 0x70, 0xf4, 0x18, 0x16, 0x55, 0x65, 0x17, 0x97, 0x44, 0xdc, 0xb7,
		0xb5, 0xb9, 0x78, 0x29, 0x6d, 0xec, 0xd4, 0x18, 0x3d, 0x80, 0x8d, 0x80, 0xbc, 0xe5, 0x4e, 0x0b,
		0x5f, 0x10, 0x45, 0xe2, 0xb2, 0x20, 0x71, 0x2d, 0x19, 0x3e, 0xc1, 0x17, 0x44, 0x12, 0xf9, 0x04,
		0xe6, 0xc5, 0xb2, 0x28, 0x82, 0x40, 0x2f, 0x1b, 0x33, 0xfd, 0x45, 0x62, 0x69, 0x4b, 0x07, 0xf4,
		0x1a, 0x6e, 0x0f, 0x96, 0x80, 0xd3, 0xad, 0xea, 0x95, 0x71, 0xaa, 0x7a, 0x6b, 0xa0, 0x06, 0xd2,
		0x57, 0x68, 0x0f, 0xd6, 0x23, 0xb7, 0x41, 0xbc, 0xd8, 0x27, 0x9e, 0x93, 0x34, 0x9a, 0xe2, 0xaa,
		0x40, 0x2c, 0x0d, 0x10, 0x57, 0x4f, 0xbb, 0x90, 0xbd, 0x96, 0x79, 0x24, 0x63, 0xe8, 0x39, 0xac,
		0xa6, 0x74, 0x09, 0x80, 0xb5, 0x91, 0x00, 0x2b, 0xca, 0x5e, 0xb8, 0x7f, 0x05, 0x8b, 0xc9, 0x54,
		0x29, 0x89, 0x8a, 0xeb, 0x3b, 0xb3, 0x95, 0x95, 0xdd, 0xe7, 0xda, 0xc9, 0x18, 0x96, 0x51, 0xf5,
		0x0b, 0xe9, 0xff, 0x22, 0xe0, 0x09, 0x39, 0x0a, 0x0d, 0x95, 0x41, 0xb0, 0xd0, 0xad, 0xa1, 0x0d,
		0xc1, 0xfe, 0x4a, 0x32, 0xa8, 0x0a, 0xa8, 0xf4, 0x1a, 0x56, 0xf3, 0xce, 0xa8, 0x00, 0xb3, 0x97,
		0xa4, 0xa3, 0xda, 0x48, 0xf2, 0x67, 0x42, 0x5d, 0x3b, 0xa9, 0x56, 0xb5, 0xec, 0xc6, 0xa2, 0x4e,
		0x38, 0x3c, 0x9d, 0x79, 0x62, 0x95, 0xff, 0x3e, 0x0f, 0xef, 0xc9, 0x30, 0xbd, 0x7c, 0xe4, 0x07,
		0xac, 0xd9, 0xf2, 0x09, 0x27, 0x5e, 0xda, 0xc1, 0x46, 0x34, 0x82, 0x67, 0xb0, 0x9c, 0x76, 0xe7,
		0xa8, 0x38, 0x23, 0xb2, 0xa4, 0xa7, 0x3c, 0xfd, 0x88, 0xdd, 0xb5, 0x47, 0x3f, 0x81, 0xcd, 0x6e,
		0xe5, 0xb8, 0x2c, 0xe0, 0xe4, 0x2d, 0x17, 0x4b, 0x7e, 0xd5, 0x2e, 0x64, 0x2f, 0x0e, 0xe4, 0x78,
		0x4f, 0xdb, 0x9b, 0xeb, 0x6b, 0x7b, 0xbf, 0x81, 0xcd, 0x88, 0x53, 0xf7, 0xb2, 0xe3, 0x60, 0xce,
		0x43, 0x7a, 0x16, 0x73, 0x12, 0x89, 0x85, 0xb9, 0xb2, 0x5b, 0xd5, 0x46, 0xf3, 0x4a, 0x58, 0x67,
		0x45, 0xb7, 0x97, 0x79, 0xd9, 0x05, 0x09, 0xd4, 0x1d, 0x41, 0x1f, 0x43, 0x31, 0x24, 0x3c, 0x0e,
		0x03, 0x27, 0x20, 0x57, 0x4e, 0x1a, 0xbd, 0xa8, 0x74, 0xb1, 0xb6, 0x97, 0xec, 0x1f, 0xc8, 0xf7,
		0x9f, 0x93, 0xab, 0x7c, 0x2a, 0xd1, 0x3e, 0x6c, 0x9f, 0xb3, 0xd0, 0x25, 0x8e, 0x1b, 0x12, 0xcc,
		0x89, 0xc6, 0x7d, 0x51, 0xb8, 0x97, 0x84, 0xd5, 0x81, 0x30, 0xea, 0xc7, 0xd0, 0x34, 0xf4, 0x25,
		0x5d, 0x43, 0x47, 0x0c, 0xd6, 0xc4, 0xba, 0x74, 0x42, 0x12, 0xc5, 0x3e, 0x8f, 0x8a, 0xcb, 0x82,
		0x8c, 0x5f, 0x6a, 0xa7, 0x3f, 0x06, 0xf1, 0x55, 0x59, 0x31, 0x12, 0x4c, 0xd6, 0xef, 0xea, 0x9b,
		0xdc, 0x50, 0x89, 0xc2, 0xe6, 0x80, 0x89, 0xa6, 0x4a, 0x7f, 0xde, 0x5b, 0xa5, 0x95, 0x31, 0xaa,
		0x54, 0x00, 0xe6, 0x6b, 0xf5, 0xdd, 0x2c, 0xfc, 0xc8, 0x1c, 0xb2, 0xda, 0xb5, 0xbe, 0x84, 0xb5,
		0xde, 0x04, 0x5b, 0xe2, 0xa3, 0x3f, 0x9d, 0x74, 0xdd, 0xda, 0xab, 0x5e, 0x9e, 0x84, 0x77, 0x16,
		0x6c, 0x63, 0x97, 0xd3, 0x36, 0xe5, 0x94, 0x44, 0x0e, 0x67, 0x8e, 0x47, 0xa3, 0x16, 0xe6, 0x6e,
		0xc3, 0xf1, 0x99, 0x8b, 0x7d, 0xbf, 0xa3, 0x4a, 0xff, 0xeb, 0x29, 0xb2, 0xad, 0x3a, 0xc5, 0x5e,
		0x86, 0x5f, 0x67, 0x87, 0x0a, 0xfd, 0x33, 0x09, 0x2e, 0xb3, 0x7f, 0x0b, 0x0f, 0xb7, 0x28, 0xfd,
		0x16, 0x76, 0x46, 0x01, 0x68, 0xb8, 0x39, 0xec, 0xe5, 0x46, 0xbf, 0x54, 0x14, 0x6e, 0x47, 0x60,
		0xa5, 0xc0, 0xc7, 0xc1, 0x39, 0xcb, 0x33, 0xf4, 0xbb, 0x19, 0xd8, 0xd1, 0x4c, 0xf3, 0x08, 0x53,
		0x7f, 0xec, 0x56, 0xb2, 0x0f, 0xf3, 0x2e, 0x8e, 0x23, 0x19, 0xcd, 0xfa, 0xee, 0x07, 0xc6, 0x36,
		0xd2, 0x45, 0x3f, 0x48, 0x7c, 0x6c, 0xe9, 0x9a, 0x6c, 0x97, 0x1e, 0xe1, 0x98, 0xfa, 0x91, 0x92,
		0x0e, 0xfa, 0xed, 0xf2, 0x04, 0x77, 0x7c, 0x86, 0x3d, 0x3b, 0x35, 0x36, 0x36, 0x17, 0xcd, 0x12,
		0x9c, 0xd7, 0x6a, 0xaa, 0xf7, 0xe0, 0x9e, 0x21, 0x07, 0x92, 0xe7, 0xf2, 0xbf, 0xbb, 0x82, 0x31,
		0xcd, 0xec, 0x75, 0x0a, 0xc6, 0x57, 0x80, 0x32, 0x5c, 0xa7, 0x49, 0x38, 0xf6, 0x30, 0xc7, 0x4a,
		0x22, 0xdd, 0x37, 0x7e, 0xe0, 0x57, 0xca, 0xd8, 0x2e, 0xf0, 0xbe, 0x91, 0xf2, 0xb7, 0x4b, 0x99,
		0xb8, 0xec, 0x9d, 0xe3, 0xb5, 0x8a, 0xcb, 0xbb, 0xb0, 0xa2, 0x96, 0x50, 0x27, 0xd9, 0x73, 0x65,
		0x26, 0x20, 0x1d, 0x3a, 0xf6, 0x12, 0xf5, 0x99, 0x19, 0x08, 0xf5, 0x39, 0x67, 0x50, 0x9f, 0xd9,
		0xc4, 0x84, 0xfa, 0xc4, 0xb9, 0x27, 0xb4, 0x0b, 0xf3, 0x34, 0x68, 0xc5, 0x5c, 0xed, 0x40, 0xe6,
		0x12, 0x94, 0xa6, 0x1a, 0xb5, 0xb3, 0xf0, 0xbf, 0xaa, 0x9d, 0xc5, 0xc9, 0xd4, 0x4e, 0x1d, 0xb6,
		0x52, 0xbc, 0xa4, 0xc3, 0xb9, 0x3e, 0x8b, 0x88, 0x00, 0x62, 0x31, 0x57, 0xda, 0x73, 0x6b, 0x00,
		0xeb, 0x50, 0x1d, 0x10, 0xed, 0x9b, 0xa9, 0x6f, 0x9d, 0x1d, 0x24, 0x9e, 0x75, 0xe9, 0x88, 0x3e,
		0x87, 0x9b, 0xe2, 0x23, 0x83, 0x90, 0xcb, 0xa3, 0x20, 0x6f, 0x08, 0xc7, 0x3e, 0xbc, 0x23, 0xd8,
		0x6c, 0x10, 0x1c, 0xf2, 0x33, 0x82, 0x79, 0x06, 0x05, 0xa3, 0xa0, 0x0a, 0x99, 0x4f, 0x8a, 0x93,
		0xd3, 0xe7, 0x89, 0x50, 0x9d, 0xef, 0xea, 0xf3, 0xd7, 0xb0, 0xdd, 0xcb, 0x84, 0xc3, 0xce, 0x1d,
		0xde, 0xa0, 0x91, 0x93, 0x3a, 0x8c, 0xd6, 0xa1, 0xa5, 0x1e, 0x66, 0x7e, 0x7d, 0x5e, 0x6f, 0xd0,
		0x68, 0x4f, 0xe1, 0x1f, 0xe7, 0x67, 0x90, 0x36, 0xab, 0xb5, 0x31, 0x2a, 0xa5, 0x3b, 0x89, 0x43,
		0xd5, 0xb5, 0x06, 0x8e, 0x4b, 0xeb, 0xd3, 0x1d, 0x97, 0x1e, 0xc2, 0x46, 0x86, 0xa3, 0xba, 0xcf,
		0x86, 0xec, 0x70, 0xe9, 0xf0, 0xa1, 0xec, 0x42, 0x1f, 0xc2, 0x42, 0x83, 0x60, 0x8f, 0x84, 0xc5,
		0x82, 0x3a, 0x44, 0x69, 0x0f, 0x23, 0xc2, 0xc4, 0x56, 0xa6, 0xe5, 0xef, 0xac, 0x4c, 0x69, 0xe6,
		0xbb, 0xc1, 0xa4, 0x4a, 0xf3, 0x23, 0x58, 0x90, 0xd2, 0x46, 0x75, 0x02, 0x73, 0xb2, 0x94, 0xad,
		0xa9, 0xf7, 0x95, 0x1f, 0x64, 0xaa, 0x62, 0x48, 0x5c, 0xaa, 0x65, 0x7f, 0x3b, 0x03, 0x0f, 0x4d,
		0x86, 0xfb, 0x9d, 0xe3, 0xc3, 0x51, 0xfd, 0xfb, 0xba, 0x7a, 0x5a, 0x37, 0x6b, 0x73, 0x53, 0x66,
		0x6d, 0xbe, 0x2f, 0x6b, 0x3f, 0x86, 0xca, 0xe8, 0x64, 0xa8, 0xcc, 0xfd, 0xd9, 0xca, 0x64, 0x41,
		0xde, 0x78, 0x22, 0x59, 0xf0, 0x18, 0x16, 0xcf, 0x31, 0xf5, 0xe3, 0x90, 0x18, 0x89, 0x3f, 0x92,
		0x36, 0x76, 0x6a, 0x6c, 0x64, 0xbe, 0xbb, 0x53, 0xeb, 0xc2, 0x52, 0xc1, 0x7f, 0x33, 0xa3, 0xad,
		0x0f, 0x69, 0xf5, 0x7d, 0xe6, 0x3c, 0x97, 0xb1, 0xb9, 0x69, 0x33, 0xd6, 0xcf, 0xfa, 0x43, 0xb8,
		0x3f, 0x22, 0x17, 0x2a, 0x6b, 0x7f, 0xb1, 0xa0, 0xac, 0xab, 0x0f, 0x1c, 0xb8, 0x64, 0x22, 0xd2,
		0xd3, 0xd6, 0x38, 0x33, 0xad, 0x8e, 0xeb, 0x27, 0xfd, 0xbe, 0xbe, 0x0d, 0x65, 0x81, 0xa9, 0x09,
		0xfc, 0x7e, 0x06, 0x1e, 0x18, 0xec, 0xbe, 0xe7, 0xc4, 0xa7, 0x59, 0x9b, 0x9b, 0x36, 0x6b, 0xfd,
		0xc4, 0xbf, 0xaf, 0xef, 0x7d, 0x3d, 0xd9, 0xe8, 0xa1, 0xde, 0x65, 0x61, 0x8f, 0xe9, 0xcb, 0x74,
		0xd7, 0xba, 0x46, 0xea, 0x4f, 0x12, 0xea, 0x0d, 0x81, 0x29, 0x5d, 0xfa, 0x3e, 0x14, 0x5c, 0x31,
		0x31, 0x27, 0x94, 0xb1, 0x12, 0x4f, 0xc4, 0xb7, 0x64, 0x6f, 0xc8, 0x71, 0x3b, 0x1d, 0x56, 0x55,
		0x32, 0x14, 0xf2, 0xff, 0xad, 0x4a, 0xea, 0x49, 0x95, 0x8c, 0xc8, 0xc6, 0xe4, 0x49, 0xfe, 0x57,
		0x77, 0xfb, 0x10, 0x37, 0x03, 0xd3, 0xc8, 0x86, 0x5f, 0xf4, 0xc9, 0x86, 0xf1, 0x2f, 0x20, 0xd2,
		0xcd, 0xf0, 0x14, 0x6e, 0xc8, 0x7f, 0x9d, 0x38, 0x6d, 0x12, 0x8a, 0xab, 0x05, 0x1a, 0x9c, 0x33,
		0x75, 0xbe, 0x1c, 0x4e, 0x14, 0x09, 0x4f, 0xa5, 0xb9, 0x38, 0x2b, 0x6f, 0x5e, 0xf5, 0x0f, 0xe5,
		0x36, 0x21, 0xdd, 0xe4, 0xd4, 0x9a, 0xfa, 0xa7, 0x95, 0x34, 0x5e, 0x31, 0x53, 0xb9, 0xe6, 0x06,
		0x6b, 0xe0, 0x7a, 0xca, 0xcc, 0x74, 0xa8, 0xbc, 0x03, 0xa0, 0xb8, 0x4d, 0xef, 0xdb, 0x97, 0xed,
		0x65, 0x35, 0x72, 0xec, 0x95, 0x2b, 0xc9, 0xd2, 0x31, 0x4f, 0x49, 0xce, 0x7e, 0xf7, 0x0f, 0x05,
		0x58, 0x96, 0xb9, 0xdc, 0x3b, 0x39, 0x46, 0x6f, 0xe1, 0x86, 0xe6, 0xce, 0x06, 0xd5, 0xc6, 0xbf,
		0xdd, 0x11, 0x1f, 0x2d, 0x4d, 0x7c, 0x1d, 0x84, 0xfe, 0x64, 0xc1, 0x6d, 0xd3, 0x2d, 0x0e, 0x7a,
		0x32, 0xed, 0x35, 0x5b, 0xe9, 0x93, 0xa9, 0xaf, 0x8c, 0xd0, 0x37, 0x16, 0x6c, 0x0d, 0xbd, 0x70,
		0x40, 0x3f, 0x1b, 0x17, 0xb8, 0x47, 0x8d, 0x95, 0x1e, 0x4f, 0xea, 0xa6, 0x82, 0xe9, 0x92, 0x93,
		0x6f, 0x01, 0x66, 0x72, 0x34, 0x17, 0x20, 0x66, 0x72, 0xb4, 0xb7, 0x09, 0x39, 0x72, 0xb4, 0x8a,
		0xd4, 0x4c, 0x8e, 0xe9, 0x48, 0x62, 0x26, 0xc7, 0x78, 0x68, 0x40, 0xef, 0xf4, 0xd2, 0xb7, 0x47,
		0x27, 0xa3, 0x4f, 0x27, 0xc6, 0xcf, 0x6d, 0x2c, 0xa5, 0xe7, 0x53, 0x7a, 0x0f, 0x96, 0xcf, 0xa0,
		0xa6, 0x33, 0x97, 0xcf, 0x50, 0x31, 0x6f, 0x2e, 0x9f, 0xe1, 0x62, 0x1b, 0x7d, 0x67, 0xc1, 0x1d,
		0xa3, 0xc0, 0x44, 0x9f, 0x4c, 0x86, 0x9c, 0x4f, 0xd4, 0xd3, 0x69, 0x5c, 0x55, 0x60, 0x7f, 0xb4,
		0xe0, 0x96, 0x41, 0x00, 0xa1, 0x8f, 0xc7, 0x26, 0xa1, 0x57, 0x01, 0x97, 0x9e, 0x4c, 0xee, 0xa8,
		0x42, 0xfa, 0x9b, 0x05, 0x77, 0x47, 0x68, 0x32, 0xf4, 0x6c, 0x52, 0xf4, 0x7c, 0xbe, 0x3e, 0x9d,
		0xce, 0xb9, 0x27, 0x63, 0x43, 0xc5, 0xc0, 0xd0, 0x8c, 0x8d, 0x12, 0x8e, 0x43, 0x33, 0x36, 0x5a,
		0xd8, 0xc9, 0x8c, 0x19, 0xf5, 0xc9, 0xd0, 0x8c, 0x8d, 0xa3, 0xf1, 0x86, 0x66, 0x6c, 0x3c, 0x49,
		0x94, 0x5b, 0x89, 0x83, 0x52, 0xc0, 0xbc, 0x12, 0x87, 0xea, 0x22, 0xf3, 0x4a, 0x1c, 0xae, 0x38,
		0xd0, 0x5f, 0x2d, 0xd8, 0x36, 0x6f, 0xcf, 0x68, 0xd8, 0x7a, 0x1a, 0x43, 0xa6, 0x94, 0x9e, 0x4d,
		0xe5, 0x2b, 0x63, 0xdb, 0x3f, 0x85, 0x1f, 0xba, 0xac, 0xa9, 0x43, 0xd8, 0x5f, 0xda, 0x6b, 0xd1,
		0x93, 0x90, 0x71, 0x76, 0x62, 0x7d, 0x5d, 0xbb, 0xa0, 0xbc, 0x11, 0x9f, 0x55, 0x5d, 0xd6, 0xac,
		0xf5, 0xfc, 0x86, 0xa3, 0x7a, 0x41, 0x02, 0xf9, 0xe3, 0x14, 0xf5, 0x73, 0x8e, 0x67, 0xb8, 0x45,
		0xdb, 0x8f, 0xce, 0x16, 0xc4, 0xd8, 0x87, 0xff, 0x0d, 0x00, 0x00, 0xff, 0xff, 0x0a, 0xdf, 0xc3,
		0x94, 0x40, 0x23, 0x00, 0x00,
	},
	// uber/cadence/api/v1/decision.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xbc, 0x59, 0x5d, 0x4f, 0xdc, 0x46,
		0x17, 0x7e, 0xcd, 0xc7, 0x7e, 0x1c, 0x20, 0x6f, 0x18, 0x12, 0x02, 0x09, 0x09, 0x64, 0x5f, 0xbd,
		0xa4, 0x09, 0x62, 0x17, 0x48, 0x1a, 0x45, 0x49, 0x15, 0x15, 0x48, 0x50, 0x90, 0x12, 0x82, 0x1c,
		0x92, 0x48, 0xbd, 0xb1, 0x86, 0xf1, 0x00, 0x23, 0xbc, 0xb6, 0x3b, 0x1e, 0x43, 0xb6, 0x52, 0xa5,
		0x5e, 0xb5, 0xbd, 0xe9, 0x0f, 0xa8, 0xd4, 0xab, 0x5e, 0xb5, 0x37, 0xed, 0x6d, 0xab, 0x5e, 0xf5,
		0x27, 0xf4, 0xa2, 0xff, 0xa3, 0x52, 0xff, 0x40, 0x35, 0xe3, 0xb1, 0x77, 0x59, 0xbc, 0x5e, 0x9b,
		0xa4, 0xb9, 0xc3, 0xe3, 0x73, 0x9e, 0x79, 0x66, 0xce, 0xcc, 0x79, 0x1e, 0xb3, 0x50, 0x0b, 0x77,
		0x29, 0x6f, 0x10, 0x6c, 0x53, 0x97, 0xd0, 0x06, 0xf6, 0x59, 0xe3, 0x68, 0xb9, 0x61, 0x53, 0xc2,
		0x02, 0xe6, 0xb9, 0x75, 0x9f, 0x7b, 0xc2, 0x43, 0x13, 0x32, 0xa6, 0xae, 0x63, 0xea, 0xd8, 0x67,
		0xf5, 0xa3, 0xe5, 0xcb, 0xd7, 0xf6, 0x3d, 0x6f, 0xdf, 0xa1, 0x0d, 0x15, 0xb2, 0x1b, 0xee, 0x35,
		0xec, 0x90, 0x63, 0x91, 0x24, 0x5d, 0x9e, 0x4b, 0x03, 0x26, 0x5e, 0xb3, 0x99, 0x44, 0xa4, 0x4e,
		0x2d, 0x70, 0x70, 0xe8, 0xb0, 0x40, 0x64, 0xc5, 0x1c, 0x7b, 0xfc, 0x70, 0xcf, 0xf1, 0x8e, 0xa3,
		0x98, 0xda, 0xd7, 0xe3, 0x50, 0x79, 0xa4, 0x19, 0xa3, 0x6f, 0x0d, 0xb8, 0x15, 0x90, 0x03, 0x6a,
		0x87, 0x0e, 0xb5, 0x30, 0x11, 0xec, 0x88, 0x89, 0x96, 0x25, 0x51, 0xad, 0x78, 0x55, 0x16, 0x16,
		0x82, 0xb3, 0xdd, 0x50, 0xd0, 0x60, 0xca, 0x98, 0x33, 0x3e, 0x18, 0x59, 0x79, 0x50, 0x4f, 0x59,
		0x61, 0xfd, 0x85, 0x86, 0x59, 0xd5, 0x28, 0x3b, 0x38, 0x38, 0x8c, 0xe7, 0x59, 0x4d, 0x20, 0x9e,
		0xfc, 0xc7, 0x9c, 0x0f, 0x72, 0x45, 0xa2, 0xcf, 0x60, 0x36, 0x10, 0x98, 0x0b, 0x4b, 0xb0, 0x26,
		0xe5, 0xa9, 0x7c, 0x06, 0x14, 0x9f, 0xe5, 0x74, 0x3e, 0x32, 0x77, 0x47, 0xa6, 0xa6, 0xb2, 0x98,
		0x09, 0x32, 0xde, 0xa3, 0x1f, 0x0c, 0x90, 0xbb, 0xef, 0x3b, 0x54, 0x50, 0x2b, 0xde, 0x40, 0x8b,
		0xbe, 0xa1, 0x24, 0x94, 0x45, 0x4b, 0x25, 0x33, 0xa8, 0xc8, 0x7c, 0x9c, 0x4a, 0x66, 0x5d, 0x63,
		0xbd, 0xd6, 0x50, 0x8f, 0x63, 0xa4, 0x54, 0x6e, 0x0b, 0x24, 0x7f, 0x38, 0xfa, 0xce, 0x80, 0x85,
		0x3d, 0xcc, 0x9c, 0xbc, 0x34, 0x87, 0x14, 0xcd, 0x8f, 0x52, 0x69, 0x6e, 0x60, 0xe6, 0xe4, 0xa3,
		0x78, 0x63, 0x2f, 0x5f, 0x28, 0xfa, 0xd1, 0x80, 0x25, 0x4e, 0x3f, 0x0d, 0x69, 0x20, 0x2c, 0x82,
		0x5d, 0x42, 0x9d, 0x1c, 0xe7, 0x6c, 0x38, 0x63, 0x2b, 0xcd, 0x08, 0x6c, 0x5d, 0x61, 0xf5, 0x3d,
		0x6c, 0x0b, 0x3c, 0x7f, 0x38, 0xfa, 0x1c, 0xe6, 0x34, 0xc5, 0xde, 0x47, 0xae, 0xa4, 0xa8, 0xad,
		0xa4, 0x57, 0x59, 0x25, 0xf7, 0x3e, 0x73, 0x57, 0x49, 0x56, 0x00, 0xfa, 0xde, 0x80, 0x45, 0x3d,
		0x7f, 0xce, 0x5a, 0x96, 0x15, 0x99, 0x87, 0x19, 0x64, 0xf2, 0x55, 0xf3, 0x26, 0xc9, 0x1b, 0x8c,
		0xfe, 0x30, 0xe0, 0x61, 0x57, 0x3d, 0xe9, 0x1b, 0x41, 0xb9, 0x8b, 0x73, 0xb3, 0xae, 0x28, 0xd6,
		0xcf, 0xfa, 0x57, 0xf7, 0xb1, 0x06, 0xce, 0xb7, 0x88, 0x7b, 0xfc, 0x8c, 0xb9, 0xe8, 0x0b, 0x03,
		0xae, 0x73, 0x4a, 0x3c, 0x6e, 0x5b, 0x4d, 0xcc, 0x0f, 0x7b, 0x54, 0xbe, 0xaa, 0x68, 0xdf, 0xee,
		0x41, 0x5b, 0x66, 0x3f, 0x53, 0xc9, 0xa9, 0xe4, 0xae, 0xf1, 0xcc, 0x08, 0xf4, 0xab, 0x01, 0x77,
		0x89, 0xe7, 0x0a, 0xe6, 0x86, 0xd4, 0xc2, 0x81, 0xe5, 0xd2, 0xe3, 0xbc, 0xdb, 0x09, 0x8a, 0xd7,
		0xe3, 0x1e, 0x7d, 0x27, 0x82, 0x5c, 0x0d, 0xb6, 0xe8, 0x71, 0xbe, 0x6d, 0x5c, 0x22, 0x05, 0x73,
		0xd0, 0xcf, 0x06, 0xac, 0x44, 0x9d, 0x9a, 0x1c, 0x30, 0xc7, 0xce, 0xcb, 0x7b, 0x44, 0xf1, 0x5e,
		0xeb, 0xdd, 0xbc, 0xd7, 0x25, 0x5a, 0x3e, 0xd2, 0x8b, 0x41, 0x91, 0x04, 0xf4, 0x9b, 0x01, 0x77,
		0x03, 0xb6, 0x2f, 0xcf, 0x6c, 0xd1, 0xc3, 0x3b, 0xaa, 0x58, 0x6f, 0xa4, 0xb3, 0x56, 0x90, 0xc5,
		0x4e, 0xed, 0x72, 0x50, 0x34, 0x09, 0xfd, 0x62, 0xc0, 0x87, 0xa1, 0x1f, 0x50, 0x2e, 0xda, 0xa4,
		0x03, 0x8a, 0x39, 0x39, 0xe8, 0x20, 0x9a, 0x4a, 0x7e, 0x2c, 0xe3, 0xa8, 0xbc, 0x54, 0x88, 0xf1,
		0xfc, 0x2f, 0x14, 0x5e, 0x7b, 0xd2, 0xf4, 0xa3, 0x12, 0x16, 0xcc, 0x59, 0x1b, 0x05, 0x68, 0xd3,
		0xa9, 0x7d, 0x53, 0x82, 0xf9, 0x7c, 0xb6, 0x01, 0xcd, 0xc2, 0x48, 0x22, 0x1b, 0xcc, 0x56, 0x46,
		0xa4, 0x6a, 0x42, 0x3c, 0xb4, 0x69, 0xa3, 0x0d, 0x18, 0x6b, 0xeb, 0x4a, 0xcb, 0xa7, 0xda, 0x1b,
		0x5c, 0x4f, 0x5d, 0x6b, 0x32, 0x59, 0xcb, 0xa7, 0xe6, 0x28, 0xee, 0x78, 0x42, 0x93, 0x50, 0xb2,
		0xbd, 0x26, 0x66, 0xae, 0xd2, 0xf3, 0xaa, 0xa9, 0x9f, 0xd0, 0x7d, 0xa8, 0x2a, 0xb9, 0x92, 0x6e,
		0x4b, 0x6b, 0xe8, 0xd5, 0x54, 0x6c, 0xb9, 0x80, 0xa7, 0x2c, 0x10, 0x66, 0x45, 0xe8, 0xbf, 0xd0,
		0x0a, 0x0c, 0x33, 0xd7, 0x0f, 0x85, 0xd6, 0xb5, 0x99, 0xd4, 0xbc, 0x6d, 0xdc, 0x72, 0x3c, 0x6c,
		0x9b, 0x51, 0x28, 0xda, 0x81, 0xe9, 0xc4, 0x98, 0x09, 0xcf, 0x22, 0x8e, 0x17, 0x50, 0x25, 0x4b,
		0x5e, 0x28, 0xb4, 0x08, 0x4d, 0xd7, 0x23, 0x53, 0x59, 0x8f, 0x4d, 0x65, 0xfd, 0x91, 0x36, 0x95,
		0xe6, 0x64, 0x9c, 0xbb, 0xe3, 0xad, 0xcb, 0xcc, 0x9d, 0x28, 0xb1, 0x1b, 0xb5, 0xed, 0xaf, 0x24,
		0x6a, 0xb9, 0x00, 0x6a, 0xe2, 0xae, 0x24, 0xea, 0x16, 0x4c, 0x6a, 0xa4, 0x6e, 0xa2, 0x95, 0x7e,
		0x90, 0x13, 0x91, 0x0d, 0x3b, 0xc9, 0x72, 0x03, 0xc6, 0x0f, 0x28, 0xe6, 0x62, 0x97, 0xe2, 0x36,
		0xbb, 0x6a, 0x3f, 0xa8, 0xf3, 0x49, 0x4e, 0x8c, 0xb3, 0x0e, 0xa3, 0x9c, 0x0a, 0xde, 0xb2, 0x7c,
		0xcf, 0x61, 0xa4, 0xa5, 0x3b, 0xce, 0x5c, 0x8f, 0x0e, 0x2e, 0x78, 0x6b, 0x5b, 0xc5, 0x99, 0x23,
		0xbc, 0xfd, 0x80, 0x6e, 0x43, 0xe9, 0x80, 0x62, 0x9b, 0x72, 0x7d, 0xf5, 0xaf, 0xa4, 0xa6, 0x3f,
		0x51, 0x21, 0xa6, 0x0e, 0x45, 0x77, 0x60, 0x32, 0x16, 0x49, 0xc7, 0x23, 0xd8, 0xb1, 0x6c, 0x16,
		0xf8, 0x58, 0x90, 0x03, 0x75, 0x05, 0x2b, 0xe6, 0x05, 0xfd, 0xf6, 0xa9, 0x7c, 0xf9, 0x48, 0xbf,
		0xab, 0x7d, 0x65, 0xc0, 0x4c, 0x96, 0x6d, 0x45, 0xd3, 0x50, 0x89, 0x9c, 0x49, 0x72, 0x05, 0xca,
		0xea, 0x79, 0xd3, 0x46, 0x4f, 0xe1, 0x62, 0x52, 0x83, 0x3d, 0xc6, 0xdb, 0x25, 0x18, 0xe8, 0xb7,
		0x6f, 0x48, 0x97, 0x60, 0x83, 0xf1, 0xb8, 0x02, 0x35, 0x02, 0x0b, 0x05, 0x2c, 0x2b, 0xba, 0x03,
		0x25, 0x4e, 0x83, 0xd0, 0x11, 0xfa, 0x0b, 0x21, 0xfb, 0x84, 0xeb, 0xd8, 0x1a, 0x86, 0x1b, 0x39,
		0x0d, 0x27, 0xba, 0x0b, 0x65, 0x69, 0x38, 0x43, 0x4e, 0x33, 0x67, 0xd8, 0x88, 0x62, 0xcc, 0x38,
		0xb8, 0xb6, 0x05, 0x0b, 0x05, 0xfc, 0x62, 0xdf, 0x2e, 0x53, 0xbb, 0x0f, 0x57, 0x33, 0x4d, 0x5e,
		0x46, 0x85, 0x6a, 0x04, 0x6e, 0xe6, 0xf6, 0x64, 0x72, 0xc1, 0x36, 0x15, 0x98, 0x39, 0x41, 0xae,
		0x2d, 0x8d, 0x83, 0x6b, 0x7f, 0x1b, 0x70, 0xef, 0xac, 0x1e, 0xaa, 0xa3, 0xf7, 0x19, 0x27, 0x7a,
		0xdf, 0x4b, 0x40, 0xa7, 0xd5, 0x51, 0x1f, 0xac, 0xf9, 0x54, 0x5e, 0xa7, 0x66, 0x33, 0xc7, 0x8f,
		0xbb, 0x87, 0xd0, 0x14, 0x94, 0xa5, 0xd7, 0xe0, 0x9e, 0xa3, 0x7a, 0xed, 0xa8, 0x19, 0x3f, 0xa2,
		0x3a, 0x4c, 0x74, 0x59, 0x09, 0xcf, 0x75, 0x5a, 0xaa, 0xed, 0x56, 0xcc, 0x71, 0xd2, 0x29, 0xf3,
		0xcf, 0x5d, 0xa7, 0x55, 0xfb, 0xc9, 0x80, 0x6b, 0xd9, 0x16, 0x4c, 0x96, 0x56, 0x7b, 0x3b, 0x17,
		0x37, 0x69, 0x5c, 0xda, 0x68, 0x68, 0x0b, 0x37, 0x69, 0xe7, 0x8e, 0x0f, 0x14, 0xd8, 0xf1, 0x8e,
		0xfe, 0x30, 0x98, 0xbb, 0x3f, 0xd4, 0xfe, 0x2a, 0xc3, 0x52, 0x51, 0x6f, 0x26, 0x25, 0x2e, 0xd9,
		0x0f, 0x25, 0x71, 0x46, 0x86, 0xc4, 0xc5, 0x80, 0x91, 0xc4, 0x1d, 0x77, 0x3c, 0x9d, 0x94, 0xb2,
		0x81, 0x33, 0x4a, 0xd9, 0x60, 0x7e, 0x29, 0xc3, 0x30, 0xd7, 0xf6, 0x54, 0x3d, 0x84, 0x62, 0xa8,
		0x5f, 0x97, 0x9a, 0x49, 0x20, 0x5e, 0xa4, 0x28, 0xc6, 0x6b, 0xb8, 0xa2, 0x96, 0xd4, 0x03, 0x7d,
		0xb8, 0x1f, 0xfa, 0x25, 0x99, 0x9d, 0x06, 0xfc, 0x1c, 0x26, 0x77, 0x31, 0x39, 0xf4, 0xf6, 0xf6,
		0x34, 0x36, 0x73, 0x05, 0xe5, 0x47, 0xd8, 0xe9, 0xaf, 0xc1, 0x17, 0x74, 0xa2, 0x82, 0xdd, 0xd4,
		0x69, 0xa7, 0x34, 0xa9, 0x7c, 0x16, 0x4d, 0xda, 0x84, 0x2a, 0x73, 0x99, 0x60, 0x58, 0x78, 0x5c,
		0x69, 0xec, 0xb9, 0x95, 0x85, 0xfe, 0xfe, 0x7f, 0x33, 0x4e, 0x31, 0xdb, 0xd9, 0x9d, 0x9d, 0xb5,
		0x5a, 0xa0, 0xb3, 0x22, 0x13, 0x26, 0x1d, 0x2c, 0xbf, 0x01, 0x23, 0x99, 0x90, 0xa5, 0xd5, 0x12,
		0x00, 0x39, 0x4e, 0xc6, 0x05, 0x99, 0xbb, 0x9e, 0xa4, 0x9a, 0x2a, 0x13, 0xfd, 0x0f, 0xc6, 0x08,
		0x97, 0x67, 0x44, 0xdb, 0x0c, 0x25, 0xd8, 0x55, 0x73, 0x54, 0x0e, 0xc6, 0x3e, 0xf1, 0x6c, 0x7a,
		0xbc, 0x08, 0x43, 0x4d, 0xda, 0xf4, 0xb4, 0x01, 0x9e, 0x4e, 0x4d, 0x79, 0x46, 0x9b, 0x9e, 0xa9,
		0xc2, 0x90, 0x09, 0xe3, 0xa7, 0x0c, 0xf5, 0xd4, 0x39, 0x95, 0xfb, 0xff, 0x74, 0xe7, 0xdf, 0x65,
		0x7d, 0xcd, 0xf3, 0x41, 0xd7, 0x48, 0xed, 0xcf, 0x32, 0x2c, 0x16, 0xfa, 0xac, 0xe9, 0xd9, 0x8e,
		0x67, 0x61, 0x24, 0xe9, 0x03, 0xcc, 0x56, 0x37, 0xb8, 0x6a, 0x42, 0x3c, 0x14, 0x79, 0xe1, 0x93,
		0x8d, 0x62, 0xf0, 0x1d, 0x34, 0x8a, 0xf7, 0xe0, 0x79, 0xf3, 0x34, 0x8a, 0xd2, 0xbf, 0xda, 0x28,
		0xca, 0x67, 0x6e, 0x14, 0xaf, 0x60, 0xc2, 0xc7, 0x9c, 0xba, 0x42, 0x23, 0xea, 0xeb, 0x1d, 0x5d,
		0xce, 0xf9, 0x1e, 0xab, 0x97, 0xf1, 0x0a, 0x45, 0x5f, 0xf2, 0x71, 0xbf, 0x7b, 0xa8, 0x53, 0x24,
		0xab, 0x27, 0x45, 0x92, 0xc0, 0x54, 0xc7, 0x31, 0xb0, 0x38, 0x0d, 0xdb, 0xd3, 0x82, 0x9a, 0xf6,
		0x56, 0x66, 0xc1, 0x37, 0x6d, 0x53, 0xa6, 0xe8, 0xa9, 0x2f, 0x1e, 0xa7, 0x0d, 0xbf, 0x1b, 0x0b,
		0x7d, 0xea, 0x5e, 0x8f, 0x66, 0xde, 0xeb, 0xb1, 0xe2, 0xf7, 0xfa, 0xdc, 0x5b, 0xdc, 0xeb, 0xff,
		0xbe, 0xdd, 0xbd, 0xfe, 0x7d, 0x00, 0x96, 0x0b, 0x7f, 0xf8, 0xbf, 0x6f, 0xab, 0x35, 0x0b, 0x23,
		0xfa, 0xff, 0x1d, 0xca, 0xfd, 0x44, 0x9f, 0xb6, 0x10, 0x0d, 0x29, 0xf7, 0x93, 0x5c, 0xd7, 0xa1,
		0xfc, 0xd7, 0xb5, 0xe3, 0x68, 0x0e, 0xe7, 0xf2, 0x6f, 0xa5, 0x5e, 0xfe, 0xed, 0x4b, 0x03, 0x96,
		0x8a, 0xfe, 0xff, 0x21, 0xbd, 0x98, 0xc6, 0x5b, 0x15, 0x73, 0xed, 0x15, 0x5c, 0x22, 0x5e, 0x33,
		0x2d, 0x7b, 0xad, 0xb2, 0xea, 0xb3, 0x6d, 0xd9, 0x0f, 0xb6, 0x8d, 0x4f, 0x1a, 0xfb, 0x4c, 0x1c,
		0x84, 0xbb, 0x75, 0xe2, 0x35, 0x1b, 0x27, 0x7e, 0x72, 0xa9, 0xef, 0x53, 0x37, 0xfa, 0x8d, 0x47,
		0xff, 0xfa, 0xf2, 0x00, 0xfb, 0xec, 0x68, 0x79, 0xb7, 0xa4, 0xc6, 0x6e, 0xff, 0x13, 0x00, 0x00,
		0xff, 0xff, 0xbc, 0x58, 0xcc, 0x6b, 0x40, 0x1a, 0x00, 0x00,
	},
	// uber/cadence/api/v1/history.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xec, 0x5c, 0x5f, 0x6c, 0x1c, 0x57,
		0xd5, 0xef, 0xec, 0xda, 0xbb, 0xde, 0xb3, 0x8e, 0x63, 0xdf, 0x24, 0x8e, 0x9d, 0xbf, 0xce, 0x24,
		0x4d, 0x5c, 0xc7, 0x59, 0x27, 0x4e, 0x9a, 0x7c, 0x69, 0xbe, 0xb6, 0x38, 0x8e, 0xad, 0xae, 0x64,
		0x92, 0x68, 0xe2, 0xa4, 0x80, 0x90, 0x96, 0xf1, 0xcc, 0x75, 0x3c, 0xf2, 0xee, 0xcc, 0x76, 0x66,
		0xd6, 0x1b, 0x23, 0xf1, 0xc4, 0x03, 0x12, 0x6a, 0x05, 0x55, 0x85, 0x44, 0x05, 0x08, 0x84, 0x04,
		0x6a, 0x11, 0x52, 0x11, 0x08, 0x01, 0xe2, 0x05, 0x90, 0x10, 0x48, 0xa0, 0xc2, 0x13, 0x2f, 0x3c,
		0xc0, 0x03, 0x0f, 0xf4, 0x8d, 0x07, 0xca, 0x1b, 0x12, 0x9a, 0x3b, 0x77, 0x76, 0x77, 0x66, 0xee,
		0x9d, 0xb9, 0xb3, 0xde, 0xb4, 0xa0, 0xe6, 0xcd, 0x73, 0xe7, 0xdc, 0x33, 0xbf, 0x7b, 0xee, 0x39,
		0xe7, 0x9e, 0x7b, 0xce, 0x59, 0xc3, 0xa9, 0xd6, 0x06, 0xb6, 0x17, 0x34, 0x55, 0xc7, 0xa6, 0x86,
		0x17, 0xd4, 0xa6, 0xb1, 0xb0, 0x73, 0x69, 0x61, 0xcb, 0x70, 0x5c, 0xcb, 0xde, 0xad, 0x34, 0x6d,
		0xcb, 0xb5, 0xd0, 0x01, 0x8f, 0xa4, 0x42, 0x49, 0x2a, 0x6a, 0xd3, 0xa8, 0xec, 0x5c, 0x3a, 0x72,
		0xe2, 0xa1, 0x65, 0x3d, 0xac, 0xe3, 0x05, 0x42, 0xb2, 0xd1, 0xda, 0x5c, 0xd0, 0x5b, 0xb6, 0xea,
		0x1a, 0x96, 0xe9, 0x4f, 0x3a, 0x72, 0x32, 0xfa, 0xde, 0x35, 0x1a, 0xd8, 0x71, 0xd5, 0x46, 0x93,
		0x12, 0xcc, 0xb0, 0x3e, 0xac, 0x59, 0x8d, 0x46, 0x87, 0x85, 0xcc, 0xa2, 0x70, 0x55, 0x67, 0xbb,
		0x6e, 0x38, 0x6e, 0x12, 0x4d, 0xdb, 0xb2, 0xb7, 0x37, 0xeb, 0x56, 0xdb, 0xa7, 0x91, 0x6f, 0x41,
		0xf1, 0x25, 0x7f, 0x41, 0xe8, 0x3a, 0x14, 0xf0, 0x0e, 0x36, 0x5d, 0x67, 0x4a, 0x9a, 0xc9, 0xcf,
		0x96, 0x17, 0x4f, 0x55, 0x18, 0x6b, 0xab, 0x50, 0xea, 0x15, 0x8f, 0x52, 0xa1, 0x13, 0xe4, 0xf7,
		0xae, 0xc1, 0x68, 0xef, 0x0b, 0x34, 0x0d, 0x23, 0xe4, 0x55, 0xcd, 0xd0, 0xa7, 0xa4, 0x19, 0x69,
		0x36, 0xaf, 0x14, 0xc9, 0x73, 0x55, 0x47, 0xd7, 0x01, 0xfc, 0x57, 0xde, 0xa2, 0xa7, 0x72, 0x33,
		0xd2, 0x6c, 0x79, 0xf1, 0x48, 0xc5, 0x97, 0x48, 0x25, 0x90, 0x48, 0x65, 0x3d, 0x90, 0x88, 0x52,
		0x22, 0xd4, 0xde, 0x33, 0x9a, 0x82, 0xe2, 0x0e, 0xb6, 0x1d, 0xc3, 0x32, 0xa7, 0xf2, 0x3e, 0x53,
		0xfa, 0x88, 0x0e, 0x43, 0xd1, 0x5b, 0xbc, 0xf7, 0xb9, 0x21, 0xf2, 0xa6, 0xe0, 0x3d, 0x56, 0x75,
		0xf4, 0x75, 0x09, 0xce, 0x07, 0x4b, 0xae, 0xe1, 0x47, 0x58, 0x6b, 0x79, 0xfb, 0x50, 0x73, 0x5c,
		0xd5, 0x76, 0xb1, 0x5e, 0xf3, 0x91, 0xa8, 0xae, 0x6b, 0x1b, 0x1b, 0x2d, 0x17, 0x3b, 0x53, 0xc3,
		0x04, 0xcf, 0xff, 0x33, 0x97, 0xfe, 0x32, 0xe5, 0xb3, 0x12, 0xb0, 0xb9, 0xe7, 0x73, 0x21, 0x4b,
		0x5e, 0xea, 0xf0, 0x78, 0xe9, 0x29, 0xe5, 0x5c, 0x5b, 0x8c, 0x14, 0x7d, 0x5b, 0x82, 0x0b, 0x0c,
		0x78, 0x9a, 0xd5, 0x68, 0xd6, 0x31, 0x13, 0x60, 0x81, 0x00, 0x7c, 0x41, 0x0c, 0xe0, 0x72, 0xc0,
		0x27, 0x0e, 0xf1, 0x99, 0xb6, 0x28, 0x31, 0x7a, 0x53, 0x82, 0x39, 0x06, 0xc8, 0x4d, 0xd5, 0xa8,
		0xb3, 0x10, 0x16, 0x09, 0xc2, 0x1b, 0x62, 0x08, 0x57, 0x09, 0x93, 0x38, 0xbc, 0xb3, 0x6d, 0x21,
		0x4a, 0xf4, 0x2d, 0xb6, 0x00, 0x3d, 0xdd, 0xd2, 0x6b, 0x56, 0xcb, 0x8d, 0xc3, 0x1b, 0x21, 0xf0,
		0x9e, 0x17, 0x83, 0xe7, 0xa9, 0x9d, 0x7e, 0xa7, 0xe5, 0xc6, 0x01, 0xce, 0xb6, 0x05, 0x69, 0xd1,
		0x1b, 0x12, 0xcc, 0xea, 0x58, 0x33, 0x1c, 0x02, 0xcc, 0xd3, 0x52, 0x47, 0xdb, 0xc2, 0x7a, 0x8b,
		0x29, 0xbc, 0x12, 0x41, 0x77, 0x9d, 0x89, 0xee, 0x16, 0x65, 0xb2, 0xae, 0x3a, 0xdb, 0xf7, 0x02,
		0x16, 0x71, 0x64, 0x67, 0x74, 0x01, 0x3a, 0xf4, 0x9a, 0x04, 0x67, 0x23, 0xa8, 0x78, 0x36, 0x01,
		0x04, 0xd3, 0xb5, 0x74, 0x4c, 0x3c, 0x73, 0x90, 0xf5, 0x54, 0x2a, 0x86, 0x94, 0x12, 0x8c, 0xa0,
		0x2c, 0x28, 0xa5, 0x04, 0xfd, 0x0f, 0x49, 0x89, 0xab, 0xfa, 0xaf, 0xc7, 0x50, 0x25, 0x68, 0xd6,
		0x28, 0x41, 0xf5, 0x7f, 0xa9, 0xa8, 0xf8, 0x4a, 0x75, 0x5a, 0x4f, 0x27, 0x43, 0x5f, 0x94, 0xe0,
		0xe9, 0x30, 0x26, 0x9e, 0x25, 0xee, 0x23, 0x80, 0xae, 0xa6, 0x02, 0xe2, 0x19, 0xe1, 0x29, 0x3d,
		0x8d, 0x88, 0x6c, 0x9b, 0xaa, 0xb9, 0xc6, 0x8e, 0xe1, 0xee, 0xa6, 0x2a, 0xf7, 0x58, 0xc2, 0xb6,
		0x2d, 0x51, 0x26, 0x69, 0xca, 0xad, 0x0a, 0xd0, 0x11, 0xe5, 0x8e, 0xa0, 0xe2, 0x29, 0xf7, 0xfe,
		0x04, 0xe5, 0x0e, 0x61, 0xe2, 0x2a, 0xb7, 0x9a, 0x4a, 0xc5, 0x90, 0x52, 0x82, 0x72, 0x8f, 0x0b,
		0x4a, 0x29, 0x49, 0xb9, 0x55, 0x01, 0x3a, 0xa2, 0x48, 0x61, 0x54, 0x3c, 0x45, 0x9a, 0x48, 0x50,
		0xa4, 0x5e, 0x48, 0x5c, 0x45, 0x52, 0xd3, 0x88, 0x88, 0xa5, 0x85, 0xc1, 0x24, 0x58, 0x1a, 0x4a,
		0xb0, 0xb4, 0x5e, 0x3c, 0x09, 0x96, 0xa6, 0xa6, 0x93, 0xa1, 0x36, 0x9c, 0xf0, 0x40, 0xd8, 0x7c,
		0xed, 0x39, 0x40, 0x80, 0x5c, 0x64, 0x02, 0xf1, 0xb8, 0xda, 0x5c, 0xb5, 0x39, 0xea, 0xf2, 0x5f,
		0xa3, 0x57, 0xe0, 0x98, 0xff, 0xe1, 0x4d, 0xc3, 0x66, 0x7d, 0xf6, 0x20, 0xf9, 0x6c, 0x85, 0xff,
		0xd9, 0x55, 0x6f, 0x5e, 0xfc, 0xa3, 0xd3, 0x2e, 0xef, 0x25, 0xfa, 0xae, 0x04, 0x0b, 0x11, 0x15,
		0x55, 0x4d, 0x0d, 0xd7, 0x6b, 0x36, 0x7e, 0xa5, 0x85, 0x1d, 0xe6, 0xea, 0x0f, 0x11, 0x18, 0x2f,
		0xa6, 0x6b, 0x2a, 0xe1, 0xa4, 0x04, 0x8c, 0xe2, 0xb8, 0xe6, 0x54, 0x61, 0x6a, 0xf4, 0x23, 0x09,
		0xae, 0x50, 0x4c, 0x01, 0x44, 0x31, 0x25, 0x9e, 0x24, 0x68, 0x97, 0x99, 0x68, 0xe9, 0xd7, 0xfc,
		0x4f, 0x8b, 0x68, 0x74, 0xc5, 0xce, 0x34, 0x03, 0x7d, 0x59, 0x82, 0x73, 0x2c, 0xf1, 0xb2, 0x80,
		0x1e, 0x16, 0xd4, 0xee, 0x65, 0xca, 0x21, 0x45, 0xbb, 0x39, 0x64, 0xe8, 0xb3, 0x70, 0xd2, 0x57,
		0x32, 0x3e, 0x92, 0x29, 0x82, 0xe4, 0x12, 0x5f, 0xcf, 0xf8, 0x10, 0x7c, 0x05, 0xe6, 0x7d, 0xfb,
		0x0b, 0x12, 0x9c, 0xa1, 0x9b, 0x47, 0x15, 0x9d, 0xb3, 0x69, 0xd3, 0x04, 0xc1, 0xb3, 0x4c, 0x04,
		0x3e, 0x73, 0x5f, 0xdf, 0x39, 0xdb, 0x34, 0xa3, 0xa5, 0xd0, 0xa0, 0xcf, 0xc1, 0x4c, 0x43, 0xb5,
		0xb7, 0xb1, 0x5d, 0xb3, 0xb1, 0x66, 0xd9, 0x3a, 0x0b, 0xc4, 0x11, 0x02, 0x62, 0x91, 0x09, 0xe2,
		0xe3, 0x64, 0xb2, 0x42, 0xe7, 0xc6, 0x11, 0x1c, 0x6f, 0x24, 0x11, 0xa0, 0x6f, 0x4a, 0x30, 0xcf,
		0xba, 0x9f, 0x18, 0x0f, 0x4d, 0x95, 0x29, 0x90, 0xa3, 0x59, 0xc2, 0xd7, 0x7b, 0x94, 0x8d, 0x48,
		0xf8, 0xca, 0xa1, 0x45, 0xdf, 0x91, 0xa0, 0xc2, 0x8a, 0xb0, 0xb1, 0xdd, 0x30, 0x4c, 0x95, 0xe9,
		0x17, 0x8e, 0x25, 0xf8, 0x85, 0x78, 0x88, 0xdd, 0x61, 0xc4, 0xf0, 0x0b, 0x6d, 0x61, 0x6a, 0xf4,
		0x63, 0x09, 0xae, 0xb0, 0xae, 0x52, 0xa9, 0x5e, 0xec, 0x38, 0x41, 0x7b, 0x4b, 0xf0, 0x46, 0x95,
		0xe6, 0xca, 0x16, 0xda, 0xd9, 0xa6, 0xf0, 0x34, 0x80, 0x6f, 0x94, 0x27, 0xb2, 0x68, 0x00, 0xdf,
		0x40, 0x67, 0xdb, 0x82, 0xb4, 0xe8, 0x6f, 0x12, 0xac, 0x44, 0x3c, 0x2e, 0x7e, 0xe4, 0x62, 0xdb,
		0x54, 0xeb, 0x35, 0x06, 0x72, 0xc3, 0x34, 0x5c, 0x83, 0xad, 0x18, 0x27, 0x09, 0xf4, 0x7b, 0xe9,
		0x2e, 0x78, 0x85, 0xf2, 0x8f, 0xad, 0xa7, 0x1a, 0x30, 0x8f, 0x2f, 0xe8, 0x05, 0x7b, 0x4f, 0x1c,
		0xd0, 0x9f, 0x25, 0xb8, 0x99, 0x61, 0x99, 0x3c, 0x8f, 0x35, 0x43, 0xd6, 0x78, 0x77, 0x0f, 0x6b,
		0xe4, 0x39, 0xb3, 0x1b, 0x76, 0xff, 0xd3, 0xd1, 0xbb, 0x12, 0x3c, 0x9f, 0xb4, 0x9c, 0x74, 0x3b,
		0x39, 0x45, 0x16, 0xb6, 0xc6, 0x5c, 0x18, 0x17, 0x4c, 0xaa, 0xbd, 0x5c, 0xc3, 0xfd, 0x4d, 0x25,
		0x71, 0x00, 0x33, 0x75, 0x62, 0xba, 0x86, 0xd9, 0xc2, 0x7a, 0x4d, 0x75, 0x6a, 0x26, 0x6e, 0xc7,
		0xd7, 0x21, 0x27, 0xc4, 0x01, 0x8c, 0x0c, 0x0a, 0x65, 0xb7, 0xe4, 0xdc, 0xc6, 0x6d, 0x46, 0x1c,
		0xd0, 0xce, 0x34, 0x03, 0xfd, 0x4a, 0x82, 0xeb, 0x24, 0x9a, 0xac, 0x69, 0x5b, 0x46, 0x5d, 0xcf,
		0x68, 0x3f, 0xa7, 0x09, 0xf4, 0x97, 0x98, 0xd0, 0x49, 0x28, 0xb9, 0xec, 0x31, 0xcd, 0x62, 0x34,
		0x97, 0x9d, 0xec, 0xd3, 0xd0, 0xcf, 0x24, 0xb8, 0x9a, 0xb2, 0x08, 0x9e, 0x75, 0x9c, 0x21, 0x2b,
		0x58, 0xc9, 0xba, 0x02, 0x9e, 0x49, 0x5c, 0x74, 0x32, 0xce, 0x41, 0xdf, 0x97, 0xe0, 0x12, 0x17,
		0x35, 0x37, 0xce, 0x7f, 0x9a, 0xc0, 0x5e, 0x62, 0x87, 0x21, 0xcc, 0xaf, 0x73, 0x03, 0xff, 0x79,
		0x2d, 0x03, 0x3d, 0xfa, 0xa1, 0x04, 0x97, 0xb9, 0x70, 0x13, 0x2e, 0x91, 0x67, 0x13, 0x94, 0x9c,
		0x0d, 0x38, 0xe1, 0x3a, 0x59, 0xd1, 0x32, 0xcd, 0x40, 0x6f, 0x4b, 0x70, 0x31, 0xb3, 0x66, 0x9c,
		0x23, 0x88, 0x3f, 0x96, 0x01, 0x31, 0x4f, 0x29, 0xce, 0x6b, 0x19, 0xf4, 0xe1, 0x1d, 0x09, 0x16,
		0xf9, 0x02, 0xe6, 0x1e, 0xc2, 0xb3, 0x04, 0xed, 0xcd, 0x2c, 0xf2, 0xe5, 0x9e, 0xc4, 0x17, 0xb4,
		0x2c, 0x13, 0xd0, 0x0f, 0x92, 0x54, 0x22, 0xe1, 0xd2, 0xfc, 0x4c, 0x66, 0xc8, 0xfc, 0xeb, 0x33,
		0x07, 0x32, 0xef, 0x22, 0xed, 0xc5, 0x66, 0x7c, 0xc8, 0x09, 0x91, 0xe4, 0x5c, 0x42, 0x6c, 0xc6,
		0xc1, 0x9c, 0x10, 0x4e, 0x2e, 0x68, 0xd9, 0xa6, 0x90, 0x43, 0xd3, 0x0f, 0xc5, 0xfb, 0x8d, 0x78,
		0xce, 0x27, 0x1c, 0x9a, 0x7e, 0xc4, 0xdd, 0x4f, 0xa8, 0x73, 0xcd, 0xe9, 0x6f, 0x2a, 0xfa, 0xb5,
		0x04, 0xcf, 0x09, 0x2c, 0x88, 0x67, 0xa3, 0xf3, 0x64, 0x35, 0xd5, 0x7e, 0x56, 0xc3, 0x33, 0xd6,
		0x2b, 0x4e, 0x1f, 0xf3, 0xd0, 0x4f, 0x25, 0x78, 0x36, 0x69, 0x01, 0xfc, 0xfb, 0xd3, 0x85, 0x84,
		0x03, 0x88, 0x0b, 0x82, 0x7f, 0x8f, 0xba, 0x88, 0x33, 0xce, 0x21, 0x0e, 0xa7, 0xd5, 0x74, 0xb0,
		0xed, 0x76, 0x81, 0x3b, 0x58, 0xb5, 0xb5, 0xad, 0x1e, 0x98, 0x71, 0xdc, 0x95, 0x04, 0xeb, 0xbd,
		0x4f, 0xd8, 0x05, 0x08, 0xee, 0x11, 0x66, 0xdd, 0x2f, 0x32, 0xac, 0xb7, 0x95, 0x65, 0xc2, 0xcd,
		0x51, 0x80, 0x2e, 0x10, 0xf9, 0xaf, 0x65, 0x38, 0x27, 0x7a, 0x7a, 0xad, 0xc2, 0xbe, 0xce, 0x1a,
		0xdd, 0xdd, 0x26, 0x26, 0xb5, 0x40, 0x5e, 0x65, 0x31, 0x60, 0xba, 0xbe, 0xdb, 0xc4, 0xca, 0x68,
		0xbb, 0xe7, 0x09, 0x7d, 0x1a, 0x0e, 0x35, 0x55, 0xdb, 0x93, 0x48, 0xaf, 0xd1, 0x6d, 0x5a, 0xb4,
		0x7c, 0x38, 0xcb, 0xe4, 0x77, 0x97, 0xcc, 0xe8, 0xb1, 0x89, 0x4d, 0x4b, 0x39, 0xd0, 0x8c, 0x0f,
		0xa2, 0xe7, 0xa0, 0x44, 0x32, 0x32, 0x75, 0xc3, 0x71, 0x49, 0x61, 0xb1, 0xbc, 0x78, 0x9c, 0x9d,
		0xf2, 0x50, 0x9d, 0xed, 0x35, 0xc3, 0x71, 0x95, 0x11, 0x97, 0xfe, 0x85, 0x16, 0x61, 0xd8, 0x30,
		0x9b, 0x2d, 0x97, 0x94, 0x1d, 0xcb, 0x8b, 0xc7, 0x38, 0x48, 0x76, 0xeb, 0x96, 0xaa, 0x2b, 0x3e,
		0x29, 0x52, 0x61, 0x26, 0x12, 0x72, 0xd4, 0x5c, 0xab, 0xa6, 0xd5, 0x2d, 0x07, 0x13, 0xff, 0x6d,
		0xb5, 0x5c, 0x5a, 0x87, 0x9c, 0x8e, 0xd5, 0x45, 0x6f, 0xd1, 0x4a, 0xb2, 0x72, 0x0c, 0x87, 0x64,
		0xbf, 0x6e, 0x2d, 0x7b, 0xf3, 0xd7, 0xfd, 0xe9, 0xe8, 0x65, 0x38, 0xda, 0x4d, 0x7b, 0xc7, 0xb9,
		0x17, 0xd2, 0xb8, 0x1f, 0x76, 0x83, 0x64, 0x76, 0x84, 0xf1, 0x0d, 0x38, 0xd2, 0x8d, 0xb0, 0xbb,
		0xab, 0xb0, 0x5b, 0x66, 0xcd, 0xd0, 0x49, 0xe9, 0xaf, 0xa4, 0x1c, 0xee, 0x50, 0x74, 0xe4, 0xac,
		0xb4, 0xcc, 0xaa, 0x8e, 0xaa, 0x50, 0xa2, 0xae, 0xd2, 0xb2, 0x49, 0x1d, 0x6e, 0x6c, 0xf1, 0x3c,
		0xdb, 0xb5, 0x53, 0x06, 0x24, 0x84, 0xae, 0x06, 0x53, 0x94, 0xee, 0x6c, 0x54, 0x85, 0x89, 0x2e,
		0x0e, 0xcf, 0x5d, 0xb5, 0x6c, 0x4c, 0x8b, 0x67, 0xec, 0x3d, 0x58, 0xf5, 0x69, 0x94, 0xf1, 0xce,
		0x34, 0x3a, 0x82, 0x14, 0x98, 0xac, 0xab, 0xde, 0x9d, 0xcf, 0x0f, 0x67, 0xc8, 0x72, 0xb0, 0xd3,
		0xaa, 0xbb, 0xb4, 0xf0, 0x95, 0xbc, 0xa7, 0x07, 0xbd, 0xb9, 0xcb, 0x9d, 0xa9, 0x0a, 0x99, 0x89,
		0xae, 0xc3, 0xb4, 0x65, 0x1b, 0x0f, 0x0d, 0xdf, 0xd1, 0x46, 0xa4, 0x54, 0x26, 0x52, 0x9a, 0x0c,
		0x08, 0x22, 0x42, 0x3a, 0x02, 0x23, 0x86, 0x8e, 0x4d, 0xd7, 0x70, 0x77, 0x49, 0x45, 0xa9, 0xa4,
		0x74, 0x9e, 0xd1, 0x65, 0x98, 0xdc, 0x34, 0x6c, 0xc7, 0x8d, 0xf3, 0xdc, 0x47, 0x28, 0x0f, 0x90,
		0xb7, 0x11, 0x86, 0xcb, 0x30, 0x6a, 0x63, 0xd7, 0xde, 0xad, 0x35, 0xad, 0xba, 0xa1, 0xed, 0xd2,
		0x2a, 0xcc, 0x0c, 0xe7, 0x82, 0xea, 0xda, 0xbb, 0x77, 0x09, 0x9d, 0x52, 0xb6, 0xbb, 0x0f, 0x68,
		0x0a, 0x8a, 0xaa, 0xeb, 0xe2, 0x46, 0xd3, 0x25, 0x15, 0x93, 0x61, 0x25, 0x78, 0x44, 0xcb, 0xb0,
		0x1f, 0x3f, 0x6a, 0x1a, 0xbe, 0xe2, 0xf8, 0x45, 0xfd, 0xf1, 0xd4, 0xa2, 0xfe, 0x58, 0x77, 0x0a,
		0xa9, 0xec, 0x9f, 0x86, 0x7d, 0x9a, 0xed, 0x59, 0x03, 0xad, 0xe8, 0x90, 0x8a, 0x43, 0x49, 0x19,
		0xf5, 0x06, 0x83, 0x2a, 0x0f, 0xfa, 0x04, 0x1c, 0xf5, 0x57, 0x1f, 0xae, 0x7e, 0x6d, 0xa8, 0xda,
		0xb6, 0xb5, 0xb9, 0x49, 0x8b, 0x02, 0x09, 0x4a, 0x3d, 0x45, 0x66, 0xf7, 0x16, 0xbe, 0x6e, 0xfa,
		0x53, 0xd1, 0x05, 0x18, 0x6a, 0xe0, 0x86, 0x45, 0xd3, 0xf9, 0xd3, 0xec, 0x44, 0x1f, 0x6e, 0x58,
		0x0a, 0x21, 0x43, 0x0a, 0x4c, 0xc4, 0x3c, 0x36, 0xcd, 0xc9, 0x3f, 0xcd, 0x3e, 0x1b, 0x23, 0x1e,
		0x56, 0x19, 0x77, 0x22, 0x23, 0xe8, 0x3e, 0x4c, 0x36, 0x6d, 0xbc, 0x53, 0x53, 0x5b, 0xae, 0xe5,
		0xe9, 0x1f, 0x76, 0x6b, 0x4d, 0xcb, 0x30, 0xdd, 0x20, 0xcb, 0xce, 0xdb, 0x2f, 0x07, 0xbb, 0x77,
		0x09, 0x9d, 0x72, 0xc0, 0x9b, 0xbf, 0xd4, 0x72, 0xad, 0x9e, 0x41, 0x74, 0x19, 0x0a, 0x5b, 0x58,
		0xd5, 0xb1, 0x4d, 0xd3, 0xdf, 0x47, 0xd9, 0x4d, 0x1d, 0x84, 0x44, 0xa1, 0xa4, 0xf2, 0xdb, 0x12,
		0x3c, 0x23, 0x1e, 0xed, 0x5f, 0x81, 0x02, 0xb5, 0x17, 0x49, 0xc0, 0x5e, 0x28, 0x2d, 0x5a, 0x85,
		0x99, 0xe4, 0x72, 0xaf, 0xa1, 0x13, 0xef, 0x9e, 0x57, 0x8e, 0xf1, 0x2b, 0xb5, 0x55, 0x5d, 0x7e,
		0x4b, 0x82, 0xb3, 0x82, 0x41, 0xc3, 0x55, 0x28, 0x06, 0x9e, 0x42, 0x12, 0xf0, 0x14, 0x01, 0xf1,
		0xc0, 0xa0, 0x5a, 0x30, 0x2b, 0x1c, 0x31, 0x2f, 0xc3, 0x28, 0x75, 0xd6, 0xdd, 0x83, 0x73, 0x8c,
		0xa3, 0x04, 0xd4, 0x37, 0x93, 0x73, 0xb3, 0xec, 0x76, 0x1f, 0xe4, 0xdf, 0x4b, 0x70, 0x46, 0xa4,
		0x69, 0x20, 0x7c, 0x02, 0x4a, 0xd9, 0x4e, 0xc0, 0xdb, 0x30, 0xc9, 0x39, 0x65, 0x72, 0x69, 0x06,
		0x79, 0xc0, 0x61, 0x9c, 0x30, 0x3d, 0x9e, 0x26, 0x1f, 0xf2, 0x34, 0xf2, 0x6b, 0x12, 0xc8, 0xe9,
		0xfd, 0x06, 0x68, 0x1e, 0x50, 0xb4, 0x06, 0xdd, 0xe9, 0x42, 0x1a, 0x77, 0x42, 0x22, 0x88, 0xb8,
		0xdb, 0x5c, 0xc4, 0xdd, 0x1e, 0x07, 0x08, 0x12, 0x82, 0x86, 0x4e, 0xd0, 0x94, 0x94, 0x12, 0x1d,
		0xa9, 0xea, 0xf2, 0x3f, 0x22, 0xe2, 0xe5, 0x5a, 0x48, 0x36, 0x44, 0xb3, 0x30, 0x1e, 0xce, 0x43,
		0x74, 0xd4, 0x6b, 0xcc, 0xe9, 0x59, 0x71, 0x04, 0x7b, 0x3e, 0x82, 0xfd, 0x1c, 0xec, 0xdf, 0x30,
		0x4c, 0xd5, 0xde, 0xad, 0x69, 0x5b, 0x58, 0xdb, 0x76, 0x5a, 0x0d, 0x12, 0xa2, 0x94, 0x94, 0x31,
		0x7f, 0x78, 0x99, 0x8e, 0xa2, 0xf3, 0x30, 0x11, 0xce, 0x9e, 0xe1, 0x47, 0x7e, 0xf8, 0x31, 0xaa,
		0x8c, 0xe3, 0xde, 0xa4, 0x16, 0x7e, 0xe4, 0xca, 0xaf, 0xe6, 0xe1, 0xb4, 0x40, 0x2b, 0xc3, 0x63,
		0x5b, 0x71, 0xd4, 0x2c, 0xf2, 0x7d, 0x98, 0x05, 0x3a, 0x01, 0xe5, 0x0d, 0xd5, 0xc1, 0xc1, 0xd1,
		0xe9, 0x8b, 0xa5, 0xe4, 0x0d, 0xf9, 0x07, 0xe6, 0x31, 0x00, 0x13, 0xb7, 0x83, 0xd7, 0xc3, 0xbe,
		0x60, 0x4d, 0xdc, 0xf6, 0xdf, 0xce, 0x03, 0xda, 0xb4, 0xec, 0x6d, 0x8a, 0x34, 0xe8, 0x47, 0x2b,
		0xf8, 0x4b, 0xf3, 0xde, 0x10, 0xac, 0x0f, 0x68, 0x63, 0xda, 0xa4, 0xe7, 0x1c, 0x55, 0xc7, 0x32,
		0x69, 0x6c, 0x44, 0x9f, 0xd0, 0x2d, 0x18, 0xd6, 0xd4, 0x96, 0x83, 0x69, 0x18, 0x54, 0x11, 0x6e,
		0x1a, 0x59, 0xf6, 0x66, 0x29, 0xfe, 0x64, 0xf9, 0xad, 0x3c, 0x9c, 0x4a, 0x6d, 0xe4, 0x78, 0x6c,
		0x9b, 0x71, 0x33, 0x58, 0x83, 0xbf, 0x0b, 0xf3, 0x82, 0x7d, 0x26, 0xbd, 0x2b, 0xe8, 0xf5, 0xc9,
		0x43, 0x59, 0x7c, 0x72, 0xaf, 0xea, 0x0f, 0x47, 0x54, 0x3f, 0xb2, 0xbf, 0x85, 0xe4, 0xfd, 0x2d,
		0x0a, 0xed, 0xef, 0x08, 0x67, 0x7f, 0x19, 0x66, 0x56, 0x62, 0x99, 0x99, 0xfc, 0x8d, 0x02, 0x9c,
		0x11, 0xe9, 0x71, 0x41, 0x27, 0xa1, 0xdc, 0x29, 0x14, 0xd3, 0x6d, 0x2a, 0x29, 0x10, 0x0c, 0x55,
		0x75, 0xef, 0x52, 0xd5, 0xad, 0x24, 0x7b, 0x46, 0x90, 0x4b, 0xb8, 0x54, 0x75, 0x3e, 0x49, 0x2e,
		0x55, 0x6a, 0xcf, 0x93, 0xa7, 0x9a, 0xba, 0xd5, 0x50, 0x0d, 0x93, 0xfa, 0x0e, 0xfa, 0x14, 0x3e,
		0x0c, 0x86, 0xfa, 0xbc, 0x0e, 0x15, 0xc4, 0xaf, 0x43, 0xeb, 0x30, 0x1d, 0x28, 0x61, 0xfc, 0x0c,
		0x29, 0xa6, 0x9d, 0x21, 0x93, 0xc1, 0xdc, 0xc8, 0x31, 0x12, 0xe1, 0x4a, 0x8f, 0x28, 0xca, 0x75,
		0x24, 0x03, 0x57, 0xff, 0x16, 0x44, 0xb9, 0xf2, 0x0f, 0xbb, 0x52, 0x5f, 0x87, 0xdd, 0x2a, 0x4c,
		0x6c, 0x61, 0xd5, 0x76, 0x37, 0xb0, 0xda, 0x45, 0x07, 0x69, 0xac, 0xc6, 0x3b, 0x73, 0xba, 0x7c,
		0xd2, 0x43, 0x94, 0x72, 0x7a, 0x88, 0x12, 0xbb, 0x2b, 0x8c, 0xf6, 0x73, 0x57, 0xe8, 0xc6, 0x9c,
		0xfb, 0xc4, 0x63, 0xce, 0xbf, 0x4b, 0x20, 0xa7, 0xf7, 0x5b, 0x7d, 0x60, 0x87, 0x7b, 0x6f, 0x18,
		0x32, 0x14, 0xbe, 0xf0, 0xbc, 0x08, 0xa3, 0xe4, 0xbe, 0x18, 0xf8, 0xad, 0x61, 0x01, 0xbf, 0x55,
		0xf6, 0x66, 0xd0, 0x07, 0xf9, 0x8f, 0x52, 0xd8, 0x15, 0x0c, 0x38, 0xb2, 0x66, 0x8b, 0x28, 0x97,
		0xc1, 0xdd, 0xe7, 0x53, 0xa3, 0x8d, 0xa1, 0xb0, 0x30, 0xe5, 0x3f, 0x48, 0x70, 0x2a, 0xbd, 0x09,
		0xa6, 0xdf, 0x00, 0xfc, 0xc3, 0x58, 0xd1, 0xcf, 0x73, 0x70, 0x5a, 0xa0, 0x95, 0xcc, 0x5b, 0x93,
		0x8e, 0x5d, 0xd5, 0xa8, 0x3b, 0x42, 0x9b, 0x14, 0x10, 0x3f, 0xb6, 0x35, 0x45, 0x23, 0xa4, 0xa1,
		0x7e, 0x22, 0xa4, 0x3d, 0xab, 0xf8, 0x57, 0x24, 0x98, 0x13, 0xef, 0x00, 0x13, 0x39, 0xf3, 0x06,
		0x73, 0x05, 0x7b, 0x47, 0x82, 0x8c, 0xbd, 0x5e, 0xe9, 0xd8, 0x0e, 0x06, 0x61, 0x90, 0xef, 0x61,
		0x68, 0x60, 0x23, 0x82, 0x38, 0x2f, 0x80, 0xf8, 0xcd, 0x88, 0x1e, 0xf2, 0xaa, 0x42, 0xfd, 0xea,
		0xe1, 0x2a, 0xcc, 0xd4, 0x55, 0xb7, 0xa7, 0xe7, 0x21, 0xda, 0x01, 0xd0, 0x95, 0xac, 0x4f, 0xc7,
		0xda, 0x4a, 0x3f, 0x6c, 0x62, 0xe8, 0x73, 0x3e, 0x83, 0x3e, 0x0f, 0xa5, 0xda, 0x68, 0x24, 0xd0,
		0x93, 0xdf, 0x95, 0xe0, 0x68, 0x42, 0x97, 0x25, 0x9a, 0x86, 0x11, 0xbf, 0xbb, 0xac, 0xb3, 0x6f,
		0x45, 0xf2, 0x5c, 0xd5, 0xd1, 0x1a, 0x1c, 0xea, 0x1c, 0xe4, 0x9b, 0x86, 0x9d, 0xe1, 0xd2, 0x8a,
		0xe8, 0x39, 0xbe, 0x6a, 0xd8, 0x38, 0xcb, 0xf1, 0x2b, 0xb2, 0xd9, 0x9f, 0x81, 0x69, 0x6e, 0xfb,
		0x66, 0xd2, 0x6a, 0x84, 0x63, 0x76, 0xf9, 0x37, 0x12, 0x1c, 0x4b, 0xea, 0xdc, 0x1b, 0xc8, 0x57,
		0x06, 0x25, 0x8f, 0x44, 0x07, 0xfd, 0x13, 0x09, 0x66, 0xd2, 0x3a, 0x00, 0x93, 0x56, 0xf3, 0x58,
		0xcd, 0x36, 0x11, 0xf9, 0xbf, 0x8b, 0x90, 0xb1, 0xd1, 0x04, 0x2d, 0xc0, 0x41, 0xd2, 0xcb, 0x12,
		0x4d, 0xfb, 0xfa, 0x6b, 0x9a, 0x30, 0x71, 0x3b, 0x92, 0xf4, 0x8d, 0x55, 0x5e, 0x72, 0xfd, 0x55,
		0x5e, 0x9e, 0xd4, 0x46, 0xc4, 0x6b, 0x23, 0x22, 0xba, 0x53, 0x14, 0xd0, 0x9d, 0x3b, 0x30, 0x49,
		0x73, 0xda, 0x14, 0xa3, 0x61, 0xba, 0xd8, 0xde, 0x51, 0xeb, 0xe9, 0xf7, 0x96, 0x83, 0x74, 0x22,
		0x81, 0x57, 0xa5, 0xd3, 0xc2, 0x75, 0x97, 0xd2, 0x9e, 0xea, 0x2e, 0x3d, 0x21, 0x1c, 0x64, 0x09,
		0xe1, 0xf8, 0x45, 0x96, 0x72, 0xdf, 0x45, 0x96, 0xee, 0x3d, 0x63, 0x54, 0xf8, 0x9e, 0xd1, 0x49,
		0xf5, 0xef, 0xdb, 0x43, 0xaa, 0x7f, 0x6c, 0x4f, 0xa9, 0x7e, 0xcf, 0x07, 0x2f, 0x64, 0xed, 0x76,
		0xeb, 0x78, 0x2b, 0xa9, 0xd7, 0x5b, 0x25, 0xdd, 0x6f, 0x36, 0xe0, 0x70, 0xa7, 0x42, 0x1e, 0xa9,
		0x9a, 0xfa, 0x76, 0x3c, 0x97, 0x58, 0x03, 0x0f, 0xd7, 0x4d, 0x0f, 0x61, 0xd6, 0xb0, 0xfc, 0x3d,
		0x89, 0x91, 0xd2, 0xe6, 0x9d, 0x2c, 0x22, 0xe6, 0x21, 0x09, 0x98, 0x47, 0x4f, 0xa4, 0x93, 0xcb,
		0x10, 0xe9, 0xc8, 0xef, 0x4b, 0x70, 0x3c, 0xb1, 0x5b, 0xdb, 0x0b, 0xf5, 0x68, 0x2f, 0xb8, 0xa9,
		0x36, 0x02, 0x51, 0x83, 0x3f, 0x74, 0x5b, 0x6d, 0xe0, 0x7e, 0x3f, 0x3d, 0xb0, 0x53, 0xa5, 0xab,
		0xf1, 0x43, 0xe2, 0x37, 0xeb, 0xaf, 0xb1, 0x36, 0x89, 0xd7, 0x9d, 0x70, 0x12, 0xca, 0xb4, 0x3f,
		0xa4, 0x57, 0x04, 0xfe, 0x10, 0x11, 0x41, 0xc7, 0xa9, 0xe7, 0xc4, 0x9d, 0x7a, 0x42, 0x9e, 0x5a,
		0xfe, 0xaa, 0x04, 0x73, 0x19, 0x3a, 0x72, 0xba, 0xf9, 0x54, 0x29, 0x94, 0x4f, 0xed, 0x77, 0x67,
		0x92, 0xa0, 0xfd, 0x32, 0x07, 0x2f, 0xec, 0xad, 0x2b, 0x79, 0x60, 0x3a, 0xdf, 0xcd, 0xd5, 0xe5,
		0x42, 0xb9, 0xba, 0xfb, 0x80, 0xe2, 0xdd, 0x2f, 0xd4, 0xbe, 0xcf, 0x8a, 0x75, 0xb8, 0x2a, 0x13,
		0xb1, 0x16, 0x56, 0x34, 0x05, 0x45, 0xcd, 0x32, 0x5d, 0xdb, 0xaa, 0x13, 0x45, 0x1b, 0x55, 0x82,
		0x47, 0x54, 0x81, 0x03, 0x91, 0x46, 0x2e, 0xcb, 0xac, 0xfb, 0x91, 0xf9, 0x88, 0x32, 0x11, 0xea,
		0xaf, 0xba, 0x63, 0xd6, 0x77, 0xe5, 0x37, 0xf2, 0x70, 0x63, 0x0f, 0x5d, 0xcf, 0xe8, 0x7e, 0xaf,
		0xdf, 0x1b, 0xe3, 0xfc, 0xa6, 0x40, 0x88, 0x73, 0x28, 0xed, 0x3c, 0xa0, 0xfb, 0x24, 0x37, 0x87,
		0xca, 0xde, 0x97, 0xa1, 0xbd, 0xee, 0xcb, 0x3c, 0xa0, 0x68, 0xaf, 0x19, 0xad, 0x50, 0xe4, 0x95,
		0x71, 0x23, 0xa4, 0x84, 0x7e, 0x0a, 0x2b, 0xd8, 0xc5, 0x42, 0x68, 0x17, 0xe5, 0x3f, 0x49, 0x70,
		0xad, 0xcf, 0x96, 0x6d, 0x0e, 0x06, 0x89, 0x83, 0xe1, 0x83, 0x55, 0x5c, 0xf9, 0x4b, 0x79, 0xb8,
		0xd6, 0x67, 0x5b, 0xdd, 0xff, 0xaa, 0xad, 0x46, 0x3c, 0xf6, 0x10, 0xdf, 0x63, 0x0f, 0x8b, 0x7b,
		0x6c, 0xae, 0xea, 0xf0, 0x1c, 0x40, 0x91, 0xe7, 0x00, 0x5e, 0xcd, 0xc3, 0x95, 0x7e, 0x5a, 0x03,
		0xc5, 0x2c, 0x5f, 0x88, 0xf3, 0x13, 0xcb, 0xef, 0x5a, 0xfe, 0x7b, 0x12, 0x5c, 0xcc, 0xda, 0xe6,
		0xf8, 0x5f, 0x6d, 0xf2, 0xfc, 0xb3, 0x4a, 0xfe, 0x9d, 0x04, 0x17, 0x32, 0xb5, 0x46, 0x0e, 0xcc,
		0x05, 0x30, 0x6f, 0x0d, 0xb9, 0xbd, 0xdd, 0x1a, 0xfe, 0x32, 0x02, 0x97, 0xfb, 0xf8, 0x8d, 0x47,
		0xcf, 0x76, 0x48, 0xa1, 0xed, 0x38, 0x09, 0xe5, 0xce, 0x76, 0x50, 0x9d, 0x2f, 0x29, 0x10, 0x0c,
		0xb1, 0x52, 0x08, 0xf9, 0x01, 0xa4, 0x10, 0xfa, 0xad, 0x27, 0x0e, 0x0f, 0x36, 0x85, 0x50, 0x78,
		0xac, 0x29, 0x84, 0x62, 0xdf, 0x29, 0x84, 0x07, 0x40, 0x3b, 0x54, 0x29, 0x47, 0x5a, 0x86, 0xf3,
		0x9b, 0x04, 0xce, 0x26, 0xb4, 0xb9, 0x12, 0x2e, 0xb4, 0x18, 0x37, 0xd1, 0x8c, 0x0e, 0xf5, 0x1a,
		0x49, 0x29, 0xec, 0xcf, 0x45, 0x54, 0x1e, 0x04, 0x54, 0x5e, 0x83, 0xa9, 0x1e, 0x75, 0xaa, 0xd9,
		0xb8, 0xd5, 0x85, 0x5f, 0x26, 0xf0, 0xe7, 0x12, 0x15, 0xa7, 0xaa, 0x2b, 0xde, 0x14, 0xba, 0x84,
		0x43, 0x6d, 0xd6, 0x70, 0xac, 0x3c, 0xb9, 0xaf, 0x9f, 0xf2, 0x64, 0xac, 0xd7, 0x70, 0x8c, 0xd1,
		0x6b, 0xd8, 0xbd, 0x69, 0xed, 0xcf, 0x9e, 0x5b, 0x18, 0xdf, 0x43, 0x6e, 0x61, 0x62, 0x6f, 0x6d,
		0x84, 0xcf, 0x41, 0x59, 0xc7, 0x75, 0x75, 0xd7, 0x57, 0xcd, 0xf4, 0x9e, 0x48, 0x20, 0xd4, 0x44,
		0x15, 0xe5, 0xd7, 0xf3, 0x70, 0x31, 0xeb, 0x6f, 0xb0, 0x3e, 0x7c, 0xf7, 0xb2, 0x16, 0xc4, 0x09,
		0x7e, 0xa5, 0xeb, 0x6a, 0xe6, 0x1f, 0x10, 0x85, 0xc2, 0x83, 0x1e, 0x43, 0x19, 0x0e, 0x1b, 0x0a,
		0xfb, 0x10, 0x2c, 0x70, 0x0e, 0xc1, 0x01, 0xe5, 0x02, 0xe5, 0xdf, 0xe6, 0x60, 0x3e, 0xcb, 0x0f,
		0xcc, 0xb8, 0xfb, 0xc1, 0x3e, 0x7d, 0x73, 0x7b, 0x3d, 0x7d, 0x07, 0xb5, 0x8b, 0x6c, 0xe9, 0x0e,
		0x71, 0xa4, 0xdb, 0xb5, 0xce, 0x61, 0xf1, 0x3c, 0xc8, 0xfb, 0x39, 0xc8, 0xf8, 0xd3, 0xb7, 0x8f,
		0x86, 0x30, 0x59, 0x65, 0x9d, 0x61, 0x66, 0x59, 0xa7, 0xdb, 0x8f, 0x50, 0x10, 0xef, 0x47, 0x90,
		0xff, 0x99, 0x83, 0xf3, 0x83, 0xf0, 0x28, 0x1f, 0x51, 0xa1, 0xf7, 0x64, 0xdc, 0x0b, 0x19, 0x32,
		0xee, 0xf2, 0xbf, 0x72, 0x70, 0x21, 0xd3, 0x2f, 0x11, 0x9f, 0x08, 0x3e, 0x26, 0xf8, 0x20, 0xa5,
		0x58, 0xc8, 0x92, 0x67, 0xfe, 0x7c, 0x9e, 0x27, 0x78, 0x5e, 0x0f, 0xc9, 0x13, 0xc1, 0x27, 0xb6,
		0xb0, 0x14, 0xfa, 0xe9, 0x7d, 0xff, 0x45, 0x0e, 0x16, 0x32, 0xfe, 0x42, 0xf4, 0xc9, 0x3e, 0x84,
		0xf6, 0x61, 0xce, 0x85, 0xfd, 0xe4, 0xcf, 0x55, 0xa3, 0xee, 0x62, 0x9b, 0x7c, 0xea, 0x38, 0x4c,
		0xaf, 0x3c, 0x58, 0xb9, 0xbd, 0x5e, 0x5b, 0xad, 0xae, 0xad, 0xaf, 0x28, 0xb5, 0xf5, 0x4f, 0xde,
		0x5d, 0xa9, 0x55, 0x6f, 0x3f, 0x58, 0x5a, 0xab, 0xde, 0x1a, 0x7f, 0x0a, 0x9d, 0x84, 0xa3, 0xf1,
		0xd7, 0x4b, 0x6b, 0x6b, 0x35, 0x32, 0x3a, 0x2e, 0xa1, 0x53, 0x70, 0x3c, 0x4e, 0xb0, 0xbc, 0x76,
		0xe7, 0xde, 0x0a, 0x25, 0xc9, 0xdd, 0x7c, 0x00, 0x87, 0x35, 0xab, 0xc1, 0x92, 0xc1, 0xcd, 0x91,
		0xa5, 0xa6, 0x71, 0xd7, 0x0b, 0x61, 0xef, 0x4a, 0x9f, 0x5a, 0x78, 0x68, 0xb8, 0x5b, 0xad, 0x8d,
		0x8a, 0x66, 0x35, 0x16, 0x42, 0xff, 0xe7, 0xb4, 0xf2, 0x10, 0x9b, 0xfe, 0x7f, 0x56, 0xa5, 0xff,
		0xf2, 0xf4, 0x86, 0xda, 0x34, 0x76, 0x2e, 0x6d, 0x14, 0xc8, 0xd8, 0xe5, 0xff, 0x04, 0x00, 0x00,
		0xff, 0xff, 0xee, 0x07, 0x38, 0xbd, 0xd5, 0x55, 0x00, 0x00,
	},
	// uber/cadence/api/v1/service_workflow.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xcc, 0x59, 0xdd, 0x6e, 0x1b, 0xc7,
		0x15, 0xc6, 0x8a, 0xfa, 0xa1, 0x0e, 0x29, 0x59, 0x1e, 0x47, 0xd2, 0x9a, 0x8a, 0x2c, 0x69, 0x9d,
		0xa6, 0xaa, 0x93, 0x50, 0xb5, 0xdc, 0xda, 0x69, 0xfa, 0x13, 0xc8, 0x72, 0xec, 0x10, 0x88, 0x03,
		0x75, 0xa5, 0xc6, 0x68, 0x6f, 0x16, 0xa3, 0xdd, 0x21, 0x39, 0xd5, 0x72, 0x67, 0x35, 0x33, 0x4b,
		0x9a, 0xb9, 0x6b, 0x11, 0xf4, 0xa2, 0x41, 0x81, 0xf6, 0x09, 0x7a, 0xd1, 0x5e, 0x17, 0x7d, 0x8a,
		0x5e, 0xf4, 0x2d, 0xfa, 0x04, 0x7d, 0x03, 0xa3, 0xd8, 0xd9, 0x59, 0x8a, 0xa4, 0x86, 0x4b, 0xc9,
		0x45, 0x60, 0xdf, 0x71, 0xce, 0x9c, 0xf3, 0xcd, 0xf9, 0x9b, 0x33, 0xe7, 0x2c, 0xe1, 0x5e, 0x72,
		0x4a, 0xf8, 0x9e, 0x8f, 0x03, 0x12, 0xf9, 0x64, 0x0f, 0xc7, 0x74, 0xaf, 0x7b, 0x7f, 0x4f, 0x10,
		0xde, 0xa5, 0x3e, 0xf1, 0x7a, 0x8c, 0x9f, 0x35, 0x43, 0xd6, 0xab, 0xc7, 0x9c, 0x49, 0x86, 0x6e,
		0xa5, 0xbc, 0x75, 0xcd, 0x5b, 0xc7, 0x31, 0xad, 0x77, 0xef, 0xd7, 0xee, 0xb4, 0x18, 0x6b, 0x85,
		0x64, 0x4f, 0xb1, 0x9c, 0x26, 0xcd, 0xbd, 0x20, 0xe1, 0x58, 0x52, 0x16, 0x65, 0x42, 0xb5, 0x6d,
		0xd3, 0x01, 0x3e, 0xeb, 0x74, 0x06, 0x1c, 0x3b, 0x26, 0x8e, 0x36, 0x15, 0x92, 0xf1, 0xbe, 0x66,
		0xd9, 0x32, 0xb1, 0x9c, 0x27, 0x64, 0xc0, 0xe0, 0x98, 0x18, 0x24, 0x16, 0x67, 0x21, 0x15, 0xb2,
		0x88, 0x67, 0xd4, 0x44, 0xe7, 0xdf, 0x0b, 0xb0, 0x79, 0x2c, 0x31, 0x97, 0x2f, 0x34, 0xfd, 0xb3,
		0x97, 0xc4, 0x4f, 0x52, 0x73, 0x5c, 0x72, 0x9e, 0x10, 0x21, 0xd1, 0x1a, 0xcc, 0x07, 0xac, 0x83,
		0x69, 0x64, 0x5b, 0xdb, 0xd6, 0xee, 0xa2, 0xab, 0x57, 0x68, 0x0b, 0x2a, 0x39, 0x96, 0x47, 0x03,
		0x7b, 0x46, 0x6d, 0x42, 0x4e, 0x6a, 0x04, 0xe8, 0x29, 0x2c, 0x0d, 0x18, 0x64, 0x3f, 0x26, 0x76,
		0x69, 0xdb, 0xda, 0xad, 0xec, 0xef, 0xd4, 0x0d, 0x5e, 0xad, 0xe7, 0xc7, 0x9f, 0xf4, 0x63, 0xe2,
		0x56, 0x7b, 0x43, 0x2b, 0xf4, 0x09, 0x2c, 0xa6, 0x86, 0x79, 0xa9, 0x65, 0xf6, 0xac, 0xc2, 0xd8,
		0x34, 0x62, 0x9c, 0x60, 0x71, 0xf6, 0x05, 0x15, 0xd2, 0x2d, 0x4b, 0xfd, 0x0b, 0xed, 0xc3, 0x1c,
		0x8d, 0xe2, 0x44, 0xda, 0x73, 0x4a, 0xee, 0x5d, 0xa3, 0xdc, 0x11, 0xee, 0x87, 0x0c, 0x07, 0x6e,
		0xc6, 0x8a, 0x30, 0x6c, 0x93, 0xdc, 0x09, 0x9e, 0x48, 0x7d, 0xe3, 0x49, 0xe6, 0xf9, 0x21, 0x13,
		0xc4, 0x93, 0xb4, 0x43, 0x58, 0x22, 0xed, 0x79, 0x05, 0x77, 0xbb, 0x9e, 0xe5, 0x42, 0x3d, 0xcf,
		0x85, 0xfa, 0x13, 0x9d, 0x0b, 0xee, 0xbb, 0x03, 0x08, 0xe5, 0xdd, 0x13, 0x76, 0x98, 0xca, 0x9f,
		0x64, 0xe2, 0xe8, 0x05, 0x6c, 0x28, 0x93, 0x26, 0xa0, 0x2f, 0x4c, 0x43, 0x5f, 0x4f, 0xa5, 0x4d,
		0xc0, 0x35, 0x28, 0xd3, 0x80, 0x44, 0x92, 0xca, 0xbe, 0x5d, 0x56, 0x11, 0x19, 0xac, 0xd1, 0x26,
		0x00, 0xcf, 0x62, 0x9a, 0xc6, 0x6b, 0x51, 0xed, 0x2e, 0x6a, 0x4a, 0x23, 0x40, 0x3e, 0xd8, 0x43,
		0xf1, 0xf4, 0x38, 0x49, 0x04, 0xf1, 0x62, 0x16, 0x52, 0xbf, 0x6f, 0xc3, 0xb6, 0xb5, 0xbb, 0xbc,
		0x7f, 0xaf, 0x30, 0x72, 0x8d, 0xc0, 0x4d, 0x45, 0x8e, 0x94, 0x84, 0xbb, 0xda, 0x33, 0x91, 0xd1,
		0x21, 0x54, 0x39, 0x91, 0xbc, 0x9f, 0x03, 0x57, 0x94, 0xa5, 0xdb, 0x46, 0x60, 0x37, 0x65, 0xd4,
		0x70, 0x15, 0x7e, 0xb1, 0x40, 0x77, 0x61, 0xc9, 0xe7, 0x69, 0x6c, 0xfc, 0x36, 0x09, 0x92, 0x90,
		0xd8, 0x55, 0x65, 0x4b, 0x35, 0x25, 0x1e, 0x6b, 0x1a, 0xfa, 0x08, 0x66, 0x3b, 0xa4, 0xc3, 0xec,
		0x25, 0xed, 0x4b, 0xd3, 0x09, 0xcf, 0x49, 0x87, 0xb9, 0x8a, 0x0d, 0xb9, 0x70, 0x53, 0x10, 0xcc,
		0xfd, 0xb6, 0x87, 0xa5, 0xe4, 0xf4, 0x34, 0x91, 0x44, 0xd8, 0xcb, 0x4a, 0xf6, 0x7b, 0x46, 0xd9,
		0x63, 0xc5, 0x7d, 0x30, 0x60, 0x76, 0x57, 0xc4, 0x18, 0x05, 0x3d, 0x80, 0xf9, 0x36, 0xc1, 0x01,
		0xe1, 0xf6, 0x0d, 0x05, 0xb4, 0x61, 0x04, 0xfa, 0x5c, 0xb1, 0xb8, 0x9a, 0x15, 0x7d, 0x02, 0x95,
		0x80, 0x84, 0xb8, 0x9f, 0xe5, 0x86, 0xbd, 0x32, 0x2d, 0x15, 0x40, 0x71, 0xab, 0x5c, 0x70, 0x1e,
		0xc1, 0x9d, 0x49, 0x77, 0x59, 0xc4, 0x2c, 0x12, 0x04, 0xad, 0xc2, 0x3c, 0x4f, 0xa2, 0x34, 0xfe,
		0xd9, 0x65, 0x9e, 0xe3, 0x49, 0xd4, 0x08, 0x9c, 0x7f, 0xcd, 0xc0, 0x9d, 0x63, 0xda, 0x8a, 0x70,
		0x78, 0xed, 0x32, 0xf0, 0x2b, 0x40, 0x83, 0xb4, 0x19, 0xe4, 0xbc, 0xaa, 0x06, 0x95, 0xfd, 0xf7,
		0x0b, 0x13, 0xe6, 0xe2, 0x88, 0x9b, 0xbd, 0x71, 0xd2, 0x48, 0x22, 0x97, 0x0a, 0x13, 0x79, 0x76,
		0x3c, 0x91, 0xb7, 0xa0, 0x22, 0x94, 0x2d, 0x5e, 0x84, 0x3b, 0x44, 0xdd, 0xfc, 0x45, 0x17, 0x32,
		0xd2, 0x97, 0xb8, 0x43, 0xd0, 0xa7, 0x50, 0xd5, 0x0c, 0x59, 0x6d, 0x98, 0xbf, 0x42, 0x6d, 0xd0,
		0x90, 0x0d, 0x55, 0x21, 0x6c, 0x58, 0xf0, 0x59, 0x24, 0x39, 0x0b, 0xd5, 0x55, 0xad, 0xba, 0xf9,
		0xd2, 0xd9, 0x81, 0xad, 0x89, 0x7e, 0xcc, 0x42, 0xe0, 0xbc, 0xb2, 0xe0, 0xfb, 0x9a, 0x87, 0xca,
		0x76, 0x71, 0xed, 0x7d, 0x01, 0x4b, 0x59, 0x89, 0xd0, 0xd6, 0x29, 0xdf, 0x57, 0xf6, 0xf7, 0xcd,
		0x19, 0x59, 0x04, 0xe5, 0x56, 0x15, 0x50, 0x0e, 0x3c, 0xe6, 0xa3, 0x99, 0xa9, 0x3e, 0x2a, 0xfd,
		0x1f, 0x3e, 0x9a, 0x1d, 0xf5, 0xd1, 0x01, 0xec, 0x4e, 0xb7, 0xbf, 0x38, 0x5f, 0xff, 0x31, 0x03,
		0x9b, 0x2e, 0x11, 0x44, 0xbe, 0x2d, 0xe9, 0xba, 0x06, 0xf3, 0x9c, 0x60, 0xc1, 0x22, 0x9d, 0xac,
		0x7a, 0x85, 0x1e, 0x81, 0x1d, 0x10, 0x9f, 0x8a, 0xf4, 0x29, 0x69, 0xd2, 0x88, 0x8a, 0xb6, 0x47,
		0xba, 0x24, 0x1a, 0x24, 0x6e, 0xc9, 0x5d, 0xcd, 0xf7, 0x9f, 0xaa, 0xed, 0xcf, 0xd2, 0xdd, 0x46,
		0x30, 0x96, 0xe3, 0x73, 0xe3, 0x39, 0x5e, 0x87, 0x5b, 0xe2, 0x8c, 0xc6, 0x9e, 0x8e, 0x11, 0x27,
		0x38, 0x8e, 0xc3, 0xbe, 0xca, 0xe4, 0xb2, 0x7b, 0x33, 0xdd, 0xca, 0x5c, 0xec, 0x66, 0x1b, 0x69,
		0x65, 0x98, 0xe4, 0xaf, 0x62, 0x4f, 0xbf, 0xb2, 0x60, 0xe7, 0x84, 0xf0, 0x0e, 0x8d, 0xb0, 0x24,
		0x6f, 0xbb, 0xb7, 0x1f, 0xc2, 0x42, 0x40, 0x24, 0xa6, 0xa1, 0xd0, 0x7d, 0x42, 0x71, 0xbe, 0xe6,
		0xcc, 0x23, 0xc5, 0x66, 0x6e, 0xb4, 0xd8, 0x38, 0xef, 0x81, 0x53, 0x64, 0xbf, 0xbe, 0xd4, 0x7f,
		0xb1, 0x60, 0xfb, 0x09, 0x11, 0x3e, 0xa7, 0xa7, 0x6f, 0x8b, 0x97, 0x9c, 0x57, 0x25, 0xd8, 0x29,
		0xd0, 0x49, 0xc7, 0x3d, 0x84, 0xf5, 0x8b, 0x6e, 0xc7, 0x67, 0x51, 0x93, 0xb6, 0xf4, 0xd3, 0xa2,
		0x8b, 0xcd, 0x83, 0xab, 0x69, 0x70, 0x38, 0x2c, 0xea, 0xae, 0x11, 0x23, 0x1d, 0x9d, 0xc2, 0xfa,
		0x65, 0x53, 0x3d, 0x1a, 0x35, 0x99, 0xb6, 0xf7, 0xde, 0xd5, 0x4e, 0x6b, 0x44, 0x4d, 0x76, 0xd1,
		0x63, 0x8c, 0x90, 0xd1, 0x0b, 0x40, 0x31, 0x89, 0x02, 0x1a, 0xb5, 0x3c, 0xec, 0x4b, 0xda, 0xa5,
		0x92, 0x12, 0x61, 0x97, 0xb6, 0x4b, 0xbb, 0x95, 0xfd, 0x5d, 0x73, 0x42, 0x64, 0xec, 0x07, 0x19,
		0x77, 0x5f, 0x81, 0xdf, 0x8c, 0x47, 0x88, 0x94, 0x08, 0xf4, 0x6b, 0x58, 0xc9, 0x81, 0xfd, 0x36,
		0x0d, 0x03, 0x4e, 0x22, 0x7b, 0x56, 0xc1, 0xd6, 0x8b, 0x60, 0x0f, 0x53, 0xde, 0x51, 0xcd, 0x6f,
		0xc4, 0x43, 0x5b, 0x9c, 0x44, 0xe8, 0xf8, 0x02, 0x3a, 0xaf, 0x07, 0xba, 0x65, 0x2d, 0xd4, 0xf8,
		0x89, 0xe6, 0x1d, 0x01, 0xcd, 0x89, 0xce, 0x37, 0x25, 0x78, 0xe7, 0x97, 0xe9, 0xcc, 0x90, 0xbb,
		0xef, 0x0d, 0x5d, 0xd7, 0x8f, 0x61, 0x4e, 0x8d, 0x2e, 0xfa, 0x11, 0x71, 0x0a, 0x91, 0x94, 0xc2,
		0x6e, 0x26, 0x80, 0x3c, 0x58, 0x53, 0x3f, 0x3c, 0x4e, 0x7e, 0x4b, 0x7c, 0x99, 0xe6, 0x67, 0x40,
		0x95, 0x52, 0xb3, 0xaa, 0x23, 0xfd, 0x81, 0x11, 0x2a, 0x83, 0x50, 0x12, 0x87, 0xb9, 0x80, 0xfb,
		0xce, 0xb9, 0x81, 0x9a, 0xe6, 0x63, 0x76, 0x80, 0xcf, 0x22, 0x41, 0x85, 0x24, 0x91, 0xdf, 0xf7,
		0x42, 0xd2, 0x25, 0xa1, 0x72, 0xff, 0xa4, 0x9e, 0x57, 0x9d, 0x70, 0x78, 0x21, 0xf2, 0x45, 0x2a,
		0xe1, 0xae, 0x9e, 0x9b, 0xc8, 0xce, 0xdf, 0x2c, 0x58, 0x1d, 0x0b, 0x83, 0xbe, 0x7b, 0x9f, 0x42,
		0x35, 0x37, 0x4f, 0x24, 0x61, 0xfe, 0xba, 0x4f, 0x79, 0x64, 0xb5, 0x1d, 0xa9, 0x00, 0x6a, 0xc0,
		0xf2, 0xb0, 0x7f, 0x48, 0xa0, 0x83, 0xe5, 0x4c, 0xf3, 0x0b, 0x09, 0xdc, 0xa5, 0xf3, 0xe1, 0xa5,
		0xf3, 0x5f, 0x0b, 0xd6, 0xf3, 0x6a, 0x31, 0x18, 0xa4, 0xa6, 0xe4, 0xcb, 0xc8, 0x64, 0x36, 0x73,
		0xbd, 0xc9, 0xec, 0x19, 0x2c, 0x0f, 0x64, 0x2f, 0xc6, 0xc3, 0xe5, 0x09, 0xe3, 0x61, 0x0e, 0x90,
		0x8d, 0x87, 0x72, 0x68, 0x95, 0x3e, 0xb1, 0x34, 0xf2, 0xc3, 0x24, 0x20, 0xde, 0x05, 0xa0, 0x90,
		0x58, 0x26, 0xd9, 0x2b, 0x50, 0x76, 0x57, 0xf5, 0x7e, 0x0e, 0x72, 0xac, 0x36, 0x9d, 0xbf, 0x5b,
		0x60, 0x5f, 0xb6, 0x58, 0x87, 0xe6, 0x27, 0xb0, 0x10, 0xb3, 0x30, 0x24, 0x5c, 0xd8, 0x96, 0xba,
		0xe2, 0x5b, 0xe6, 0xa8, 0x28, 0x1e, 0x75, 0xfd, 0x72, 0x7e, 0xf4, 0x1c, 0x56, 0x2e, 0x29, 0x92,
		0x39, 0xe7, 0x6e, 0xa1, 0x6d, 0x99, 0x5a, 0xee, 0xb2, 0x1c, 0x55, 0x53, 0xc0, 0xa6, 0xb2, 0x55,
		0x53, 0x8f, 0x30, 0x97, 0x2a, 0x77, 0xc5, 0x77, 0x18, 0x1d, 0xe7, 0x0f, 0x33, 0x70, 0x67, 0xd2,
		0xa9, 0xda, 0x43, 0xe7, 0xb0, 0xa9, 0xcb, 0x6b, 0x7f, 0xc8, 0xf1, 0xf1, 0x80, 0x51, 0xfb, 0xad,
		0x5e, 0x78, 0xe4, 0x00, 0xf7, 0x39, 0x91, 0x38, 0xc0, 0x12, 0xbb, 0xb5, 0x1c, 0xf4, 0xf2, 0xd1,
		0xe9, 0x91, 0x83, 0x6e, 0xca, 0x78, 0xe4, 0xcc, 0xeb, 0x1d, 0x99, 0x83, 0x5e, 0x3e, 0xd2, 0xf9,
		0xd6, 0x82, 0x9a, 0xea, 0x9c, 0x8e, 0x25, 0xf5, 0xcf, 0xfa, 0x57, 0xbd, 0x19, 0xdf, 0xd1, 0x93,
		0xbe, 0x09, 0x1b, 0x46, 0x65, 0x74, 0x17, 0xb2, 0x0e, 0xab, 0xcf, 0x88, 0x3c, 0x0c, 0x13, 0x21,
		0x75, 0x52, 0x66, 0x6a, 0x3a, 0xbf, 0xb7, 0x60, 0x6d, 0x7c, 0x47, 0x87, 0xb1, 0x0d, 0xb7, 0x45,
		0x12, 0xc7, 0x8c, 0x4b, 0x12, 0x78, 0x7e, 0x48, 0xd3, 0xe6, 0xb4, 0x4b, 0xb8, 0xd0, 0x21, 0x4c,
		0x15, 0xfe, 0xd0, 0x3c, 0x6e, 0xe4, 0x52, 0x87, 0x4a, 0xe8, 0x2b, 0x2d, 0xe3, 0xae, 0x0b, 0xf3,
		0x86, 0xf3, 0x6d, 0x09, 0x9c, 0x67, 0x86, 0x16, 0xf4, 0xf3, 0xec, 0xcb, 0xd7, 0x1b, 0x7a, 0x9c,
		0x36, 0x60, 0x31, 0xc6, 0x2d, 0xe2, 0x09, 0xfa, 0x75, 0x56, 0x82, 0xe6, 0xdc, 0x72, 0x4a, 0x38,
		0xa6, 0x5f, 0x13, 0xf4, 0x3e, 0xdc, 0x88, 0xc8, 0xcb, 0x34, 0xc5, 0x5a, 0xc4, 0x93, 0xec, 0x8c,
		0x44, 0x7a, 0x98, 0x59, 0x4a, 0xc9, 0x47, 0xb8, 0x45, 0x4e, 0x52, 0x22, 0xfa, 0x00, 0x50, 0x0f,
		0x53, 0xe9, 0x35, 0x19, 0xf7, 0x22, 0xd2, 0xcb, 0x7a, 0x7c, 0xf5, 0x82, 0x94, 0xdd, 0x1b, 0xe9,
		0xce, 0x53, 0xc6, 0xbf, 0x24, 0x3d, 0xd5, 0xdc, 0x23, 0x0f, 0x6e, 0xeb, 0x8f, 0x7d, 0x7a, 0x16,
		0x68, 0xd2, 0x50, 0x12, 0x9e, 0x15, 0xc1, 0x79, 0x55, 0x04, 0xdf, 0x33, 0xda, 0xa3, 0xc4, 0x9f,
		0x2a, 0x66, 0x55, 0x07, 0xd7, 0x34, 0xcc, 0x18, 0x1d, 0xdd, 0x85, 0x25, 0x35, 0x1c, 0x60, 0xee,
		0xb7, 0x69, 0x17, 0x67, 0x43, 0x6a, 0xd9, 0xad, 0xa6, 0xc4, 0x03, 0x4d, 0x73, 0xfe, 0x63, 0xc1,
		0xdd, 0xc2, 0x68, 0xe8, 0xfc, 0x78, 0x08, 0x0b, 0xfa, 0x98, 0xc2, 0xe7, 0x29, 0x17, 0xcb, 0x99,
		0xd1, 0x2f, 0xa0, 0xc2, 0x71, 0xcf, 0xcb, 0x65, 0xb3, 0x9b, 0x69, 0xae, 0x3f, 0x4f, 0xb0, 0xc4,
		0x8f, 0x43, 0x76, 0xea, 0x02, 0xc7, 0x3d, 0x0d, 0x64, 0x72, 0x7d, 0xc9, 0xe4, 0xfa, 0x1a, 0x94,
		0x33, 0x3b, 0x49, 0xa0, 0xcb, 0xfd, 0x60, 0xbd, 0xff, 0xcf, 0x2a, 0x54, 0x72, 0x03, 0x0f, 0x8e,
		0x1a, 0xe8, 0x77, 0x16, 0xac, 0x99, 0x07, 0x4e, 0xf4, 0x1a, 0x23, 0x75, 0xed, 0xc1, 0xb5, 0x64,
		0xb4, 0x3f, 0xbf, 0xb1, 0x60, 0x7d, 0xc2, 0x27, 0x02, 0x34, 0x01, 0xb0, 0xf0, 0xc3, 0x4c, 0xed,
		0x47, 0xd7, 0x13, 0xd2, 0x6a, 0xfc, 0xd5, 0x82, 0xed, 0x69, 0x53, 0x38, 0xfa, 0x59, 0x11, 0xf4,
		0xb4, 0x8f, 0x17, 0xb5, 0x9f, 0xbf, 0xa6, 0xb4, 0xd6, 0x30, 0x0d, 0x96, 0x79, 0x66, 0x9d, 0x10,
		0xac, 0xc2, 0x0f, 0x02, 0x13, 0x82, 0x35, 0x65, 0x28, 0xfe, 0x93, 0x05, 0xb5, 0xc9, 0xd3, 0x1f,
		0x7a, 0x68, 0x7e, 0x68, 0xa6, 0x8d, 0xcb, 0xb5, 0x47, 0xd7, 0x96, 0xd3, 0xfa, 0xfc, 0xd1, 0x82,
		0xdb, 0x13, 0x47, 0x3a, 0xf4, 0x63, 0xf3, 0xed, 0x9a, 0x32, 0x96, 0xd6, 0x1e, 0x5e, 0x57, 0x4c,
		0x2b, 0xd3, 0x84, 0xa5, 0x91, 0xb6, 0x16, 0x15, 0x74, 0xe3, 0x63, 0x13, 0x48, 0xed, 0xde, 0x55,
		0x58, 0xf5, 0x39, 0x0c, 0x56, 0xc6, 0xdb, 0x34, 0xf4, 0x61, 0xa1, 0xce, 0x63, 0xaf, 0x74, 0xed,
		0xa3, 0x2b, 0x72, 0x0f, 0x65, 0x9e, 0xb9, 0xf9, 0x99, 0x90, 0x79, 0x85, 0xfd, 0xd9, 0x84, 0xcc,
		0x9b, 0xd2, 0x5d, 0xbd, 0x84, 0x5b, 0x86, 0x97, 0x1e, 0xed, 0x4d, 0xce, 0x62, 0x63, 0x83, 0x52,
		0xfb, 0xe1, 0xd5, 0x05, 0xf4, 0xc9, 0x14, 0x96, 0x47, 0x5b, 0x05, 0x64, 0x0e, 0x96, 0xb1, 0xd3,
		0xa8, 0x7d, 0x70, 0x25, 0x5e, 0x7d, 0xd4, 0x9f, 0x2d, 0xd8, 0x28, 0x78, 0x83, 0xd0, 0xa3, 0x49,
		0x60, 0x53, 0x7a, 0x88, 0xda, 0xc7, 0xd7, 0x17, 0xcc, 0x54, 0x7a, 0xfc, 0x15, 0xac, 0xfb, 0xac,
		0x63, 0x12, 0x7f, 0x5c, 0x3e, 0x88, 0xe9, 0x11, 0x67, 0x92, 0x1d, 0x59, 0xbf, 0xd9, 0x6b, 0x51,
		0xd9, 0x4e, 0x4e, 0xeb, 0x3e, 0xeb, 0xec, 0x8d, 0xfc, 0xcb, 0x56, 0x6f, 0x91, 0x28, 0xfb, 0x6b,
		0x50, 0xff, 0xe1, 0xf6, 0x53, 0x1c, 0xd3, 0xee, 0xfd, 0xd3, 0x79, 0x45, 0x7b, 0xf0, 0xbf, 0x00,
		0x00, 0x00, 0xff, 0xff, 0xd3, 0xc7, 0x82, 0x84, 0x7f, 0x1c, 0x00, 0x00,
	},
	// uber/cadence/shared/v1/history.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x8c, 0x92, 0xc1, 0x4a, 0xeb, 0x40,
		0x18, 0x85, 0x49, 0x4b, 0x6f, 0xef, 0x9d, 0xf6, 0x56, 0x19, 0x50, 0x6a, 0x41, 0x68, 0x83, 0x48,
		0x71, 0x31, 0x21, 0x15, 0x71, 0xe1, 0x46, 0xa5, 0x8a, 0x71, 0x19, 0x8a, 0x0b, 0x37, 0x21, 0xc9,
		0xfc, 0x36, 0x83, 0x76, 0xa6, 0xcc, 0x4c, 0x82, 0x3e, 0x8b, 0x8f, 0xe0, 0x4b, 0x4a, 0x26, 0xd3,
		0x96, 0x58, 0x17, 0xdd, 0xe5, 0xcc, 0x9c, 0xf3, 0xfd, 0x27, 0xc9, 0x8f, 0x4e, 0xf2, 0x04, 0xa4,
		0x97, 0xc6, 0x14, 0x78, 0x0a, 0x9e, 0xca, 0x62, 0x09, 0xd4, 0x2b, 0x7c, 0x2f, 0x63, 0x4a, 0x0b,
		0xf9, 0x41, 0x96, 0x52, 0x68, 0x81, 0x0f, 0x4b, 0x17, 0xb1, 0x2e, 0x52, 0xb9, 0x48, 0xe1, 0x0f,
		0x46, 0xb5, 0x74, 0xbc, 0x64, 0x5b, 0x51, 0xf7, 0xcb, 0x41, 0x07, 0x33, 0x19, 0x73, 0xc5, 0x80,
		0xeb, 0x29, 0xa4, 0x4c, 0x31, 0xc1, 0x03, 0xfe, 0x22, 0xf0, 0x23, 0xda, 0x53, 0x69, 0x06, 0x34,
		0x7f, 0x03, 0x1a, 0x41, 0x01, 0x5c, 0xf7, 0x9d, 0xa1, 0x33, 0xee, 0x4c, 0x46, 0xa4, 0x36, 0x2e,
		0x5e, 0x32, 0x52, 0xf8, 0xe4, 0xa1, 0xc2, 0xde, 0x95, 0xc6, 0xb0, 0xb7, 0x4e, 0x1a, 0x8d, 0xef,
		0xd1, 0x7f, 0xa5, 0x63, 0xa9, 0xd7, 0xa4, 0xc6, 0xae, 0xa4, 0xae, 0xcd, 0x19, 0xe5, 0x06, 0x08,
		0x3f, 0x81, 0x2c, 0x2b, 0x5a, 0x53, 0xa0, 0x61, 0x81, 0x8f, 0xd0, 0x5f, 0x43, 0x8d, 0x18, 0x35,
		0x15, 0x9b, 0x61, 0xdb, 0xe8, 0x80, 0xe2, 0x3e, 0x6a, 0x17, 0x55, 0xc0, 0x8c, 0x6c, 0x86, 0x2b,
		0xe9, 0xe6, 0xa8, 0x57, 0x47, 0xe1, 0x11, 0xea, 0x26, 0x32, 0xe6, 0x69, 0x16, 0x69, 0xf1, 0x0a,
		0xdc, 0xa0, 0xba, 0x61, 0xa7, 0x3a, 0x9b, 0x95, 0x47, 0xf8, 0x1a, 0xb5, 0x98, 0x86, 0x85, 0xea,
		0x37, 0x86, 0xcd, 0x71, 0x67, 0x72, 0x46, 0x7e, 0xff, 0xf0, 0x64, 0xbb, 0x64, 0x58, 0x05, 0xdd,
		0x4f, 0x07, 0xed, 0xd7, 0x6e, 0x19, 0x28, 0x7c, 0x83, 0x8e, 0xd3, 0x5c, 0xca, 0xf2, 0x15, 0x6c,
		0xbd, 0xc8, 0xfe, 0xa5, 0x88, 0x71, 0x0a, 0xef, 0xa6, 0x4a, 0x2b, 0x1c, 0x58, 0xd3, 0x0f, 0x7a,
		0xe9, 0xc0, 0x53, 0xf4, 0x2f, 0x5b, 0xf1, 0x6c, 0xbb, 0xd3, 0xdd, 0xda, 0x85, 0x9b, 0xe0, 0xed,
		0xe5, 0xf3, 0xc5, 0x9c, 0xe9, 0x2c, 0x4f, 0x48, 0x2a, 0x16, 0x5e, 0x6d, 0x7b, 0xc8, 0x1c, 0xb8,
		0x67, 0x76, 0x66, 0xb3, 0x86, 0x57, 0xd5, 0x53, 0xe1, 0x27, 0x7f, 0xcc, 0xcd, 0xf9, 0x77, 0x00,
		0x00, 0x00, 0xff, 0xff, 0x84, 0xb4, 0x3e, 0x6c, 0xb0, 0x02, 0x00, 0x00,
	},
	// uber/cadence/shared/v1/tasklist.proto
	[]byte{
		0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xe2, 0x52, 0x2d, 0x4d, 0x4a, 0x2d,
		0xd2, 0x4f, 0x4e, 0x4c, 0x49, 0xcd, 0x4b, 0x4e, 0xd5, 0x2f, 0xce, 0x48, 0x2c, 0x4a, 0x4d, 0xd1,
		0x2f, 0x33, 0xd4, 0x2f, 0x49, 0x2c, 0xce, 0xce, 0xc9, 0x2c, 0x2e, 0xd1, 0x2b, 0x28, 0xca, 0x2f,
		0xc9, 0x17, 0x12, 0x03, 0x29, 0xd3, 0x83, 0x2a, 0xd3, 0x83, 0x28, 0xd3, 0x2b, 0x33, 0xd4, 0x8a,
		0xe2, 0xe2, 0x0a, 0x49, 0x2c, 0xce, 0x0e, 0xce, 0x2f, 0x2d, 0x4a, 0x4e, 0x15, 0x12, 0xe7, 0x12,
		0x0e, 0x71, 0x0c, 0xf6, 0x8e, 0x0f, 0xf6, 0x0f, 0x0d, 0x72, 0x76, 0x8d, 0xf7, 0xf4, 0x0b, 0x73,
		0xf4, 0xf1, 0x74, 0x11, 0x60, 0x40, 0x97, 0xf0, 0xf0, 0x0c, 0x0e, 0xf1, 0x0f, 0x8a, 0x14, 0x60,
		0x14, 0x92, 0xe2, 0x12, 0x43, 0x96, 0x70, 0x71, 0x8a, 0x77, 0x72, 0x74, 0xf6, 0xf6, 0xf1, 0x77,
		0x17, 0x60, 0x72, 0x32, 0x8f, 0x32, 0x4d, 0xcf, 0x2c, 0xc9, 0x28, 0x4d, 0xd2, 0x4b, 0xce, 0xcf,
		0xd5, 0x47, 0x71, 0xa7, 0x5e, 0x7a, 0x6a, 0x9e, 0x3e, 0xd8, 0x65, 0x08, 0x27, 0x5b, 0x43, 0x58,
		0x65, 0x86, 0x49, 0x6c, 0x60, 0x19, 0x63, 0x40, 0x00, 0x00, 0x00, 0xff, 0xff, 0x1f, 0x73, 0x06,
		0x1c, 0xdc, 0x00, 0x00, 0x00,
	},
}

func init() {
	yarpc.RegisterClientBuilder(
		func(clientConfig transport.ClientConfig, structField reflect.StructField) MatchingAPIYARPCClient {
			return NewMatchingAPIYARPCClient(clientConfig, protobuf.ClientBuilderOptions(clientConfig, structField)...)
		},
	)
}
